﻿/*
DateInput控件
*/

(function($) {
	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				css: "date_input", // 控件的默认样式
				format: "yyyy-MM-dd", // 时间模式
				defaultValue: "1970-01-01", // 默认值
				lenient: true // 是否宽松
			}, options);

			return this.each(function() {
				this.style.imeMode = "disabled";
				$(this).addClass(settings.css);
				$(this).attr("disabled", !settings.enable);
				$(this).val(settings.defaultValue);
				$(this).bind("paste", function(event) { return false; });
				$(this).keydown(function(event) {
					var keyCode = event.which;

					if (keyCode == 9// Tab
						|| keyCode == 16// Shift
						|| keyCode == 17// Ctrl
						|| keyCode == 18// Alt
						|| keyCode == 35// End
						|| keyCode == 36// Home
						|| keyCode == 37// 向左
						|| keyCode == 39// 向右
						|| keyCode == 13// 回车
					) {
						return true;
					}

					var splitPoints = [];
					var index = -1;
					while((index = settings.format.indexOf("-", index + 1)) != -1) {
						splitPoints.push({ index: index, length: 1 });
					}

					var format = settings.format;
					var datePartFrags = [ false, false, false ];
					var formatFrags = [ "yyyy", "MM", "dd" ];
					var regFrags = [ "([1-9]\\d{3})", "(0[1-9]|1[0-2])", "(0[1-9]|[12]\\d|3[01])" ];
					for (var i = 0; i < formatFrags.length; i++) {
						var formatIndex = format.indexOf(formatFrags[i]);
						if (formatIndex != -1) {
							datePartFrags[i] = true;
							format = format.replace(formatFrags[i], regFrags[i]);
						}
					}
					var reg = new RegExp("^" + format + "$");

					var range = getSelectRange(this);
					var isSplitIndex = false;
					var isAfterSplitIndex = false;
					var index = 0;
					for (; index < splitPoints.length; index++) {
						if (splitPoints[index]["index"] == range.start) {
							isSplitIndex = true;
							break;
						}
						if (splitPoints[index]["index"] == range.start - splitPoints[index]["length"]) {
							isAfterSplitIndex = true;
							break;
						}
					}
					var $this = $(this);
					var oldText = $this.val();
					var startIndex = range.start + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var endIndex = range.start + 1 + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var letter = String.fromCharCode(keyCode);
					var posIndex = endIndex;

					if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
						letter = String.fromCharCode(keyCode - 48);
					}
					if (!event.shiftKey && keyCode == 189) {// 负号
						letter = oldText.charAt(startIndex);
						var splitIndex = oldText.indexOf("-", isSplitIndex ? startIndex - 1 : startIndex);
						if (splitIndex != -1) {
							posIndex = splitIndex + 1;
						} else {
							posIndex = oldText.length;
						}
					}
					if (keyCode == 38 || keyCode == 40) {// 向上或向下
						posIndex = isSplitIndex ? startIndex - 1 : startIndex;
						var splitBefore = oldText.lastIndexOf("-", isSplitIndex ? posIndex - 1 : posIndex);
						var splitAfter = oldText.indexOf("-", posIndex);
						if (splitAfter == -1) {
							splitAfter = oldText.length;
						}
						startIndex = splitBefore + 1;
						endIndex = splitAfter;
						
						var datePart = oldText.substring(startIndex, endIndex) * 1;
						datePart += keyCode == 38 ? 1 : -1;
						letter = datePart < 10 ? "0" + datePart : datePart;
					}
					if (keyCode == 8) {// 退格
						startIndex = range.start - 1 - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						endIndex = range.start - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 46) {// 删除
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 32) {// 空格
						letter = "0";
					}

					var changeType = "day";
					if (datePartFrags[0]) {
						var yearIndex = settings.format.indexOf("yyyy");
						if (posIndex >= yearIndex && posIndex <= yearIndex + 4) {
							changeType = "year";
						}
					}

					if (datePartFrags[1]) {
						var monthIndex = settings.format.indexOf("MM");
						if (posIndex >= monthIndex && posIndex <= monthIndex + 2) {
							changeType = "month";
						}
					}

					var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
					if (reg.test(newText)) {
						if (!validate(newText, reg, datePartFrags)) {
							if (settings.lenient) {
								if (keyCode == 38 || keyCode == 40 || keyCode == 8 || keyCode == 46) {
									newText = getCorrectOutBounds(newText, datePartFrags, keyCode == 8 || keyCode == 46, changeType);
								} else {
									return false;
								}
							} else {
								return false;
							}
						}
						$this.val(newText);
						setSelectRange(this, posIndex, posIndex);
					} else if (settings.lenient) {
						if (keyCode == 38 || keyCode == 40 || keyCode == 8 || keyCode == 46) {
							newText = getCorrectOutBounds(newText, datePartFrags, keyCode == 8 || keyCode == 46, changeType);
						} else {
							return false;
						}
						$this.val(newText);
						setSelectRange(this, posIndex, posIndex);
					}
					return false;
				});
			});

			// 获取选中区间
			function getSelectRange(textControl) {
				var range = {};
				if (document.selection) {
					if (textControl.tagName != undefined && textControl.tagName == "INPUT") {
						var selectionRange = document.selection.createRange();
						var textControlRange = textControl.createTextRange();
						range.text = selectionRange.text;
						var oldLength = range.text.length;
						selectionRange.setEndPoint("EndToStart", textControlRange);
						var newLength = selectionRange.text.length;
						range.start = newLength;
						range.end = newLength + oldLength;
					}
				} else {
					range.start = textControl.selectionStart;
					range.end = textControl.selectionEnd;
					range.text = textControl.value.substring(range.start, range.end);
				}
				return range;
			}

			// 设置选中区间
			function setSelectRange(textControl, startIndex, endIndex) {
				if (document.selection) {
					var range = textControl.createTextRange();
					range.collapse(true);
					range.moveStart("character", startIndex);
					range.moveEnd("character", endIndex - startIndex);
					range.select();
				} else {
					textControl.setSelectionRange(startIndex, endIndex);
					textControl.focus();
				}
			}
			
			// 验证日期是否合法
			function validate(dateStr, reg, datePartFrags) {
				var dateParts = reg.exec(dateStr);
				var year;
				var month;
				var day;
				if (!dateParts) {
					return false;
				}
				
				if (datePartFrags[0]) {
					year = dateParts[1] * 1;
				}
				if (datePartFrags[1]) {
					month = dateParts[datePartFrags[0] ? 2 : 1] * 1;
				}
				if (datePartFrags[2]) {
					day = dateParts[datePartFrags[0] && datePartFrags[1] ? 3 : datePartFrags[0] || datePartFrags[1] ? 2 : 1] * 1;
				}
				if (day) {
					return day <= getMaxDay(year, month);
				}
				return true;
			}

			// 获取修正后的越界值
			function getCorrectOutBounds(dateStr, datePartFrags, isClear, changeType) {
				var regFrags = [ "(\\d{4})", "(\\d{2})", "(\\d{2})" ];
				var formatParts = [];
				for (var i = 0; i < datePartFrags.length; i++) {
					if (datePartFrags[i]) {
						formatParts.push(regFrags[i]);
					}
				}

				var dateParts = new RegExp("^" + formatParts.join("-") + "$").exec(dateStr);
				var year;
				var month;
				var day;
				var result = dateStr;

				if (datePartFrags[0]) {
					year = dateParts[1] * 1;
				}
				if (datePartFrags[1]) {
					month = dateParts[datePartFrags[0] ? 2 : 1] * 1;
				}
				if (datePartFrags[2]) {
					day = dateParts[datePartFrags[0] && datePartFrags[1] ? 3 : datePartFrags[0] || datePartFrags[1] ? 2 : 1] * 1;
				}

				var maxDay = getMaxDay(year, month);
				if (changeType == "day") {
					if (day === 0) {
						if (isClear) {
							day = 1;
						} else {
							if (month) {
								month--;
							}
							day = getMaxDay(year, month);
						}
					} else if (day > maxDay) {
						if (month) {
							month++;
						}
						day -= maxDay;
					}

					if (month === 0) {
						if (isClear) {
							month = 1;
						} else {
							if (year) {
								year--;
							}
							month = 12;
							day = getMaxDay(year, month);
						}
					} else if (month > 12) {
						if (year) {
							year++;
						}
						month -= 12;
					}
				}

				if (changeType == "month") {
					maxDay = getMaxDay(year, month);
					if (month === 0) {
						if (isClear) {
							month = 1;
						} else {
							if (year) {
								year--;
							}
							month = 12;
						}
					} else if (month > 12) {
						if (year) {
							year++;
						}
						month -= 12;
					}
					if (day > maxDay) {
						day = maxDay;
					}
				}

				if (changeType == "year") {
					maxDay = getMaxDay(year, month);
					if (day > maxDay) {
						day = maxDay;
					}
				}
				return getFormatDate(year, month, day);
			}

			// 判断是否闰年
			function isLeapYear(year) {
				return (year % 100 != 0 && year % 4 == 0) || year % 400 == 0;
			}

			// 获取年月对应的最大天数
			function getMaxDay(year, month) {
				switch(month) {
					case 2:
						return !year || isLeapYear(year) ? 29 : 28;
					case 4:
					case 6:
					case 9:
					case 11:
						return 30;
					default:
						return 31;
				}
			}

			// 获取格式化的日期字符串
			function getFormatDate(year, month, day) {
				return $.map([ year, month, day ], function(element) {
					return element < 10 ? "0" + element : element;
				}).join("-");
			}
		}
	};

	$.fn.dateInput = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.dateInput");
		}
	};
})(jQuery);