﻿/*
DatePicker控件
*/

(function ($) {
	// 定义常量
	$.extend({
		// DatePicker显示模式
		DATEPICKER_DISPLAY_MODE_POPUP_SHARE: "popup_share",           // 共享弹出
		DATEPICKER_DISPLAY_MODE_POPUP: "popup",                       // 弹出
		DATEPICKER_DISPLAY_MODE_INNER: "inner",                       // 内嵌

		// DatePicker周常量
		DATEPICKER_SUN: 0,                                            // 周日
		DATEPICKER_MON: 1,                                            // 周一
		DATEPICKER_TUE: 2,                                            // 周二
		DATEPICKER_WED: 3,                                            // 周三
		DATEPICKER_THU: 4,                                            // 周四
		DATEPICKER_FRI: 5,                                            // 周五
		DATEPICKER_SAT: 6,                                            // 周六

		// DatePicker格式占位符常量（假定日期时间为：2014-02-07 09:03:06）
		DATEPICKER_YEAR: "yy",                                        // 短年格式占位符，例：14
		DATEPICKER_FULL_YEAR: "yyyy",                                 // 完整年格式占位符，例：2014
		DATEPICKER_MONTH: "M",                                        // 最短月格式占位符，例：2
		DATEPICKER_SHORT_MONTH: "MM",                                 // 短月格式占位符，例：02
		DATEPICKER_MEDIUM_MONTH: "MMM",                               // 中等月格式占位符，例：二、Feb、……
		DATEPICKER_FULL_MONTH: "MMMM",                                // 完整月格式占位符，例：三月、February、……
		DATEPICKER_DATE: "d",                                         // 短日期格式占位符，例：7
		DATEPICKER_FULL_DATE: "dd",                                   // 完整日期格式占位符，例：07
		DATEPICKER_HOUR: "H",                                         // 短小时格式占位符，例：9
		DATEPICKER_FULL_HOUR: "HH",                                   // 完整小时格式占位符，例：09
		DATEPICKER_HOUR_T: "h",                                       // 12小时制短小时格式占位符，例：9
		DATEPICKER_FULL_HOUR_T: "hh",                                 // 12小时制完整小时格式占位符，例：09
		DATEPICKER_MINUTE: "m",                                       // 短分钟格式占位符，例：3
		DATEPICKER_FULL_MINUTE: "mm",                                 // 完整分钟格式占位符，例：03
		DATEPICKER_SECOND: "s",                                       // 短秒格式占位符，例：6
		DATEPICKER_FULL_SECOND: "ss",                                 // 完整秒格式占位符，例：06
		DATEPICKER_DAY: "D",                                          // 短星期格式占位符，例：5
		DATEPICKER_MEDIUM_DAY: "DDD",                                 // 中等星期格式占位符，例：五、Fri、……
		DATEPICKER_FULL_DAY: "DDDD",                                  // 完整星期格式占位符，例：星期五、Friday、……
		DATEPICKER_AM_PM: "tt",                                       // 上午/下午格式占位符，例：上午、AM、……
		DATEPICKER_TIME: "T",                                         // 毫秒格式占位符，例：1391734986000
		DATEPICKER_DATE_OBJ: "dateObj",                               // 日期对象

		// DatePicker时间模式常量
		DATEPICKER_TIME_MODE_12_HOUR: "12-HOUR",                      // 12小时制
		DATEPICKER_TIME_MODE_24_HOUR: "24-HOUR",                      // 24小时制

		// DatePicker日期段类型常量
		DATEPICKER_YEARLY: "yearly",                                  // 每年的
		DATEPICKER_MONTHLY: "monthly",                                // 每月的
		DATEPICKER_WEEKLY: "weekly",                                  // 每周的
		DATEPICKER_ANY: "any"                                         // 任意
	});

	var methods = {
		/**
		 * 初始化
		 */
		init: function (options) {
			var settings = $.extend(true, {
				enable: true, // 是否启用
				readonly: false,// 是否只读（内嵌模式下有效）
				displayMode: $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE, // 显示模式
				lang: "zh-cn", // 语言类型
				firstDay: $.DATEPICKER_SUN, // 每周的起始天
				highlightWeekEnd: true, // 是否高亮显示周末
				defaultDate: null, // 默认日期时间
				dateFormat: "yyyy-MM-dd", // 日期格式
				showIcon: true, // 是否显示图标（显示模式为共享弹出或弹出时有效）
				lenient: true, // 是否宽松
				position: null, // 弹出的绝对位置（格式为{ left: number, top: number }，默认为null，一旦设置，则相对位置设置无效）
				offset: null, // 弹出的相对位置偏移量（格式为{ left: number, top: number }，默认为null，一旦设置，则根据视口自适应位置无效）
				showTime: false, // 是否显示时间
				timeMode: $.DATEPICKER_TIME_MODE_24_HOUR, // 时间模式
				/**
				 * 禁用的日期段，为对象数组，格式：
				 * 每年的：{ start: "MM-dd", end: "MM-dd", type: $.DATEPICKER_YEARLY }
				 * 每月的：{ start: d, end: d, type: $.DATEPICKER_MONTHLY }
				 * 每周的：{ start: D, end: D, type: $.DATEPICKER_WEEKLY }
				 * 任意：{ start: "yyyy-MM-dd", end: "yyyy-MM-dd", type: $.DATEPICKER_ANY }
				 *
				 * 注：若start或end其中一个省略，则表示之后或之前日期一直禁用，但起码必有一个出现
				 */
				disablePeriods: [
					{ end: "1899-12-31", type: $.DATEPICKER_ANY },
					{ start: "2100-01-01", type: $.DATEPICKER_ANY }
				],
				/**
				 * 特殊的日期段，为对象数组，格式：
				 * {
				 *		skin: {
				 *			dateCellCss: "", // 特殊日期单元格样式
				 *			dateCellHoverCss: "", // 特殊日期单元格悬停样式
				 *			dateCellSelectedCss: "", // 特殊日期单元格选中样式
				 *			dateCellDisabledCss: "", // 特殊日期单元格禁用样式
				 *			dateHighlightCss: "", // 特殊日期高亮样式（开启高亮时生效）
				 *			dateOtherMonthCss: "" // 特殊日期其他月份日期样式
				 *		},
				 *		periods: [] // 特殊日期段数组，格式：{ start: "yyyy-MM-dd", end: "yyyy-MM-dd" }
				 * }
				 *
				 * 注：skin里面不需要的样式可以省略，省略的会使用默认样式
				 */
				specialPeriods: [],
				showOther: {// 是否显示其他月份日期
					last: true, // 显示上个月日期
					next: true // 显示下个月日期
				},
				enableKeyboard: { // 是否开启键盘选择
					year: true, // 年份键盘选择（上下键）
					month: true, // 月份键盘选择（上下键）
					date: true, // 日期键盘选择（上下左右键）
					time: true // 时间键盘选择（上下键）
				},
				callback: {// 回调函数
					/**
					 * 选择日期事件前置函数
					 * datePicker: 日期控件
					 * oldValue: 旧的日期值（Date类型，只读）
					 * newValue: 新的日期值（Date类型，读写）
					 *
					 * 注：返回值为false则取消选择日期事件
					 */
					beforePicked: function(datePicker, oldValue, newValue) { },
					/**
					 * 选择日期事件函数
					 * datePicker: 日期控件
					 * oldValue: 旧的日期值（Date类型，只读）
					 * newValue: 新的日期值（Date类型，只读）
					 */
					onPicked: function(datePicker, oldValue, newValue) {  },
					/**
					 * 月份改变事件函数
					 * datePicker: 日期控件
					 * oldValue: 旧的月份值（Number类型）
					 * newValue: 新的月份值（Number类型）
					 */
					onMonthChanged: function(datePicker, oldValue, newValue) {  },
					/**
					 * 年份改变事件函数
					 * datePicker: 日期控件
					 * oldValue: 旧的月份值（Number类型）
					 * newValue: 新的月份值（Number类型）
					 */
					onYearChanged: function(datePicker, oldValue, newValue) {  },
                    /**
                     * 单击今天事件函数
                     * datePicker: 日期控件
                     */
                    onTodayClicked:function(datePicker) {  }
 				},
				skin: {// 皮肤默认样式
					containerCss: "date_picker_container", // 容器默认样式
					iconCss: "date_picker_icon", // 容器图标默认样式
					topCss: "date_picker_top", // 头部默认样式
					readonlyCss: "date_picker_readonly", // 只读模式默认样式
					yearInputCss: "date_picker_year_input", // 年份输入框默认样式
					yearContainerCss: "date_picker_year_container", // 年份容器默认样式
					yearRowCss: "date_picker_year_row", // 年份容器行默认样式
					yearCellCss: "date_picker_year_cell", // 年份容器单元格默认样式
					yearCellHoverCss: "date_picker_year_cell_hover", // 年份容器单元格悬停默认样式
					yearCellSelectedCss: "date_picker_year_cell_selected", // 年份容器单元格选中默认样式
					yearCellDisabledCss: "date_picker_year_cell_disabled", // 年份容器单元格禁用默认样式
					yearFunctionButtonContainerCss: "date_picker_year_function_button_container", // 年份功能按钮容器默认样式
					yearContainerPrevButtonCss: "date_picker_year_container_prev_button", // 上一组年份按钮容器默认样式
					yearContainerNextButtonCss: "date_picker_year_container_next_button", // 下一组年份按钮容器默认样式
					yearPopupCss: "date_picker_year_popup", // 年份弹出层默认样式
					monthInputCss: "date_picker_month_input", // 月份输入框默认样式
					monthContainerCss: "date_picker_month_container", // 月份容器默认样式
					monthRowCss: "date_picker_month_row", // 月份容器行默认样式
					monthCellCss: "date_picker_month_cell", // 月份容器单元格默认样式
					monthCellHoverCss: "date_picker_month_cell_hover", // 月份容器单元格悬停默认样式
					monthCellSelectedCss: "date_picker_month_cell_selected", // 月份容器单元格选中默认样式
					monthCellDisabledCss: "date_picker_month_cell_disabled", // 月份容器单元格禁用默认样式
					monthPopupCss: "date_picker_month_popup", // 月份弹出层默认样式
					yearPrevButtonCss: "date_picker_year_prev_button", // 上一年按钮默认样式
					yearNextButtonCss: "date_picker_year_next_button", // 下一年按钮默认样式
					monthPrevButtonCss: "date_picker_month_prev_button", // 上一月按钮默认样式
					monthNextButtonCss: "date_picker_month_next_button", // 下一月按钮默认样式
					centerCss: "date_picker_center", // 中部默认样式
					dateContainerCss: "date_picker_date_container", // 日期容器默认样式
					dateHeaderCss: "date_picker_date_header", // 日期容器头部默认样式
					dateHeaderCellCss: "date_picker_date_header_cell", // 日期容器头部单元格默认样式
					dateRowOddCss: "date_picker_date_row_odd", // 日期容器奇数行默认样式
					dateRowEvenCss: "date_picker_date_row_even", // 日期容器偶数行默认样式
					dateCellCss: "date_picker_date_cell", // 日期容器单元格默认样式
					dateCellHoverCss: "date_picker_date_cell_hover", // 日期容器单元格悬停默认样式
					dateCellSelectedCss: "date_picker_date_cell_selected", // 日期容器单元格选中默认样式
					dateCellDisabledCss: "date_picker_date_cell_disabled", // 日期容器单元格禁用默认样式
					dateHighlightCss: "date_picker_date_highlight", // 日期容器高亮默认样式（开启高亮时生效）
					dateOtherMonthCss: "date_picker_date_other_month", // 日期容器其他月份日期默认样式
					bottomCss: "date_picker_bottom", // 底部默认样式
					timeInput12HourCss: "date_picker_time_input_12_hour", // 12小时制时间输入框默认样式
					timeInput24HourCss: "date_picker_time_input_24_hour", // 24小时制时间输入框默认样式
					timeButtonContainerCss: "date_picker_time_button_container", // 时间按钮容器默认样式
					timeUpButtonCss: "date_picker_time_up_button", // 时间上调按钮默认样式
					timeDownButtonCss: "date_picker_time_down_button", // 时间下调按钮默认样式
					functionButtonContainerCss: "date_picker_function_button_container", // 功能按钮容器默认样式
					todayButtonCss: "date_picker_today_button", // 今天按钮默认样式
					okButtonCss: "date_picker_ok_button", // 确定按钮默认样式
					popupCss: "date_picker_popup", // 弹出层默认样式
					popupMaskCss: "date_picker_popup_mask" // 弹出层遮盖默认样式
				}
			}, options);

			var lang = {
				"en-us": {
					shortDay0: "Sun",
					shortDay1: "Mon",
					shortDay2: "Tue",
					shortDay3: "Wed",
					shortDay4: "Thu",
					shortDay5: "Fri",
					shortDay6: "Sat",
					fullDay0: "Sunday",
					fullDay1: "Monday",
					fullDay2: "Tuesday",
					fullDay3: "Wednesday",
					fullDay4: "Thursday",
					fullDay5: "Friday",
					fullDay6: "Saturday",
					shortMonth0: "Jan",
					shortMonth1: "Feb",
					shortMonth2: "Mar",
					shortMonth3: "Apr",
					shortMonth4: "May",
					shortMonth5: "Jun",
					shortMonth6: "Jul",
					shortMonth7: "Aug",
					shortMonth8: "Sep",
					shortMonth9: "Oct",
					shortMonth10: "Nov",
					shortMonth11: "Dec",
					fullMonth0: "January",
					fullMonth1: "February",
					fullMonth2: "March",
					fullMonth3: "April",
					fullMonth4: "May",
					fullMonth5: "June",
					fullMonth6: "July",
					fullMonth7: "August",
					fullMonth8: "September",
					fullMonth9: "October",
					fullMonth10: "November",
					fullMonth11: "December",
					am: "AM",
					pm: "PM",
					todayButtonText: "Today",
					okButtonText: "OK",
					disableDateTip: "The default date is disabled, please change the parameter!"
				},
				"zh-cn": {
					shortDay0: "日",
					shortDay1: "一",
					shortDay2: "二",
					shortDay3: "三",
					shortDay4: "四",
					shortDay5: "五",
					shortDay6: "六",
					fullDay0: "星期日",
					fullDay1: "星期一",
					fullDay2: "星期二",
					fullDay3: "星期三",
					fullDay4: "星期四",
					fullDay5: "星期五",
					fullDay6: "星期六",
					shortMonth0: "一",
					shortMonth1: "二",
					shortMonth2: "三",
					shortMonth3: "四",
					shortMonth4: "五",
					shortMonth5: "六",
					shortMonth6: "七",
					shortMonth7: "八",
					shortMonth8: "九",
					shortMonth9: "十",
					shortMonth10: "十一",
					shortMonth11: "十二",
					fullMonth0: "一月",
					fullMonth1: "二月",
					fullMonth2: "三月",
					fullMonth3: "四月",
					fullMonth4: "五月",
					fullMonth5: "六月",
					fullMonth6: "七月",
					fullMonth7: "八月",
					fullMonth8: "九月",
					fullMonth9: "十月",
					fullMonth10: "十一月",
					fullMonth11: "十二月",
					am: "AM",
					pm: "PM",
					todayButtonText: "今天",
					okButtonText: "确定",
					disableDateTip: "默认日期已被禁用，请修改参数！"
				},
                "es-mx": {
                    shortDay0: "Dom",
                    shortDay1: "Lun",
                    shortDay2: "Mar",
                    shortDay3: "Miér",
                    shortDay4: "Jue",
                    shortDay5: "Vier",
                    shortDay6: "Sáb",
                    fullDay0: "Domingo",
                    fullDay1: "Lunes",
                    fullDay2: "Martes",
                    fullDay3: "Miércoles",
                    fullDay4: "Jueves",
                    fullDay5: "Viernes",
                    fullDay6: "Sábado",
                    shortMonth0: "Ene",
                    shortMonth1: "Feb",
                    shortMonth2: "Mar",
                    shortMonth3: "Abr",
                    shortMonth4: "May",
                    shortMonth5: "Jun",
                    shortMonth6: "Jul",
                    shortMonth7: "Ago",
                    shortMonth8: "Sep",
                    shortMonth9: "Oct",
                    shortMonth10: "Nov",
                    shortMonth11: "Dic",
                    fullMonth0: "Enero",
                    fullMonth1: "Febrero",
                    fullMonth2: "Marzo",
                    fullMonth3: "Abril",
                    fullMonth4: "Mayo",
                    fullMonth5: "Junio",
                    fullMonth6: "Julio",
                    fullMonth7: "Agosto",
                    fullMonth8: "Septiembre",
                    fullMonth9: "Octubre",
                    fullMonth10: "Noviembre",
                    fullMonth11: "Diciembre",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Hoy",
                    okButtonText: "Seguro",
                    disableDateTip: "¡La fecha por defecto está deshabilitado, por favor cambiar el parámetro!"
                },
                "pt-br": {
                    shortDay0: "Dom",
                    shortDay1: "Seg",
                    shortDay2: "Ter",
                    shortDay3: "Qua",
                    shortDay4: "Qui",
                    shortDay5: "Sex",
                    shortDay6: "Sáb",
                    fullDay0: "Domingo",
                    fullDay1: "Segunda",
                    fullDay2: "Terça",
                    fullDay3: "Quarta",
                    fullDay4: "Quinta",
                    fullDay5: "Sexta",
                    fullDay6: "Sábado",
                    shortMonth0: "Jan",
                    shortMonth1: "Fev",
                    shortMonth2: "Mar",
                    shortMonth3: "Abr",
                    shortMonth4: "Mai",
                    shortMonth5: "Jun",
                    shortMonth6: "Jul",
                    shortMonth7: "Ago",
                    shortMonth8: "Set",
                    shortMonth9: "Out",
                    shortMonth10: "Nov",
                    shortMonth11: "Dez",
                    fullMonth0: "Janeiro",
                    fullMonth1: "Fevereiro",
                    fullMonth2: "Março",
                    fullMonth3: "Abril",
                    fullMonth4: "Maio",
                    fullMonth5: "Junho",
                    fullMonth6: "Julho",
                    fullMonth7: "Agosto",
                    fullMonth8: "Setembro",
                    fullMonth9: "Outubro",
                    fullMonth10: "Novembro",
                    fullMonth11: "Dezembro",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Hoje",
                    okButtonText: "Ok",
                    disableDateTip: "A data padrão está desabilitada, por favor altere os parâmetros!"
                },
                "pl-pl": {
                    shortDay0: "Sun",
                    shortDay1: "Mon",
                    shortDay2: "Tue",
                    shortDay3: "Wed",
                    shortDay4: "Thu",
                    shortDay5: "Fri",
                    shortDay6: "Sat",
                    fullDay0: "Sunday",
                    fullDay1: "Monday",
                    fullDay2: "Tuesday",
                    fullDay3: "Wednesday",
                    fullDay4: "Thursday",
                    fullDay5: "Friday",
                    fullDay6: "Saturday",
                    shortMonth0: "Jan",
                    shortMonth1: "Feb",
                    shortMonth2: "Mar",
                    shortMonth3: "Apr",
                    shortMonth4: "May",
                    shortMonth5: "Jun",
                    shortMonth6: "Jul",
                    shortMonth7: "Aug",
                    shortMonth8: "Sep",
                    shortMonth9: "Oct",
                    shortMonth10: "Nov",
                    shortMonth11: "Dec",
                    fullMonth0: "January",
                    fullMonth1: "February",
                    fullMonth2: "March",
                    fullMonth3: "April",
                    fullMonth4: "May",
                    fullMonth5: "June",
                    fullMonth6: "July",
                    fullMonth7: "August",
                    fullMonth8: "September",
                    fullMonth9: "October",
                    fullMonth10: "November",
                    fullMonth11: "December",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Today",
                    okButtonText: "OK",
                    disableDateTip: "The default date is disabled, please change the parameter!"
                },
                "it-it": {
                    shortDay0: "Dom",
                    shortDay1: "Lun",
                    shortDay2: "Mar",
                    shortDay3: "Mer",
                    shortDay4: "Gio",
                    shortDay5: "Ven",
                    shortDay6: "Sab",
                    fullDay0: "Domenica",
                    fullDay1: "Lunedì",
                    fullDay2: "Martedì",
                    fullDay3: "Mercoledì",
                    fullDay4: "Giovedì",
                    fullDay5: "Venerdì",
                    fullDay6: "Sabato",
                    shortMonth0: "Gen",
                    shortMonth1: "Feb",
                    shortMonth2: "Mar",
                    shortMonth3: "Apr",
                    shortMonth4: "Mag",
                    shortMonth5: "Giu",
                    shortMonth6: "Lug",
                    shortMonth7: "Ago",
                    shortMonth8: "Set",
                    shortMonth9: "Ott",
                    shortMonth10: "Nov",
                    shortMonth11: "Dic",
                    fullMonth0: "Gennaio",
                    fullMonth1: "Febbraio",
                    fullMonth2: "Marzo",
                    fullMonth3: "Aprile",
                    fullMonth4: "Maggio",
                    fullMonth5: "Giugno",
                    fullMonth6: "Luglio",
                    fullMonth7: "Agosto",
                    fullMonth8: "Settembre",
                    fullMonth9: "Ottobre",
                    fullMonth10: "Novembre",
                    fullMonth11: "Dicembre",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Oggi",
                    okButtonText: "OK",
                    disableDateTip: "La data di default è disabilitata, si prega di modificare il parametro!"
                },
                "de-de": {
                    shortDay0: "Sun",
                    shortDay1: "Mon",
                    shortDay2: "Tue",
                    shortDay3: "Wed",
                    shortDay4: "Thu",
                    shortDay5: "Fri",
                    shortDay6: "Sat",
                    fullDay0: "Sunday",
                    fullDay1: "Monday",
                    fullDay2: "Tuesday",
                    fullDay3: "Wednesday",
                    fullDay4: "Thursday",
                    fullDay5: "Friday",
                    fullDay6: "Saturday",
                    shortMonth0: "Jan",
                    shortMonth1: "Feb",
                    shortMonth2: "Mar",
                    shortMonth3: "Apr",
                    shortMonth4: "May",
                    shortMonth5: "Jun",
                    shortMonth6: "Jul",
                    shortMonth7: "Aug",
                    shortMonth8: "Sep",
                    shortMonth9: "Oct",
                    shortMonth10: "Nov",
                    shortMonth11: "Dec",
                    fullMonth0: "January",
                    fullMonth1: "February",
                    fullMonth2: "March",
                    fullMonth3: "April",
                    fullMonth4: "May",
                    fullMonth5: "June",
                    fullMonth6: "July",
                    fullMonth7: "August",
                    fullMonth8: "September",
                    fullMonth9: "October",
                    fullMonth10: "November",
                    fullMonth11: "December",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Today",
                    okButtonText: "OK",
                    disableDateTip: "The default date is disabled, please change the parameter!"
                },
                "zh-tw": {
                    shortDay0: "日",
                    shortDay1: "一",
                    shortDay2: "二",
                    shortDay3: "三",
                    shortDay4: "四",
                    shortDay5: "五",
                    shortDay6: "六",
                    fullDay0: "星期日",
                    fullDay1: "星期一",
                    fullDay2: "星期二",
                    fullDay3: "星期三",
                    fullDay4: "星期四",
                    fullDay5: "星期五",
                    fullDay6: "星期六",
                    shortMonth0: "一",
                    shortMonth1: "二",
                    shortMonth2: "三",
                    shortMonth3: "四",
                    shortMonth4: "五",
                    shortMonth5: "六",
                    shortMonth6: "七",
                    shortMonth7: "八",
                    shortMonth8: "九",
                    shortMonth9: "十",
                    shortMonth10: "十一",
                    shortMonth11: "十二",
                    fullMonth0: "一月",
                    fullMonth1: "二月",
                    fullMonth2: "三月",
                    fullMonth3: "四月",
                    fullMonth4: "五月",
                    fullMonth5: "六月",
                    fullMonth6: "七月",
                    fullMonth7: "八月",
                    fullMonth8: "九月",
                    fullMonth9: "十月",
                    fullMonth10: "十一月",
                    fullMonth11: "十二月",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "今天",
                    okButtonText: "確定",
                    disableDateTip: "默認日期已被禁用，請修改參數！"
                },
				"tr-tr": {
                    shortDay0: "Paz.",
                    shortDay1: "Pts.",
                    shortDay2: "Sal.",
                    shortDay3: "Çar.",
                    shortDay4: "Per.",
                    shortDay5: "Cum.",
                    shortDay6: "Cts.",
                    fullDay0: "Paz.",
                    fullDay1: "Pts.",
                    fullDay2: "Sal.",
                    fullDay3: "Çar.",
                    fullDay4: "Per.",
                    fullDay5: "Cum.",
                    fullDay6: "Cts.",
                    shortMonth0: "Oca",
                    shortMonth1: "Şub",
                    shortMonth2: "Mar",
                    shortMonth3: "Nis",
                    shortMonth4: "May",
                    shortMonth5: "Haz",
                    shortMonth6: "Tem",
                    shortMonth7: "Ağu",
                    shortMonth8: "Eyl",
                    shortMonth9: "Eki",
                    shortMonth10: "Kas",
                    shortMonth11: "Ara",
                    fullMonth0: "Ocak",
                    fullMonth1: "Şubat",
                    fullMonth2: "Mart",
                    fullMonth3: "Nisan",
                    fullMonth4: "Mayıs",
                    fullMonth5: "Haziran",
                    fullMonth6: "Temmuz",
                    fullMonth7: "Ağustos",
                    fullMonth8: "Eylül",
                    fullMonth9: "Ekim",
                    fullMonth10: "Kasım",
                    fullMonth11: "Aralık",
                    am: "AM",
                    pm: "PM",
                    todayButtonText: "Bugün",
                    okButtonText: "Ok",
                    disableDateTip: "Tarih Tipi"
                }
			};

			return this.each(function () {
				var $this = $(this);
				var baseId = settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE ? "base" : $this.attr("id");
				var value = $this.data("WidgetData") ? $this.data("WidgetData")["value"] : new Date();
				var tempTime = $this.data("WidgetData") ? new Date(value.getTime()) : new Date();
				if (!$this.data("WidgetData") && settings.defaultDate) {
					settings.defaultDate.setMilliseconds(0);
					value.setTime(settings.defaultDate.getTime());
					tempTime.setTime(settings.defaultDate.getTime());
				}

				if (isDisableDate(value.getFullYear(), value.getMonth(), value.getDate())) {
					throw new Error(lang[settings.lang]["disableDateTip"]);
				}

				$("#" + baseId + "_date_picker_container").remove();

				var container = $("<div id=\"" + baseId + "_date_picker_container\" class=\"" + settings.skin.containerCss + "\"></div>");

				$this.data("WidgetData", {
					target: $this,
					settings: settings,
					lang: lang,
					baseId: baseId,
					container: container,
					privateFunction: {
						updateControl: updateControl,
						isDisableDate: isDisableDate
					},
					value: value,
					tempTime: tempTime
				});

				if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER) {
					$this.append(container);
					render.call($this);
				} else if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP || settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) {
					if (settings.showIcon) {
						$this.addClass(settings.skin.iconCss);
					}
					if (settings.defaultDate) {
						$this.val($this.wbDatePicker("getWithFormat"));
					}
					$this.attr("disabled", !settings.enable);
					$this.unbind("click");
					$this.unbind("keydown");
					$this.click(function(event) {
						var offset = $this.offset();
						var height = $this.outerHeight();
						var left = settings.position ? settings.position.left
							: settings.offset ? offset.left + settings.offset.left : offset.left;
						var top = settings.position ? settings.position.top
							: settings.offset ? offset.top + height + settings.offset.top : offset.top + height;

						var divPopup = $("#" + baseId + "_date_picker_popup");
						var divPopupMask = $("#" + baseId + "_date_picker_popup_mask");
						if (divPopup.size() == 0) {
							divPopup = $("<div id=\"" + baseId + "_date_picker_popup\" class=\"" + settings.skin.popupCss + "\"></div>");
							divPopupMask = $("<iframe id=\"" + baseId + "_date_picker_popup_mask\" class=\"" + settings.skin.popupMaskCss + "\" scrolling=\"no\" frameborder=\"0\" src=\"about:blank\"></iframe>");
							var body = $("body");
							body.append(divPopup);
							body.append(divPopupMask);
							divPopup.hide();
							divPopupMask.hide();
						}
						divPopup.empty();
						divPopup.append(container);

						// 自动根据窗口大小调整位置
						if (!settings.position && !settings.offset) {
							top = getFitTop($this, divPopup, top);
						}

						if ($this.val() != "") {
							$this.data("innerCall", true);
							$this.wbDatePicker("setWithFormat", $this.val());
							tempTime.setTime(value.getTime());
							$this.data("originalValue", $this.wbDatePicker("get", $.DATEPICKER_DATE_OBJ));
							$this.data("innerCall", false);
						}

						render.call($this);
						showPopup.call($this, left, top);
					}).keydown(function(event) {
						if (!settings.enableKeyboard.date) {
							return;
						}

						var keyCode = event.which;
						var dateAdd = 0;
						switch (keyCode) {
							case 38:// 向上
								dateAdd = -7;
								break;
							case 40:// 向下
								dateAdd = 7;
								break;
							case 37:// 向左
								dateAdd = -1;
								break;
							case 39:// 向右
								dateAdd = 1;
								break;
							default:
								return;
						}

						var onlyStartDate = $this.data("onlyStartDate");
						var onlyEndDate = $this.data("onlyEndDate");
						do {
							value.setDate(value.getDate() + dateAdd);
							var year = value.getFullYear();
							var month = value.getMonth();
							var date = value.getDate();
							if ((keyCode == 39 || keyCode == 40) && onlyStartDate) {
								if (value >= onlyStartDate) {
									value.setFullYear(onlyStartDate.getFullYear());
									value.setMonth(onlyStartDate.getMonth());
									value.setDate(onlyStartDate.getDate() - 1);
									break;
								}
							}
							if ((keyCode == 37 || keyCode == 38) && onlyEndDate) {
								if (value <= onlyEndDate) {
									value.setFullYear(onlyEndDate.getFullYear());
									value.setMonth(onlyEndDate.getMonth());
									value.setDate(onlyEndDate.getDate() + 1);
									break;
								}
							}
						} while (isDisableDate(year, month, date));

						updateControl.call($this);

						initializing = false;

						return false;
					});
				}

				// 判断是否有“之前所有日期”和“之后所有日期”禁用情况，避免向前或向后改变日期死循环
				$.each(settings.disablePeriods, function(index, element) {
					var type = element.type;
					var start = element.start;
					var end = element.end;
					var dateParts;
					if (type == $.DATEPICKER_ANY) {
						if (!start) {
							var onlyEndDate = $this.data("onlyEndDate");
							dateParts = end.split("-");
							var endDate = new Date(dateParts[0] * 1, dateParts[1] * 1 - 1, dateParts[2] * 1);

							if (onlyEndDate) {
								if (endDate > onlyEndDate) {
									$this.data("onlyEndDate", endDate);
								}
							} else {
								$this.data("onlyEndDate", endDate);
							}
						}
						if (!end) {
							var onlyStartDate = $this.data("onlyStartDate");
							dateParts = start.split("-");
							var startDate = new Date(dateParts[0] * 1, dateParts[1] * 1 - 1, dateParts[2] * 1);

							if (onlyStartDate) {
								if (startDate < onlyStartDate) {
									$this.data("onlyStartDate", startDate);
								}
							} else {
								$this.data("onlyStartDate", startDate);
							}
						}
					}
				});
			});

			// 渲染控件
			function render() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var container = $this.data("WidgetData")["container"];
				var value = $this.data("WidgetData")["value"];
				container.html("");

				// 如果没有控件获得焦点，默认让“今天”按钮获得焦点，以便触发keydown
				container.click(function(event) {
					var targetId = $(event.target).attr("id");
					if (targetId != baseId + "_date_picker_year_input"
						&& targetId != baseId + "_date_picker_month_input"
						&& targetId != baseId + "_date_picker_time_input") {
						if (settings.enable) {
							$("#" + baseId + "_date_picker_today_button").focus();
						}
					}
				}).keydown(function(event) {
					// 禁用键盘或整个控件禁用或只读
					if (!settings.enableKeyboard.date || !settings.enable
						|| (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER
								&& settings.readonly)) {
						return;
					}

					var focusElement = $(this).find(":focus");
					if (focusElement.size() > 0) {
						var focusId = focusElement.attr("id");
						if (focusId != baseId + "_date_picker_year_input"
							&& focusId != baseId + "_date_picker_month_input"
							&& focusId != baseId + "_date_picker_time_input") {
							var keyCode = event.which;
							var dateAdd = 0;
							switch (keyCode) {
								case 38:// 向上
									dateAdd =  -7;
									break;
								case 40:// 向下
									dateAdd = 7;
									break;
								case 37:// 向左
									dateAdd = -1;
									break;
								case 39:// 向右
									dateAdd = 1;
									break;
								default:
									return;
							}

							var value = $this.data("WidgetData")["value"];
							var onlyStartDate = $this.data("onlyStartDate");
							var onlyEndDate = $this.data("onlyEndDate");
							do {
								value.setDate(value.getDate() + dateAdd);
								var year = value.getFullYear();
								var month = value.getMonth();
								var date = value.getDate();
								if ((keyCode == 39 || keyCode == 40) && onlyStartDate) {
									if (value >= onlyStartDate) {
										value.setFullYear(onlyStartDate.getFullYear());
										value.setMonth(onlyStartDate.getMonth());
										value.setDate(onlyStartDate.getDate() - 1);
										break;
									}
								}
								if ((keyCode == 37 || keyCode == 38) && onlyEndDate) {
									if (value <= onlyEndDate) {
										value.setFullYear(onlyEndDate.getFullYear());
										value.setMonth(onlyEndDate.getMonth());
										value.setDate(onlyEndDate.getDate() + 1);
										break;
									}
								}
							} while (isDisableDate(year, month, date));

							updateControl.call($this);

							return false;
						}
					}
				});

				// 初始化头部年月区
				var dpTop = $("<div id=\"" + baseId + "_date_picker_top\" class=\"" + settings.skin.topCss + "\"></div>");
				container.append(dpTop);
				renderTop.call(this);

				// 初始化中部日期区
				var dpCenter = $("<div id=\"" + baseId + "_date_picker_center\" class=\"" + settings.skin.centerCss + "\"></div>");
				container.append(dpCenter);
				setUnselectable(dpCenter);
				renderCenter.call(this);

				if (settings.displayMode != $.DATEPICKER_DISPLAY_MODE_INNER
					|| !settings.readonly) {
					// 初始化底部时间区
					var dpBottom = $("<div id=\"" + baseId + "_date_picker_bottom\" class=\"" + settings.skin.bottomCss + "\"></div>");
					container.append(dpBottom);
					renderBottom.call(this);
				}
			}

			// 渲染头部年月区
			function renderTop() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var container = $this.data("WidgetData")["container"];
				var dpTop = container.find("#" + baseId + "_date_picker_top");
				var tempTime = $this.data("WidgetData")["tempTime"];
				var year = tempTime.getFullYear(); // 设置的年份
				var month = tempTime.getMonth(); // 设置的月份

				if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER && settings.readonly) {
					var divYearMonth = $("<div id=\"" + baseId + "_date_picker_readonly\" class=\"" + settings.skin.readonlyCss + "\">" + $this.wbDatePicker("getWithFormat") + "</div>");
					dpTop.append(divYearMonth);
				} else {
					var btnYearPrev = $("<input id=\"" + baseId + "_date_picker_year_prev_button\" type=\"button\" class=\"" + settings.skin.yearPrevButtonCss + "\"/>");
					dpTop.append(btnYearPrev);
					btnYearPrev.click(function (event) {
						// 首先确定可用年份
						var oldYear = tempTime.getFullYear();
						var year = oldYear - 1;
						var onlyEndDate = $this.data("onlyEndDate");
						while (isDisableYear(year)) {
							if (onlyEndDate) {
								if (year < onlyEndDate.getFullYear()) {
									return;
								}
							}
							year--;
						}
						tempTime.setFullYear(year);
	
						// 若该年份的对应日期不可用，在本年内搜索最近的可用日期
						if (isDisableDate(year, tempTime.getMonth(), tempTime.getDate())) {
							var date = searchEnableDate(tempTime, new Date(year, 11, 31, 23, 59, 59), true);
							if (!date) {
								date = searchEnableDate(tempTime, new Date(year, 0, 1, 0, 0, 0));
							}
							tempTime.setMonth(date.getMonth());
							tempTime.setDate(date.getDate());
						}

						if ($.type(settings.callback.onYearChanged) == "function") {
							settings.callback.onYearChanged($this, oldYear, year);
						}

						updateControl.call($this);
					}).attr("disabled", !settings.enable);
	
					var btnMonthPrev = $("<input id=\"" + baseId + "_date_picker_month_prev_button\" type=\"button\" class=\"" + settings.skin.monthPrevButtonCss + "\"/>");
					dpTop.append(btnMonthPrev);
					btnMonthPrev.click(function (event) {
						// 首先确定可用月份
						var oldMonth = tempTime.getMonth();
						tempTime.setMonth(tempTime.getMonth() - 1);
						var onlyEndDate = $this.data("onlyEndDate");
						while (isDisableMonth(tempTime.getFullYear(), tempTime.getMonth())) {
							var year = tempTime.getFullYear();
							var month = tempTime.getMonth();
							if (onlyEndDate) {
								if (tempTime < onlyEndDate) {
									tempTime.setMonth(tempTime.getMonth() + 1);
									break;
								}
							}
							tempTime.setMonth(tempTime.getMonth() - 1);
						}
	
						// 若该月份的对应日期不可用，在本月内搜索最近的可用日期
						var year = tempTime.getFullYear();
						var month = tempTime.getMonth();
						if (isDisableDate(year, month, tempTime.getDate())) {
							var date = searchEnableDate(tempTime, new Date(year, month + 1, 1, 0, 0, 0), true);
							if (!date) {
								date = searchEnableDate(tempTime, new Date(year, month, 1, 0, 0, 0));
							}
							tempTime.setDate(date.getDate());
						}

						if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER) {
							$this.data("WidgetData")["value"].setTime(tempTime.getTime());
						}

						if ($.type(settings.callback.onMonthChanged) == "function") {
							settings.callback.onMonthChanged($this, oldMonth, tempTime.getMonth());
						}

						updateControl.call($this);
					}).attr("disabled", !settings.enable);

					var txtYear = $("<input id=\"" + baseId + "_date_picker_year_input\" type=\"text\" class=\"" + settings.skin.yearInputCss + "\" maxlength=\"4\" value=\"" + year + "\"/>");
					dpTop.append(txtYear);
					txtYear[0].style.imeMode = "disabled";
					txtYear.bind("paste", function(event) { return false; }).click(function(event) {
						$(this).data("oldYear", tempTime.getFullYear());
						$(this).data("oldMonth", tempTime.getMonth());
						showYearContainer.call($this);
					}).blur(function(event) {
						var oldYear = $(this).data("oldYear") * 1;
						var oldMonth = $(this).data("oldMonth") * 1;
						if ($(this).val() == "" || isDisableYear($(this).val() * 1)) {
							$(this).val(oldYear + "");
							txtMonth.val(oldMonth + 1 + "");
							return;
						}
	
						tempTime.setFullYear($(this).val() * 1);
	
						// 若该年份的对应日期不可用，在本年内搜索最近的可用日期
						var year = tempTime.getFullYear();
						if (isDisableDate(year, tempTime.getMonth(), tempTime.getDate())) {
							var firstDate = new Date(year, 0, 1, 0, 0, 0);
							var lastDate = new Date(year, 11, 31, 23, 59, 59);
							var date = searchEnableDate(tempTime, oldYear > year ? lastDate : firstDate, oldYear > year);
							if (!date) {
								date = searchEnableDate(tempTime, oldYear < year ? lastDate : firstDate, oldYear < year);
							}
							tempTime.setMonth(date.getMonth());
							tempTime.setDate(date.getDate());
						}
	
						setTimeout(function(event) {
							var divYearPopup = $("#" + baseId + "_date_picker_year_popup");
							if (!divYearPopup.data("selected")) {
								if ($.type(settings.callback.onYearChanged) == "function") {
									settings.callback.onYearChanged($this, oldYear, tempTime.getFullYear());
								}
								divYearPopup.hide();
								updateControl.call($this);
							}
							divYearPopup.data("selected", false);
						}, 200);
					}).keydown(function(event) {
						var keyCode = event.which;
	
						if (keyCode == 8// 退格
							|| keyCode == 9// Tab
							|| keyCode == 16// Shift
							|| keyCode == 17// Ctrl
							|| keyCode == 18// Alt
							|| keyCode == 35// End
							|| keyCode == 36// Home
							|| keyCode == 46// 删除
							|| keyCode == 37// 向左
							|| keyCode == 39// 向右
						) {
							return true;
						}
	
						var reg = /^[1-9]\d{0,3}$/;
						var range = getSelectRange(this);
						var oldText = $(this).val();
						var startIndex = range.start;
						var endIndex = range.end;
						var letter = String.fromCharCode(keyCode);
						var posIndex = range.start + letter.length;
	
						if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
							letter = String.fromCharCode(keyCode - 48);
						}
						if (settings.enableKeyboard.year && (keyCode == 38 || keyCode == 40)) {// 向上或向下
							var year = oldText * 1;
							var addYear = 0;
							if (keyCode == 38) {
								addYear = -1;
							}
							if (keyCode == 40) {
								addYear = 1;
							}
	
							// 确定可用年份
							year += addYear;
							var onlyStartDate = $this.data("onlyStartDate");
							var onlyEndDate = $this.data("onlyEndDate");
							while (isDisableYear(year)) {
								if (keyCode == 38 && onlyEndDate) {
									if (year < onlyEndDate.getFullYear()) {
										return;
									}
								}
								if (keyCode == 40 && onlyStartDate) {
									if (year > onlyStartDate.getFullYear()) {
										return;
									}
								}
								year += addYear;
							}
	
							// 重新渲染年份容器
							var tempTime = $this.data("WidgetData")["tempTime"];
							var tempYear = tempTime.getFullYear();
							tempTime.setFullYear(year);
	
							// 确定可用月份
							if (isDisableDate(year, month, 1)) {
								var firstDate = new Date(year, 0, 1, 0, 0, 0);
								var lastDate = new Date(year, 11, 31, 23, 59, 59);
								var date = searchEnableDate(tempTime, keyCode == 38 ? lastDate : firstDate, keyCode == 38);
								if (!date) {
									date = searchEnableDate(tempTime, keyCode == 40 ? lastDate : firstDate, keyCode == 40);
								}
								tempTime.setMonth(date.getMonth());
								txtMonth.val(date.getMonth() + 1);
							}
	
							$("#" + baseId + "_date_picker_year_popup").data("startYear", year - year % 10);
							renderYearContainer.call($this);
							tempTime.setFullYear(tempYear);
	
							letter = year + "";
							posIndex = startIndex;
							startIndex = 0;
							endIndex = Math.max(oldText.length, letter.length);
						}
	
						if (keyCode == 13) {// 回车
							$("#" + baseId + "_date_picker_today_button").focus();
							return false;
						}
	
						var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
						if (newText == "" || reg.test(newText)) {
							$(this).val(newText);
							setSelectRange(this, posIndex, posIndex);
						}
						return false;
					}).attr("disabled", !settings.enable);
	
					var txtMonth = $("<input id=\"" + baseId + "_date_picker_month_input\" type=\"text\" class=\"" + settings.skin.monthInputCss + "\" maxlength=\"2\" value=\"" + (month + 1) + "\"/>");
					dpTop.append(txtMonth);
					txtMonth[0].style.imeMode = "disabled";
					txtMonth.bind("paste", function(event) { return false; }).click(function(event) {
						$(this).data("oldYear", tempTime.getFullYear());
						$(this).data("oldMonth", tempTime.getMonth());
						var divMonthPopup = $("#" + baseId + "_date_picker_month_popup");
						if (divMonthPopup.size() == 0 || divMonthPopup.is(":hidden")) {
							showMonthContainer.call($this);
						}
					}).blur(function(event) {
						var oldYear = $(this).data("oldYear") * 1;
						var oldMonth = $(this).data("oldMonth") * 1;
						var year = txtYear.val() * 1;
						var month = $(this).val() * 1 - 1;
	
						if ($(this).val() == "" || isDisableMonth(year, month)) {
							txtYear.val(oldYear);
							$(this).val(oldMonth + 1 + "");
							return;
						}
	
						tempTime.setFullYear(year);
						tempTime.setMonth($(this).val() * 1 - 1);
	
						// 若该月份的对应日期不可用，在本月内搜索最近的可用日期
						if (isDisableDate(year, month, tempTime.getDate())) {
							var firstDate = new Date(year, month, 1, 0, 0, 0);
							var lastDate = new Date(year, month + 1, 1, 0, 0, 0);
							var date = searchEnableDate(tempTime, oldMonth > month ? lastDate : firstDate, oldMonth > month);
							if (!date) {
								date = searchEnableDate(tempTime, oldMonth < month ? lastDate : firstDate, oldMonth < month);
							}
							tempTime.setDate(date.getDate());
						}
	
						setTimeout(function(event) {
							var divMonthPopup = $("#" + baseId + "_date_picker_month_popup");
							if (!divMonthPopup.data("selected")) {
								if ($.type(settings.callback.onMonthChanged) == "function") {
									settings.callback.onMonthChanged($this, oldMonth, tempTime.getMonth());
								}

								divMonthPopup.hide();
								updateControl.call($this);
							}
							divMonthPopup.data("selected", false);
						}, 200);
						
					}).keydown(function(event) {
						var keyCode = event.which;
	
						if (keyCode == 9// Tab
							|| keyCode == 16// Shift
							|| keyCode == 17// Ctrl
							|| keyCode == 18// Alt
							|| keyCode == 35// End
							|| keyCode == 36// Home
							|| keyCode == 37// 向左
							|| keyCode == 39// 向右
						) {
							return true;
						}
	
						var reg = /^(1[0-2]|[1-9])$/;
						var range = getSelectRange(this);
						var oldText = $(this).val();
						var startIndex = range.start;
						var endIndex = range.end;
						var letter = String.fromCharCode(keyCode);
						var posIndex = range.start + letter.length;
	
						if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
							letter = String.fromCharCode(keyCode - 48);
						}
						if (settings.enableKeyboard.month && (keyCode == 38 || keyCode == 40)) {// 向上或向下
							var year = txtYear.val();
							var addMonth = 0;
							if (keyCode == 38) {
								addMonth = -1;
							}
							if (keyCode == 40) {
								addMonth = 1;
							}
	
							// 确定可用月份
							var tempTime = $this.data("WidgetData")["tempTime"];
							var onlyStartDate = $this.data("onlyStartDate");
							var onlyEndDate = $this.data("onlyEndDate");
							var tempDate = new Date(year, oldText * 1 - 1 + addMonth, tempTime.getDate());
							while (isDisableMonth(tempDate.getFullYear(), tempDate.getMonth())) {
								if (keyCode == 38 && onlyEndDate) {
									if (tempDate < onlyEndDate) {
										tempDate.setMonth(tempDate.getMonth() - addMonth);
										break;
									}
								}
								if (keyCode == 40 && onlyStartDate) {
									if (tempDate > onlyStartDate) {
										tempDate.setMonth(tempDate.getMonth() - addMonth);
										break;
									}
								}
								tempDate.setMonth(tempDate.getMonth() + addMonth);
							}
	
							if (year != tempDate.getFullYear()) {
								if (settings.lenient) {
									txtYear.val(tempDate.getFullYear());
								} else {
									return false;
								}
							}
							$(this).val(tempDate.getMonth() + 1);
	
							// 重新渲染月份容器
							var tempYear = tempTime.getFullYear();
							var tempMonth = tempTime.getMonth();
							tempTime.setFullYear(tempDate.getFullYear());
							tempTime.setMonth(tempDate.getMonth());
							renderMonthContainer.call($this);
							tempTime.setFullYear(tempYear);
							tempTime.setMonth(tempMonth);
	
							return false;
						}
						if (keyCode == 8) {// 退格
							if (startIndex == endIndex) {
								startIndex--;
							}
							letter = "";
							posIndex = startIndex;
						}
						if (keyCode == 46) {// 删除
							if (startIndex == endIndex) {
								endIndex++;
							}
							letter = "";
							posIndex = startIndex;
						}
						if (keyCode == 13) {// 回车
							$("#" + baseId + "_date_picker_today_button").focus();
							return false;
						}
	
						var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
						if (newText == "" || reg.test(newText)) {
							$(this).val(newText);
							setSelectRange(this, posIndex, posIndex);
						}
						return false;
					}).attr("disabled", !settings.enable);

					var btnMonthNext = $("<input id=\"" + baseId + "_date_picker_month_next_button\" type=\"button\" class=\"" + settings.skin.monthNextButtonCss + "\"/>");
					dpTop.append(btnMonthNext);
					btnMonthNext.click(function (event) {
						// 首先确定可用月份
						var oldMonth = tempTime.getMonth();
						tempTime.setMonth(oldMonth + 1);
						var onlyStartDate = $this.data("onlyStartDate");
						while (isDisableMonth(tempTime.getFullYear(), tempTime.getMonth())) {
							var year = tempTime.getFullYear();
							var month = tempTime.getMonth();
							if (onlyStartDate) {
								if (tempTime > onlyStartDate) {
									tempTime.setMonth(tempTime.getMonth() - 1);
									break;
								}
							}
							tempTime.setMonth(tempTime.getMonth() + 1);
						}
	
						// 若该月份的对应日期不可用，在本月内搜索最近的可用日期
						var year = tempTime.getFullYear();
						var month = tempTime.getMonth();
						if (isDisableDate(year, month, tempTime.getDate())) {
							var date = searchEnableDate(tempTime, new Date(year, month, 1, 0, 0, 0));
							if (!date) {
								date = searchEnableDate(tempTime, new Date(year, month + 1, 1, 0, 0, 0), true);
							}
							tempTime.setDate(date.getDate());
						}

						if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER) {
							$this.data("WidgetData")["value"].setTime(tempTime.getTime());
						}

						if ($.type(settings.callback.onMonthChanged) == "function") {
							settings.callback.onMonthChanged($this, oldMonth, tempTime.getMonth());
						}

						updateControl.call($this);
					}).attr("disabled", !settings.enable);
	
					var btnYearNext = $("<input id=\"" + baseId + "_date_picker_year_next_button\" type=\"button\" class=\"" + settings.skin.yearNextButtonCss + "\"/>");
					dpTop.append(btnYearNext);
					btnYearNext.click(function (event) {
						// 首先确定可用年份
						var oldYear = tempTime.getFullYear();
						var year = oldYear + 1;
						var onlyStartDate = $this.data("onlyStartDate");
						while (isDisableYear(year)) {
							if (onlyStartDate) {
								if (year > onlyStartDate.getFullYear()) {
									return;
								}
							}
							year++;
						}
						tempTime.setFullYear(year);
	
						// 若该年份的对应日期不可用，在本年内搜索最近的可用日期
						if (isDisableDate(year, tempTime.getMonth(), tempTime.getDate())) {
							var date = searchEnableDate(tempTime, new Date(year, 0, 1, 0, 0, 0));
							if (!date) {
								date = searchEnableDate(tempTime, new Date(year, 11, 31, 23, 59, 59), true);
							}
							tempTime.setMonth(date.getMonth());
							tempTime.setDate(date.getDate());
						}

						if ($.type(settings.callback.onYearChanged) == "function") {
							settings.callback.onYearChanged($this, oldYear, year);
						}

						updateControl.call($this);
					}).attr("disabled", !settings.enable);
				}
			}

			// 显示年份容器
			function showYearContainer() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var tempTime = $this.data("WidgetData")["tempTime"];
				var curYear = tempTime.getFullYear();
				var dpTop = $("#" + baseId + "_date_picker_top");
				var divYearPopup = $("#" + baseId + "_date_picker_year_popup");
				if (divYearPopup.size() == 0) {
					var divYearPopup = $("<div id=\"" + baseId + "_date_picker_year_popup\" class=\"" + settings.skin.yearPopupCss + "\"></div>");
					$("body").append(divYearPopup);
					divYearPopup.hide();
				}
				divYearPopup.html("");
				divYearPopup.data("startYear", curYear - curYear % 10);

				var dpYearContainer = $("<table id=\"" + baseId + "_date_picker_year_container\" class=\""
					+ settings.skin.yearContainerCss + "\" cellpadding=\"0\" cellspacing=\"0\"></table>");
				divYearPopup.append(dpYearContainer);
				setUnselectable(dpYearContainer);

				renderYearContainer.call($this);

				var dpYearFunctionContainer = $("<div id=\"" + baseId + "_date_picker_year_function_button_container\" class=\""
					+ settings.skin.yearFunctionButtonContainerCss + "\"></div>");
				divYearPopup.append(dpYearFunctionContainer);
				dpYearFunctionContainer.click(function(event) {
					// 标识已选择年份，年份输入框失去焦点时不再设置年份
					divYearPopup.data("selected", true);

					setTimeout(function() {
						$("#" + baseId + "_date_picker_year_input").focus();
					}, 100);
				});

				var btnYearContainerPrev = $("<input id=\"" + baseId + "_date_picker_year_container_prev_button\" type=\"button\" class=\"" + settings.skin.yearContainerPrevButtonCss + "\"/>");
				dpYearFunctionContainer.append(btnYearContainerPrev);
				btnYearContainerPrev.click(function (event) {
					var startYear = divYearPopup.data("startYear") - 10;
					var isAllDisableYear = true;
					
					for (var i = 0; i < 10; i++) {
						isAllDisableYear = isAllDisableYear && isDisableYear(startYear + i);
					}
					if (isAllDisableYear) {
						return;
					}
					divYearPopup.data("startYear", startYear);
					renderYearContainer.call($this);
				});

				var btnYearContainerNext = $("<input id=\"" + baseId + "_date_picker_year_container_next_button\" type=\"button\" class=\"" + settings.skin.yearContainerNextButtonCss + "\"/>");
				dpYearFunctionContainer.append(btnYearContainerNext);
				btnYearContainerNext.click(function (event) {
					var startYear = divYearPopup.data("startYear") + 10;
					var isAllDisableYear = true;
					
					for (var i = 0; i < 10; i++) {
						isAllDisableYear = isAllDisableYear && isDisableYear(startYear + i);
					}
					if (isAllDisableYear) {
						return;
					}
					divYearPopup.data("startYear", startYear);
					renderYearContainer.call($this);
				});

				var offset = dpTop.offset();
				var width = dpTop.outerWidth();
				var left = offset.left + width / 2 - dpYearContainer.outerWidth();
				var top = offset.top + dpTop.outerHeight();

				divYearPopup.css({
					left: left,
					top: top
				});
				divYearPopup.show();
				setAutoHide(divYearPopup, [ baseId + "_date_picker_year_input" ]);
			}

			// 渲染年份容器
			function renderYearContainer() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var tempTime = $this.data("WidgetData")["tempTime"];
				var curYear = tempTime.getFullYear();
				var startYear = $("#" + baseId + "_date_picker_year_popup").data("startYear");
				var dpYearContainer = $("#" + baseId + "_date_picker_year_container");

				dpYearContainer.html("");

				var yearRow = 5;
				for (var i = startYear; i < startYear + yearRow; i++) {
					var dpYearRow = $("<tr id=\"" + baseId + "_date_picker_year_row" + i + "\" class=\"" + settings.skin.yearRowCss + "\"></tr>");
					dpYearContainer.append(dpYearRow);

					var isDisabled = isDisableYear(i); // 禁用状态
					var isSelected = curYear == i; // 选中状态
					var dpYearCell = $("<td id=\"" + baseId + "_date_picker_year_cell" + i + "\" class=\""
						+ (isDisabled ? settings.skin.yearCellDisabledCss
						: isSelected ? settings.skin.yearCellSelectedCss
						: settings.skin.yearCellCss) + "\">" + i + "</td>");
					dpYearCell.data("YearCell", {
						value: i,
						originalCss: isDisabled ? settings.skin.yearCellDisabledCss
							: isSelected ? settings.skin.yearCellSelectedCss : settings.skin.yearCellCss
					});
					dpYearCell.data("controlId", $this.attr("id"));
					if (!isDisabled) {
						dpYearCell.hover(yearCellMouseOver, yearCellMouseOut).click(yearCellMouseClick);
					}
					dpYearRow.append(dpYearCell);

					isDisabled = isDisableYear(i + yearRow); // 禁用状态
					isSelected = curYear == i + yearRow; // 选中状态
					dpYearCell = $("<td id=\"" + baseId + "_date_picker_year_cell" + (i + yearRow) + "\" class=\""
						+ (isDisabled ? settings.skin.yearCellDisabledCss
						: isSelected ? settings.skin.yearCellSelectedCss
						: settings.skin.yearCellCss) + "\">" + (i + yearRow) + "</td>");
					dpYearCell.data("YearCell", {
						value: i + yearRow,
						originalCss: isDisabled ? settings.skin.yearCellDisabledCss
							: isSelected ? settings.skin.yearCellSelectedCss : settings.skin.yearCellCss
					});
					dpYearCell.data("controlId", $this.attr("id"));
					if (!isDisabled) {
						dpYearCell.hover(yearCellMouseOver, yearCellMouseOut).click(yearCellMouseClick);
					}
					dpYearRow.append(dpYearCell);
				}
			}

			// 显示月份容器
			function showMonthContainer() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var tempTime = $this.data("WidgetData")["tempTime"];
				var year = tempTime.getFullYear();
				var dpTop = $("#" + baseId + "_date_picker_top");
				var txtMonth = $("#" + baseId + "_date_picker_month_input");
				var divMonthPopup = $("#" + baseId + "_date_picker_month_popup");
				if (divMonthPopup.size() == 0) {
					var divMonthPopup = $("<div id=\"" + baseId + "_date_picker_month_popup\" class=\"" + settings.skin.monthPopupCss + "\"></div>");
					$("body").append(divMonthPopup);
					divMonthPopup.hide();
				}
				divMonthPopup.html("");

				var dpMonthContainer = $("<table id=\"" + baseId + "_date_picker_month_container\" class=\""
					+ settings.skin.monthContainerCss + "\" cellpadding=\"0\" cellspacing=\"0\"></table>");
				divMonthPopup.append(dpMonthContainer);
				setUnselectable(dpMonthContainer);

				renderMonthContainer.call($this);

				var offset = dpTop.offset();
				var width = dpTop.outerWidth();
				var left = offset.left + width / 2 + 2;
				var top = offset.top + dpTop.outerHeight();

				divMonthPopup.css({
					left: left,
					top: top
				});
				divMonthPopup.show();
				setAutoHide(divMonthPopup, [ baseId + "_date_picker_month_input" ]);
			}

			// 渲染月份容器
			function renderMonthContainer() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var tempTime = $this.data("WidgetData")["tempTime"];
				var year = tempTime.getFullYear();
				var curMonth = tempTime.getMonth();
				var dpMonthContainer = $("#" + baseId + "_date_picker_month_container");

				dpMonthContainer.html("");

				var monthRow = 6;
				for (var i = 0; i < monthRow; i++) {
					var dpMonthRow = $("<tr id=\"" + baseId + "_date_picker_month_row" + i + "\" class=\"" + settings.skin.monthRowCss + "\"></tr>");
					dpMonthContainer.append(dpMonthRow);

					var isDisabled = isDisableMonth(year, i); // 禁用状态
					var isSelected = curMonth == i; // 选中状态
					var dpMonthCell = $("<td id=\"" + baseId + "_date_picker_month_cell" + i + "\" class=\""
						+ (isDisabled ? settings.skin.monthCellDisabledCss
						: isSelected ? settings.skin.monthCellSelectedCss
						: settings.skin.monthCellCss) + "\">" + lang[settings.lang]["shortMonth" + i] + "</td>");
					dpMonthCell.data("MonthCell", {
						value: i,
						originalCss: isDisabled ? settings.skin.monthCellDisabledCss
							: isSelected ? settings.skin.monthCellSelectedCss : settings.skin.monthCellCss
					});
					dpMonthCell.data("controlId", $this.attr("id"));
					if (!isDisabled) {
						dpMonthCell.hover(monthCellMouseOver, monthCellMouseOut).click(monthCellMouseClick);
					}
					dpMonthRow.append(dpMonthCell);

					isDisabled = isDisableMonth(year, i + monthRow); // 禁用状态
					isSelected = curMonth == i + monthRow; // 选中状态
					dpMonthCell = $("<td id=\"" + baseId + "_date_picker_month_cell" + (i + monthRow) + "\" class=\""
						+ (isDisabled ? settings.skin.monthCellDisabledCss
						: isSelected ? settings.skin.monthCellSelectedCss
						: settings.skin.monthCellCss) + "\">" + lang[settings.lang]["shortMonth" + (i + monthRow)] + "</td>");
					dpMonthCell.data("MonthCell", {
						value: i + monthRow,
						originalCss: isDisabled ? settings.skin.monthCellDisabledCss
							: isSelected ? settings.skin.monthCellSelectedCss : settings.skin.monthCellCss
					});
					dpMonthCell.data("controlId", $this.attr("id"));
					if (!isDisabled) {
						dpMonthCell.hover(monthCellMouseOver, monthCellMouseOut).click(monthCellMouseClick);
					}
					dpMonthRow.append(dpMonthCell);
				}
			}

			// 渲染中部日期区
			function renderCenter() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var container = $this.data("WidgetData")["container"];
				var dpCenter = container.find("#" + baseId + "_date_picker_center");
				var tempTime = $this.data("WidgetData")["tempTime"];
				var year = tempTime.getFullYear(); // 设置的年份
				var month = tempTime.getMonth(); // 设置的月份
				var date = tempTime.getDate(); // 设置的日期
				var tempDate = new Date(tempTime.getTime()); // 临时使用的日期对象
				tempDate.setDate(1);
				var firstDayIndex = tempDate.getDay() - settings.firstDay; // 当月第一天的索引
				var isLastRow = false; // 是否最后一行
				var dateFragment = document.createDocumentFragment();
				var dpDateContainer = $("<table id=\"" + baseId + "_date_picker_date_container\" class=\""
					+ settings.skin.dateContainerCss + "\" cellpadding=\"0\" cellspacing=\"0\"></table>");
				dateFragment.appendChild(dpDateContainer[0]);

				// 生成日期头部
				var dpDateHeader = $("<tr id=\"" + baseId + "_date_picker_date_header\" class=\"" + settings.skin.dateHeaderCss + "\"></tr>");
				dpDateHeader.attr("disabled", !settings.enable);
				dpDateContainer.append(dpDateHeader);
				for (var i = 0; i < 7; i++) {
					var dayOfWeek = i + settings.firstDay;
					if (dayOfWeek >= 7) {
						dayOfWeek -= 7;
					}
					var dpDateHeaderCell = $("<td id=\"" + baseId + "_date_picker_date_header_cell" + i + "\" class=\"" + settings.skin.dateHeaderCellCss + "\">"
						+ lang[settings.lang]["shortDay" + dayOfWeek] + "</td>");
					dpDateHeader.append(dpDateHeaderCell);
				}

				// 确定渲染开始日期，确保前后两个月的日期必然出现
				var startDate = 1 - firstDayIndex;
				if (startDate == 1 && settings.showOther.last) {
					startDate -= 7;
				}
				tempDate.setDate(startDate);

				// 生成日期行
				for (var i = 0; (settings.showOther.next && i < 6) || !isLastRow; i++) {
					var dpDateRow = $("<tr id=\"" + baseId + "_date_picker_date_row" + i + "\" class=\"" + (i % 2 == 0 ? settings.skin.dateRowOddCss : settings.skin.dateRowEvenCss) + "\"></tr>");
					dpDateContainer.append(dpDateRow);

					// 生成日期单元格
					for (var j = 0; j < 7; j++) {
						var curYear = tempDate.getFullYear();
						var curDate = tempDate.getDate();
						var curMonth = tempDate.getMonth();
						var dayOfWeek = j + settings.firstDay;
						if (dayOfWeek >= 7) {
							dayOfWeek -= 7;
						}
						var isLastMonth = curYear < year || (curYear == year && curMonth < month); // 是否上个月
						var isNextMonth = curYear > year || (curYear == year && curMonth > month); // 是否下个月
						var isEmptyCell = (isLastMonth && !settings.showOther.last) || (isNextMonth && !settings.showOther.next);// 是否空单元格
						var isDisabled = isDisableDate(curYear, curMonth, curDate) || !settings.enable; // 禁用状态
						var isSelected = curDate == date && curMonth == month; // 选中状态
						var isHighlight = (dayOfWeek == 0 || dayOfWeek == 6) && settings.highlightWeekEnd; // 高亮状态
						var specialClasses = getSpecialClasses(curYear, curMonth, curDate);
						var cellText = isEmptyCell ? "&nbsp;" : curDate;
						var dpDateCell = $("<td id=\"" + baseId + "_date_picker_date_cell" + j + "\" class=\""
							+ (isDisabled ? specialClasses && specialClasses.dateCellDisabledCss ? specialClasses.dateCellDisabledCss : settings.skin.dateCellDisabledCss
							: isSelected ? specialClasses && specialClasses.dateCellSelectedCss ? specialClasses.dateCellSelectedCss : settings.skin.dateCellSelectedCss
							: isLastMonth || isNextMonth ? specialClasses && specialClasses.dateOtherMonthCss ? specialClasses.dateOtherMonthCss : settings.skin.dateOtherMonthCss
							: isHighlight ? specialClasses && specialClasses.dateOtherMonthCss ? specialClasses.dateHighlightCss : settings.skin.dateHighlightCss
							: specialClasses && specialClasses.dateCellCss ? specialClasses.dateCellCss : settings.skin.dateCellCss)
							+ "\">" + cellText + "</td>");
						dpDateCell.data("DateCell", {
							type: isLastMonth ? -1 : isNextMonth ? 1 : 0,
							value: curDate,
							originalCss: isDisabled ? specialClasses && specialClasses.dateCellDisabledCss ? specialClasses.dateCellDisabledCss : settings.skin.dateCellDisabledCss
								: isLastMonth || isNextMonth ? specialClasses && specialClasses.dateOtherMonthCss ? specialClasses.dateOtherMonthCss : settings.skin.dateOtherMonthCss
								: isHighlight ? specialClasses && specialClasses.dateOtherMonthCss ? specialClasses.dateHighlightCss : settings.skin.dateHighlightCss
								: specialClasses && specialClasses.dateCellCss ? specialClasses.dateCellCss : settings.skin.dateCellCss,
							selectedCss: specialClasses && specialClasses.dateCellSelectedCss ? specialClasses.dateCellSelectedCss : settings.skin.dateCellSelectedCss,
							hoverCss: specialClasses && specialClasses.dateCellHoverCss ? specialClasses.dateCellHoverCss : settings.skin.dateCellHoverCss
						});
						dpDateCell.data("controlId", $this.attr("id"));
						if (settings.displayMode != $.DATEPICKER_DISPLAY_MODE_INNER || !settings.readonly) {
							if (!isDisabled && !isEmptyCell) {
								dpDateCell.hover(dateCellMouseOver, dateCellMouseOut)
									.click(dateCellMouseClick).dblclick(dateCellMouseDblClick);
							}
						}
						dpDateRow.append(dpDateCell);

						tempDate.setDate(curDate + 1);
						curYear = tempDate.getFullYear();
						curMonth = tempDate.getMonth();
						if (curYear > year || (curYear == year && curMonth > month)) {
							isLastRow = true;
						}
					}
				}
				dpCenter.html("");
				dpCenter.append($(dateFragment));
			}

			// 渲染底部时间区
			function renderBottom() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var container = $this.data("WidgetData")["container"];
				var tempTime = $this.data("WidgetData")["tempTime"];
				var lang = $this.data("WidgetData")["lang"];
				var dpBottom = container.find("#" + baseId + "_date_picker_bottom");
				var is12HourMode = settings.timeMode == $.DATEPICKER_TIME_MODE_12_HOUR;
				var hourMask = is12HourMode ? $.DATEPICKER_FULL_HOUR_T : $.DATEPICKER_FULL_HOUR;

				var txtTime = $("<input id=\"" + baseId + "_date_picker_time_input\" type=\"text\" class=\""
					+ (is12HourMode ? settings.skin.timeInput12HourCss : settings.skin.timeInput24HourCss)
					+ "\" maxlength=\"8\" value=\"" + formatTime.call($this, tempTime, is12HourMode) + "\"/>");
				dpBottom.append(txtTime);
				if (!settings.showTime) {
					txtTime.css("visibility", "hidden");
				}
				txtTime[0].style.imeMode = "disabled";
				txtTime.data("posIndex", 0);
				txtTime.bind("paste", function(event) { return false; }).click(function(event) {
					$(this).data("posIndex", getSelectRange(this)["start"]);
				}).keydown(function(event) {
					var keyCode = event.which;

					if (keyCode == 9// Tab
						|| keyCode == 16// Shift
						|| keyCode == 17// Ctrl
						|| keyCode == 18// Alt
						|| keyCode == 35// End
						|| keyCode == 36// Home
						|| keyCode == 37// 向左
						|| keyCode == 39// 向右
						|| keyCode == 13// 回车
					) {
						return true;
					}

					var splitPoints = [
						{ index: 2, length: 1 },
						{ index: 5, length: 1 }
					];

					if (is12HourMode) {
						splitPoints.push({ index: 8, length: 1 });
					}

					var reg = new RegExp(is12HourMode
						? "^(0[1-9]|1[0-2]):([0-5]\\d):([0-5]\\d) (" + lang[settings.lang]["am"] + "|" + lang[settings.lang]["pm"] + ")$"
						: "^(2[0-3]|[01]\\d):([0-5]\\d):([0-5]\\d)$");

					var range = getSelectRange(this);
					var isSplitIndex = false;
					var isAfterSplitIndex = false;
					var index = 0;
					for (; index < splitPoints.length; index++) {
						if (splitPoints[index]["index"] == range.start) {
							isSplitIndex = true;
							break;
						}
						if (splitPoints[index]["index"] == range.start - splitPoints[index]["length"]) {
							isAfterSplitIndex = true;
							break;
						}
					}

					var $this = $(this);
					var oldText = $this.val();
					var oldHour = oldText.substr(0, 2) * 1;
					var startIndex = range.start + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var endIndex = range.start + 1 + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var letter = String.fromCharCode(keyCode);
					var posIndex = endIndex;
					var ampmChange = false;// 是否发生了上午/下午之间的切换

					if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
						letter = String.fromCharCode(keyCode - 48);
					}
					if (event.shiftKey && keyCode == 186) {// 冒号
						letter = oldText.charAt(startIndex);
						var splitIndex = oldText.indexOf(":", isSplitIndex ? startIndex - 1 : startIndex);
						if (splitIndex != -1) {
							posIndex = splitIndex + 1;
						} else {
							splitIndex = oldText.indexOf(" ", isSplitIndex ? startIndex - 1 : startIndex);
							if (splitIndex != -1) {
								posIndex = splitIndex + 1;
							} else {
								posIndex = oldText.length;
							}
						}
					}
					if (settings.enableKeyboard.time && (keyCode == 38 || keyCode == 40)) {// 向上或向下
						posIndex = isSplitIndex ? startIndex - 1 : startIndex;
						var splitBefore = oldText.lastIndexOf(" ", isSplitIndex ? posIndex - 1 : posIndex);
						if (splitBefore == -1) {
							splitBefore = oldText.lastIndexOf(":", isSplitIndex ? posIndex - 1 : posIndex);
						}

						var splitAfter = oldText.indexOf(":", posIndex);
						if (splitAfter == -1) {
							var splitAfter = oldText.indexOf(" ", posIndex);
							if (splitAfter == -1) {
								splitAfter = oldText.length;
							}
						}
						startIndex = splitBefore + 1;
						endIndex = splitAfter;
						
						if (startIndex > 8) {
							var ampmPart = oldText.substring(startIndex, endIndex);
							letter = lang[settings.lang][ampmPart == lang[settings.lang]["am"] ? "pm" : "am"];
						} else {
							var timePart = oldText.substring(startIndex, endIndex) * 1;
							timePart += keyCode == 38 ? 1 : -1;
							letter = timePart < 10 ? "0" + timePart : timePart;
						}

						// 12小时制时，11和12之间才是上午/下午的分界点
						if (is12HourMode && startIndex == 0) {
							if (letter * 1 > 12) {
								letter = "01";
							} else if (letter * 1 < 1) {
								letter = "12";
							}
							ampmChange = (oldHour == 11 && letter * 1 == 12) || (oldHour == 12 && letter * 1 == 11);
						}
					}
					if (keyCode == 8) {// 退格
						startIndex = range.start - 1 - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						endIndex = range.start - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						// 12小时制不能有0点
						letter = is12HourMode && startIndex == 1 && oldText.substr(0, 1) == "0" ? "1" : "0";
						posIndex = startIndex;
					}
					if (keyCode == 46) {// 删除
						// 12小时制不能有0点
						letter = is12HourMode && startIndex == 1 && oldText.substr(0, 1) == "0" ? "1" : "0";
						posIndex = startIndex;
					}
					if (keyCode == 32) {// 空格
						// 12小时制不能有0点
						letter = is12HourMode && startIndex == 1 && oldText.substr(0, 1) == "0" ? "1" : "0";
					}

					var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);

					// 不是12小时制时，只需要检测字符串是否合法，是12小时制时，还需要检测是否发生了上午/下午的切换
					if ((!is12HourMode && reg.test(newText)) || (is12HourMode && reg.test(newText) && !ampmChange)) {
						$this.val(newText);
						setSelectRange(this, posIndex, posIndex);
						txtTime.data("posIndex", posIndex);

						var timeParts = reg.exec(newText);
						var hour = timeParts[1] * 1;

						// 12小时制时间转换为24小时制时间
						if (is12HourMode) {
							if (timeParts[4] == lang[settings.lang]["pm"] && hour != 12) {
								hour += 12;
							}
							if (timeParts[4] == lang[settings.lang]["am"] && hour == 12) {
								hour = 0;
							}
						}

						tempTime.setHours(hour);
						tempTime.setMinutes(timeParts[2] * 1);
						tempTime.setSeconds(timeParts[3] * 1);
					} else if (settings.enableKeyboard.time && settings.lenient && (keyCode == 38 || keyCode == 40)) {
						var reg = new RegExp(is12HourMode
							? "^(\\d-?\\d):(\\d-?\\d):(\\d-?\\d) (" + lang[settings.lang]["am"] + "|" + lang[settings.lang]["pm"] + ")$"
							: "^(\\d-?\\d):(\\d-?\\d):(\\d-?\\d)$");
						var timeParts = reg.exec(newText);
						if (timeParts) {
							var hourParts = /^(\d)(-?\d)$/.exec(timeParts[1]);
							var minuteParts = /^(\d)(-?\d)$/.exec(timeParts[2]);
							var secondParts = /^(\d)(-?\d)$/.exec(timeParts[3]);
							var hour;
							var minute;
							var second;
							var minute2Hour = 0;
							var second2Minute = 0;

							if (secondParts) {
								second = secondParts[1] * 10 + secondParts[2] * 1;
								second2Minute = second < 0 ? -1 : second > 59 ? 1 : 0;
								second = (second + 60) % 60;
							}
							if (minuteParts) {
								minute = minuteParts[1] * 10 + minuteParts[2] * 1 + second2Minute;
								minute2Hour = minute < 0 ? -1 : minute > 59 ? 1 : 0;
								minute = (minute + 60) % 60;
							}
							if (hourParts) {
								hour = is12HourMode ? hourParts[1] * 10 + hourParts[2] * 1 + minute2Hour
									: (hourParts[1] * 10 + hourParts[2] * 1 + minute2Hour + 24) % 24;
							}

							if (is12HourMode) {
								if (hour > 12) {
									hour = 1;
								}
								if (hour < 1) {
									hour = 12;
								}
							}

							var timeStr = $.map([ hour, minute, second ], function(element) {
								return element < 10 ? "0" + element : element;
							}).join(":");

							if (is12HourMode) {
								var ampmPart = timeParts[4];
								var ampm = ampmPart == lang[settings.lang]["am"] ? "am" : "pm";
								if ((oldHour == 11 && hour == 12) || (oldHour == 12 && hour == 11)) {
									ampm = ampm == "am" ? "pm" : "am";
									ampmPart = lang[settings.lang][ampm];
								}
								timeStr += " " + ampmPart;

								if (ampm == "pm" && hour != 12) {
									hour += 12;
								}
								if (ampm == "am" && hour == 12) {
									hour = 0;
								}
							}

							$this.val(timeStr);
							setSelectRange(this, posIndex, posIndex);
							txtTime.data("posIndex", posIndex);

							tempTime.setHours(hour);
							tempTime.setMinutes(minute);
							tempTime.setSeconds(second);
						}
					}

					return false;
				}).keyup(function(event) {
					// 向左或向右时，记录光标移动后的位置
					if (event.which == 37 || event.which == 39) {
						$(this).data("posIndex", getSelectRange(this).start);
						return true;
					}
				}).attr("disabled", !settings.enable);

				var timeButtonContainer = $("<div id=\"" + baseId + "_date_picker_time_button_container\" class=\"" + settings.skin.timeButtonContainerCss + "\"></div>");
				dpBottom.append(timeButtonContainer);
				if (!settings.showTime) {
					timeButtonContainer.css("visibility", "hidden");
				}

				var btnTimeUp = $("<input id=\"" + baseId + "_date_picker_time_up_button\" type=\"button\" class=\"" + settings.skin.timeUpButtonCss + "\"/>");
				timeButtonContainer.append(btnTimeUp);
				btnTimeUp.click(function (event) {
					var posIndex = txtTime.data("posIndex");
					var oldTime = tempTime.getTime();
					var oldHour = tempTime.getHours();
					var fieldRanges = [
						{ start: 0, end: 2, getter: "getHours", setter: "setHours", max: 23 },
						{ start: 3, end: 5, getter: "getMinutes", setter: "setMinutes", max: 59 },
						{ start: 6, end: 8, getter: "getSeconds", setter: "setSeconds", max: 59 }
					];

					for (var i = 0; i < fieldRanges.length; i++) {
						if (posIndex >= fieldRanges[i].start && posIndex <= fieldRanges[i].end) {
							var fieldValue = tempTime[fieldRanges[i].getter]() + 1;
							if (!settings.lenient && fieldValue > fieldRanges[i].max) {
								return;
							}
							tempTime[fieldRanges[i].setter](fieldValue);
							// 12小时制时，如果不宽松，则在中午阻止改变
							if (!settings.lenient && is12HourMode) {
								var hour = tempTime.getHours();
								if (oldHour == 11 && hour == 12) {
									tempTime.setTime(oldTime);
									break;
								}
							}
							break;
						}
					}

					// 12小时制时切换上午/下午
					if (is12HourMode && posIndex >= 9) {
						var ampm = txtTime.val().substr(9) == lang[settings.lang]["am"] ? "am" : "pm";
						if (ampm == "am") {
							tempTime.setHours(tempTime.getHours() + 12);
						}
						if (ampm == "pm" && settings.lenient) {
							tempTime.setHours(tempTime.getHours() - 12);
						}
					}
					txtTime.val(formatTime.call($this, tempTime, is12HourMode));
				}).attr("disabled", !settings.enable);

				var btnTimeDown = $("<input id=\"" + baseId + "_date_picker_time_down_button\" type=\"button\" class=\"" + settings.skin.timeDownButtonCss + "\"/>");
				timeButtonContainer.append(btnTimeDown);
				btnTimeDown.click(function (event) {
					var posIndex = txtTime.data("posIndex");
					var oldTime = tempTime.getTime();
					var oldHour = tempTime.getHours();
					var fieldRanges = [
						{ start: 0, end: 2, getter: "getHours", setter: "setHours", min: 0 },
						{ start: 3, end: 5, getter: "getMinutes", setter: "setMinutes", min: 0 },
						{ start: 6, end: 8, getter: "getSeconds", setter: "setSeconds", min: 0 }
					];

					for (var i = 0; i < fieldRanges.length; i++) {
						if (posIndex >= fieldRanges[i].start && posIndex <= fieldRanges[i].end) {
							var fieldValue = tempTime[fieldRanges[i].getter]() - 1;
							if (!settings.lenient && fieldValue < fieldRanges[i].min) {
								return;
							}
							tempTime[fieldRanges[i].setter](fieldValue);
							// 12小时制时，如果不宽松，则在中午阻止改变
							if (!settings.lenient && is12HourMode) {
								var hour = tempTime.getHours();
								if (oldHour == 12 && hour == 11) {
									tempTime.setTime(oldTime);
									break;
								}
							}
							break;
						}
					}

					// 12小时制时切换上午/下午
					if (is12HourMode && posIndex >= 9) {
						var ampm = txtTime.val().substr(9) == lang[settings.lang]["am"] ? "am" : "pm";
						if (ampm == "am" && settings.lenient) {
							tempTime.setHours(tempTime.getHours() + 12);
						}
						if (ampm == "pm") {
							tempTime.setHours(tempTime.getHours() - 12);
						}
					}

					txtTime.val(formatTime.call($this, tempTime, is12HourMode));
				}).attr("disabled", !settings.enable);

				var functionButtonContainer = $("<div id=\"" + baseId + "_date_picker_function_button_container\" class=\"" + settings.skin.functionButtonContainerCss + "\"></div>");
				dpBottom.append(functionButtonContainer);

				var btnToday = $("<input id=\"" + baseId + "_date_picker_today_button\" type=\"button\" class=\"" + settings.skin.todayButtonCss + "\" value=\"" + lang[settings.lang]["todayButtonText"] + "\"/>");
				functionButtonContainer.append(btnToday);
				btnToday.click(function(event) {
					var tempTime = $this.data("WidgetData")["tempTime"];
					var today = new Date();
					if (isDisableDate(today.getFullYear(), today.getMonth(), today.getDate())) {
						return;
					}

					$this.data("innerCall", true);
					tempTime.setTime(today.getTime());
					$this.data("innerCall", false);

					if ((settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
						|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) && !settings.showTime) {
						confirmDate.call($this);
					} else {
						updateControl.call($this);
					}

                    if ($.type(settings.callback.onTodayClicked) == "function") {
                        if (settings.callback.onTodayClicked($this) === false) {
                            return;
                        }
                    }
				}).attr("disabled", !settings.enable);
				if (!settings.enable) {
					btnToday.attr("disabled", !settings.enable);
					btnToday.css("cursor", "default");
				} else {
					if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER) {
						btnToday.focus();
					}
				}

				var btnOK = $("<input id=\"" + baseId + "_date_picker_ok_button\" type=\"button\" class=\"" + settings.skin.okButtonCss + "\" value=\"" + lang[settings.lang]["okButtonText"] + "\"/>");
				functionButtonContainer.append(btnOK);
				btnOK.click(function(event) {
					if ((settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
						|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) && settings.showTime) {
						confirmDate.call($this);
					}
				});

				if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER
					|| ((settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
						|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) && !settings.showTime)) {
					btnOK.hide();
				}
			}

			// 更新控件显示
			function updateControl() {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var container = $this.data("WidgetData")["container"];
				var is12HourMode = settings.timeMode == $.DATEPICKER_TIME_MODE_12_HOUR;
				var hourMask = is12HourMode ? $.DATEPICKER_FULL_HOUR_T : $.DATEPICKER_FULL_HOUR;
				if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER) {
					confirmDate.call($this);
				}
				var tempTime = $this.data("WidgetData")["tempTime"];
				container.find("#" + baseId + "_date_picker_year_input").val(tempTime.getFullYear());
				container.find("#" + baseId + "_date_picker_month_input").val(tempTime.getMonth() + 1);
				renderCenter.call($this);
				container.find("#" + baseId + "_date_picker_time_input").val(formatTime.call($this, tempTime, is12HourMode));
			}

			// 日期单元格鼠标单击
			function dateCellMouseClick(event) {
				var $this = $(this);
				var controlId = $this.data("controlId");
				var control = $("#" + controlId);
				var settings = control.data("WidgetData")["settings"];
				var tempTime = control.data("WidgetData")["tempTime"];
				var data = $this.data("DateCell");
				clearTimeout(control.data("timeOutId"));
				// 延迟执行防止单击双击冲突
				control.data("timeOutId", setTimeout(function() {
					tempTime.setMonth(tempTime.getMonth() + data.type);
					tempTime.setDate(data.value);

					if ((settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
						|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) && !settings.showTime) {
						confirmDate.call(control);
					} else {
						updateControl.call(control);
					}
				}, 200));
			}

			// 日期单元格鼠标双击
			function dateCellMouseDblClick(event) {
				var $this = $(this);
				var controlId = $this.data("controlId");
				var control = $("#" + controlId);
				var settings = control.data("WidgetData")["settings"];
				var tempTime = control.data("WidgetData")["tempTime"];
				var data = $this.data("DateCell");

				// 不显示时间时，只触发单击
				if (!settings.showTime) {
					return;
				}

				clearTimeout(control.data("timeOutId"));

				tempTime.setMonth(tempTime.getMonth() + data.type);
				tempTime.setDate(data.value);

				if ((settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
					|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) && settings.showTime) {
					confirmDate.call(control);
				} else {
					updateControl.call(control);
				}
			}

			// 日期单元格鼠标进入
			function dateCellMouseOver(event) {
				var $this = $(this);
				if ($this.hasClass($this.data("DateCell")["selectedCss"])) {
					return;
				}
				var hoverCss = $this.data("DateCell")["hoverCss"];
				$this.removeClass().addClass(hoverCss);
			}

			// 日期单元格鼠标移出
			function dateCellMouseOut(event) {
				var $this = $(this);
				if ($this.hasClass($this.data("DateCell")["selectedCss"])) {
					return;
				}
				var originalCss = $this.data("DateCell")["originalCss"];
				$this.removeClass();
				if (originalCss) {
					$(this).removeClass().addClass(originalCss);
				}
			}

			// 年份单元格鼠标单击
			function yearCellMouseClick(event) {
				var $this = $(this);
				var controlId = $this.data("controlId");
				var control = $("#" + controlId);
				var baseId = control.data("WidgetData")["baseId"];
				var divYearPopup = $("#" + baseId + "_date_picker_year_popup");
				var tempTime = control.data("WidgetData")["tempTime"];
				var year = $this.data("YearCell").value;
				var oldYear = tempTime.getFullYear();

				if (oldYear == year) {
					divYearPopup.hide();
					return;
				}

				tempTime.setFullYear(year);

				// 若该年份的对应日期不可用，在本年内搜索最近的可用日期
				if (isDisableDate(year, tempTime.getMonth(), tempTime.getDate())) {
					var firstDate = new Date(year, 0, 1, 0, 0, 0);
					var lastDate = new Date(year, 11, 31, 23, 59, 59);
					var date = searchEnableDate(tempTime, oldYear > year ? lastDate : firstDate, oldYear > year);
					if (!date) {
						date = searchEnableDate(tempTime, oldYear < year ? lastDate : firstDate, oldYear < year);
					}
					tempTime.setMonth(date.getMonth());
					tempTime.setDate(date.getDate());
				}

				// 标识已选择年份，年份输入框失去焦点时不再设置年份
				divYearPopup.data("selected", true);

				divYearPopup.hide();

				$("#" + baseId + "_date_picker_today_button").focus();

				if ($.type(settings.callback.onYearChanged) == "function") {
					settings.callback.onYearChanged($("#" + baseId), oldYear, tempTime.getFullYear());
				}

				updateControl.call(control);
			}

			// 年份单元格鼠标进入
			function yearCellMouseOver(event) {
				var $this = $(this);
				if ($this.hasClass(settings.skin.yearCellSelectedCss)) {
					return;
				}
				$this.removeClass().addClass(settings.skin.yearCellHoverCss);
			}

			// 年份单元格鼠标移出
			function yearCellMouseOut(event) {
				var $this = $(this);
				if ($this.hasClass(settings.skin.yearCellSelectedCss)) {
					return;
				}
				var originalCss = $this.data("YearCell")["originalCss"];
				$this.removeClass();
				if (originalCss) {
					$(this).removeClass().addClass(originalCss);
				}
			}

			// 月份单元格鼠标单击
			function monthCellMouseClick(event) {
				var $this = $(this);
				var controlId = $this.data("controlId");
				var control = $("#" + controlId);
				var baseId = control.data("WidgetData")["baseId"];
				var divMonthPopup = $("#" + baseId + "_date_picker_month_popup");
				var tempTime = control.data("WidgetData")["tempTime"];
				var month = $this.data("MonthCell").value;
				var oldYear = tempTime.getFullYear();
				var oldMonth = tempTime.getMonth();

				if (oldMonth == month) {
					divMonthPopup.hide();
					return;
				}

				tempTime.setMonth(month);

				// 若该月份的对应日期不可用，在本月内搜索最近的可用日期
				if (isDisableDate(oldYear, month, tempTime.getDate())) {
					var firstDate = new Date(oldYear, month, 1, 0, 0, 0);
					var lastDate = new Date(oldYear, month + 1, 1, 0, 0, 0);
					var date = searchEnableDate(tempTime, oldMonth > month ? lastDate : firstDate, oldMonth > month);
					if (!date) {
						date = searchEnableDate(tempTime, oldMonth < month ? lastDate : firstDate, oldMonth < month);
					}
					tempTime.setDate(date.getDate());
				}

				// 标识已选择月份，月份输入框失去焦点时不再设置月份
				divMonthPopup.data("selected", true);

				divMonthPopup.hide();

				$("#" + baseId + "_date_picker_today_button").focus();

				if ($.type(settings.callback.onMonthChanged) == "function") {
					settings.callback.onMonthChanged($("#" + baseId), oldMonth, tempTime.getMonth());
				}

				updateControl.call(control);
			}

			// 月份单元格鼠标进入
			function monthCellMouseOver(event) {
				var $this = $(this);
				if ($this.hasClass(settings.skin.monthCellSelectedCss)) {
					return;
				}
				$this.removeClass().addClass(settings.skin.monthCellHoverCss);
			}

			// 月份单元格鼠标移出
			function monthCellMouseOut(event) {
				var $this = $(this);
				if ($this.hasClass(settings.skin.monthCellSelectedCss)) {
					return;
				}
				var originalCss = $this.data("MonthCell")["originalCss"];
				$this.removeClass();
				if (originalCss) {
					$(this).removeClass().addClass(originalCss);
				}
			}

			// 获取文本框选中区间
			function getSelectRange(textControl) {
				var range = {};
				if (document.selection) {
					if (textControl.tagName != undefined && textControl.tagName == "INPUT") {
						var selectionRange = document.selection.createRange();
						var textControlRange = textControl.createTextRange();
						range.text = selectionRange.text;
						var oldLength = range.text.length;
						selectionRange.setEndPoint("EndToStart", textControlRange);
						var newLength = selectionRange.text.length;
						range.start = newLength;
						range.end = newLength + oldLength;
					}
				} else {
					range.start = textControl.selectionStart;
					range.end = textControl.selectionEnd;
					range.text = textControl.value.substring(range.start, range.end);
				}
				return range;
			}

			// 设置文本框选中区间
			function setSelectRange(textControl, startIndex, endIndex) {
				if (document.selection) {
					var range = textControl.createTextRange();
					range.collapse(true);
					range.moveStart("character", startIndex);
					range.moveEnd("character", endIndex - startIndex);
					range.select();
				} else {
					textControl.setSelectionRange(startIndex, endIndex);
					textControl.focus();
				}
			}

			// 根据视口高度获取自适应弹出垂直位置
			function getFitTop(element, popup, top, win) {
				if (!win) {
					win = window;
				}
				var clientHeight = $(win).height();
				var oldPosition = popup.css("position");
				var oldLeft = popup.css("left");
				var oldTop = popup.css("top");
				popup.css({
					position: "absolute",
					left: -10000,
					top: -10000
				});
				popup.show();
				var height = popup.outerHeight();
				var scrollTop = $(document).scrollTop();// 文档滚动条高度
				var elementTop = element.offset().top;// 元素距文档顶部高度
				var clientTop = elementTop - scrollTop;// 元素距视口顶部高度
				// 在弹出层超出视口下边界且元素上部高度大于下部高度时，弹出在上部，否则弹出在下部
				if (top - scrollTop + height > clientHeight && clientTop > clientHeight - clientTop - element.outerHeight()) {
					top = element.offset().top - height;
				}
				popup.css({
					position: oldPosition,
					left: oldLeft,
					top: oldTop
				});
				return top;
			}

			// 显示弹出层
			function showPopup(left, top) {
				var $this = $(this);
				var baseId = $this.data("WidgetData")["baseId"];
				var settings = $this.data("WidgetData")["settings"];
				var container = $this.data("WidgetData")["container"];

				var divPopup = $("#" + baseId + "_date_picker_popup");
				var divPopupMask = $("#" + baseId + "_date_picker_popup_mask");

				// 针对IE宽度有修正
				divPopup.css({
					left: left,
					top: top
				});
				divPopup.show();

				divPopupMask.css({
					left: left,
					top: top,
					width: divPopup.outerWidth() - ($.browser.msie ? 2 : 4) + 2,
					height: divPopup.outerHeight()
				});
				divPopupMask.show();
				setAutoHide(divPopup, [ baseId, baseId + "_date_picker_year_popup", baseId + "_date_picker_month_popup" ]);
			}

			// 隐藏弹出层
			function hidePopup() {
				var baseId = $(this).data("WidgetData")["baseId"];
				$("#" + baseId + "_date_picker_popup").hide();
				$("#" + baseId + "_date_picker_popup_mask").hide();
			}

			// 设置点击页面其他位置自动隐藏列表
			function setAutoHide($element, relatedIds, win) {
				var elementId = $element.attr("id");
				var autoHide = function(event) {
					/**
					 * 当点击控件本身或其关联控件时，由它们自己处理事件
					 * 不在控件上阻止事件冒泡到document，是为了防止两个控件互相不能隐藏对方的弹出层
					 */
					var targetId = $(event.target).attr("id");

					if (elementId == targetId || $element.find("#" + targetId).size() > 0) {
						return;
					}

					if (relatedIds) {
						for (var i = 0; i < relatedIds.length; i++) {
							var relatedElement = $("#" + relatedIds[i]);
							if (relatedIds[i] == targetId
								|| relatedElement.find("#" + targetId).size() > 0) {
								return;
							}
						}
					}
					$element.hide();
					$("#" + $element.attr("id") + "_mask").hide();
				};
				if (!win) {
					win = window.top;
				}
				var $document = $(win.document);

				// 防止文档对象多次绑定事件处理函数
				if (!$document.data("autoHide_" + elementId + "_binded")) {
					$document.mousedown(autoHide);
					$document.data("autoHide_" + elementId + "_binded", true);
				}
				$("frame, iframe", win.document).each(function(index, element) {
					setAutoHide($element, relatedIds, element.contentWindow);
				});
			}

			// 设置元素不能被选中
			function setUnselectable($element) {
				$element
					.attr("unselectable", "on")
					.css({
						"-moz-user-select": "-moz-none",
						"-moz-user-select":"none",
						"-o-user-select": "none",
						"-khtml-user-select": "none",
						"-webkit-user-select":"none",
						"-ms-user-select": "none",
						"user-select": "none"
					})
					.bind("selectstart", function() {
						return false;
					});
			}

			// 选择日期确定
			function confirmDate() {
				var $this = $(this);
				var oldValue = $this.data("originalValue");
				oldValue = oldValue ? new Date(oldValue.getTime()) : new Date();
				var tempTime = $this.data("WidgetData")["tempTime"];
				var newValue = $this.data("WidgetData")["value"];
				newValue.setTime(tempTime.getTime());
				if ($.type(settings.callback.beforePicked) == "function") {
					if (settings.callback.beforePicked($this, oldValue, newValue) === false) {
						newValue.setTime(oldValue.getTime());
						return;
					}
				}
				if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP
					|| settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) {
					$this.val($this.wbDatePicker("getWithFormat"));
					hidePopup.call($this);
				}
				$this.data("originalValue", new Date(newValue.getTime()));

				if ($.type(settings.callback.onPicked) == "function") {
					settings.callback.onPicked($this, new Date(oldValue.getTime()), new Date(newValue.getTime()))
				}
			}

			// 判断是否禁用年份
			function isDisableYear(year) {
				var isDisabled = false;
				var disablePeriods = settings.disablePeriods;
				if (disablePeriods) {
					$.each(disablePeriods, function(index, element) {
						var type = element.type;
						var start = element.start;
						var end = element.end;
						var isGreat = true;
						var isLess = true;
						var dateParts;
						if (type == $.DATEPICKER_ANY) {
							if (start) {
								dateParts = start.split("-");
								isGreat = year > dateParts[0] * 1 || (year == dateParts[0] * 1 && dateParts[1] * 1 == 1 && dateParts[2] * 1 == 1);
							}
							if (end) {
								dateParts = end.split("-");
								isLess = year < dateParts[0] * 1 || (year == dateParts[0] * 1 && dateParts[1] * 1 == 12 && dateParts[2] * 1 == 31);
							}
							isDisabled = isDisabled || (isGreat && isLess);
						}
						return !isDisabled;
					});
				}
				return isDisabled;
			}

			// 判断是否禁用月份
			function isDisableMonth(year, month) {
				var isDisabled = false;
				var disablePeriods = settings.disablePeriods;
				if (disablePeriods) {
					$.each(disablePeriods, function(index, element) {
						var type = element.type;
						var start = element.start;
						var end = element.end;
						var isGreat = true;
						var isLess = true;
						var dateParts;
						if (type == $.DATEPICKER_YEARLY) {
							if (start) {
								dateParts = start.split("-");
								isGreat = month > dateParts[0] * 1 - 1 || month == dateParts[0] * 1 - 1 && dateParts[1] * 1 == 1;
							}
							if (end) {
								dateParts = end.split("-");
								isLess = month < dateParts[0] * 1 - 1 || month == dateParts[0] * 1 - 1 && dateParts[1] * 1 >= getMaxDate(month, year);
							}
							isDisabled = isDisabled || (isGreat && isLess);
						} else if (type == $.DATEPICKER_ANY) {
							if (start) {
								dateParts = start.split("-");
								isGreat = year > dateParts[0] * 1
									|| (year == dateParts[0] * 1 && month > dateParts[1] * 1 - 1)
									|| (year == dateParts[0] * 1 && month == dateParts[1] * 1 - 1 && dateParts[2] * 1 == 1);
							}
							if (end) {
								dateParts = end.split("-");
								isLess = year < dateParts[0] * 1
									|| (year == dateParts[0] * 1 && month < dateParts[1] * 1 - 1)
									|| (year == dateParts[0] * 1 && month == dateParts[1] * 1 - 1 && dateParts[2] * 1 == getMaxDate(month, year));
							}
							isDisabled = isDisabled || (isGreat && isLess);
						}
						return !isDisabled;
					});
				}
				return isDisabled;
			}

			// 判断是否禁用日期
			function isDisableDate(year, month, date) {
				var isDisabled = false;
				var disablePeriods = settings.disablePeriods;
				var dayOfWeek;
				if (disablePeriods) {
					$.each(disablePeriods, function(index, element) {
						var type = element.type;
						var start = element.start;
						var end = element.end;
						var isGE = true;
						var isLE = true;
						var dateParts;
						if (type == $.DATEPICKER_YEARLY) {
							if (start) {
								dateParts = start.split("-");
								isGE = month > dateParts[0] * 1 - 1
									|| (month == dateParts[0] * 1 - 1 && date >= dateParts[1] * 1);
							}
							if (end) {
								dateParts = end.split("-");
								isLE = month < dateParts[0] * 1 - 1
									|| (month == dateParts[0] * 1 - 1 && date <= dateParts[1] * 1);
							}
						} else if (type == $.DATEPICKER_MONTHLY) {
							if (start) {
								isGE = date >= start * 1;
							}
							if (end) {
								isLE = date <= end * 1;
							}
						} else if (type == $.DATEPICKER_WEEKLY) {
							if (dayOfWeek === undefined) {
								dayOfWeek = new Date(year, month, date).getDay();
							}
							if (start) {
								isGE = dayOfWeek >= start * 1;
							}
							if (end) {
								isLE = dayOfWeek <= end * 1;
							}
						} else if (type == $.DATEPICKER_ANY) {
							if (start) {
								dateParts = start.split("-");
								isGE = year > dateParts[0] * 1
									|| (year == dateParts[0] * 1 && month > dateParts[1] * 1 - 1)
									|| (year == dateParts[0] * 1 && month == dateParts[1] * 1 - 1 && date >= dateParts[2] * 1);
							}
							if (end) {
								dateParts = end.split("-");
								isLE = year < dateParts[0] * 1
									|| (year == dateParts[0] * 1 && month < dateParts[1] * 1 - 1)
									|| (year == dateParts[0] * 1 && month == dateParts[1] * 1 - 1 && date <= dateParts[2] * 1);
							}
						}
						isDisabled = isDisabled || (isGE && isLE);
						return !isDisabled;
					});
				}
				return isDisabled;
			}

			// 搜索最近的启用日期
			function searchEnableDate(baseDate, stopDate, forward) {
				var tempDate = new Date(baseDate.getTime());
				while (isDisableYear(tempDate.getFullYear())) {
					tempDate.setFullYear(tempDate.getFullYear() + (forward ? 1 : -1));
					if (stopDate && (forward ? tempDate > stopDate : tempDate < stopDate)) {
						return null;
					}
				}

				while (isDisableMonth(tempDate.getFullYear(), tempDate.getMonth())) {
					tempDate.setMonth(tempDate.getMonth() + (forward ? 1 : -1));
					if (stopDate && (forward ? tempDate > stopDate : tempDate < stopDate)) {
						return null;
					}
				}

				while (isDisableDate(tempDate.getFullYear(), tempDate.getMonth(), tempDate.getDate())) {
					tempDate.setDate(tempDate.getDate() + (forward ? 1 : -1));
					if (stopDate && (forward ? tempDate > stopDate : tempDate < stopDate)) {
						return null;
					}
				}
				return tempDate;
			}

			// 获取特殊日期样式列表
			function getSpecialClasses(year, month, date) {
				var classes = null;
				$.each(settings.specialPeriods, function(index, element) {
					var inPeriods = false;
					if (element.periods) {
						$.each(element.periods, function(index, element) {
							var startDateParts = element.start.split("-");
							var endDateParts = element.end.split("-");
							if ((year > startDateParts[0] * 1
								|| (year == startDateParts[0] * 1 && month > startDateParts[1] * 1 - 1)
								|| (year == startDateParts[0] * 1 && month == startDateParts[1] * 1 - 1 && date >= startDateParts[2] * 1))
								&& (year < endDateParts[0] * 1
								|| (year == endDateParts[0] * 1 && month < endDateParts[1] * 1 - 1)
								|| (year == endDateParts[0] * 1 && month == endDateParts[1] * 1 - 1 && date <= endDateParts[2] * 1))) {
								inPeriods = true;
								return false;
							}
						});
					}
					if (inPeriods) {
						classes = element.skin;
						return false;
					}
				});
				return classes;
			}

			// 获取某月的最大日期
			function getMaxDate(month, year) {
				switch(month) {
					case 0:
					case 2:
					case 4:
					case 6:
					case 7:
					case 9:
					case 11:
						return 31;
					case 3:
					case 5:
					case 8:
					case 10:
						return 30;
					case 1:
						var maxDate = 28;
						if (year) {
							maxDate = (year % 100 != 0 && year % 4 == 0) || year % 400 == 0 ? 29 : 28;
						}
						return maxDate;
				}
			}

			// 格式化时间
			function formatTime(time, tHour) {
				if (time instanceof Date) {
					var formatResult = (tHour ? $.DATEPICKER_FULL_HOUR_T : $.DATEPICKER_FULL_HOUR) + ":mm:ss";
					var hours = time.getHours();
					var hour12 = hours;
					if (hour12 >= 12) {
						hour12 -= 12;
					}
					if (hour12 == 0) {
						hour12 = 12;
					}
					var minutes = time.getMinutes();
					var seconds = time.getSeconds();
					var formatFrags = [ "HH", "hh", "mm", "ss" ];
					var timeFrags = [ hours, hour12, minutes, seconds ];
					for (var i = 0; i < formatFrags.length; i++) {
						formatResult = formatResult.replace(formatFrags[i], formatFrags[i].length > 1 && timeFrags[i] < 10 ? "0" + timeFrags[i] : timeFrags[i]);
					}
					if (tHour) {
						formatResult += " " + $(this).data("WidgetData")["lang"][$(this).data("WidgetData")["settings"]["lang"]][hours < 12 ? "am" : "pm"];
					}
				}
				return formatResult;
			}
		},
		/**
		 * 设置日期控件属性
		 */
		setProperty: function (settings) {
			var $this = $(this);
			var value = $this.data("WidgetData")["value"];
			var oldSettings = $this.data("WidgetData")["settings"];
			if (settings["disablePeriods"] && settings["disablePeriods"].length > 0) {
				oldSettings["disablePeriods"] = settings["disablePeriods"].concat();
				delete settings["disablePeriods"];
			}
			if (settings["specialPeriods"] && settings["specialPeriods"].length > 0) {
				oldSettings["specialPeriods"] = settings["specialPeriods"].concat();
				delete settings["specialPeriods"];
			}
			$this.wbDatePicker($.extend(true, oldSettings, settings));
			settings = $this.data("WidgetData")["settings"];
			// 若为弹出时更新文本框的值
			if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP || settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) {
				$this.val($this.wbDatePicker("getWithFormat"));
			}
		},
		/**
		 * 设置日期字段值
		 *
		 * field有效值：
		 *
		 * $.DATEPICKER_FULL_YEAR(yyyy)，设置年份
		 * $.DATEPICKER_MONTH(M)，设置月份
		 * $.DATEPICKER_DATE(d)，设置日期
		 * $.DATEPICKER_HOUR(H)，设置小时
		 * $.DATEPICKER_MINUTE(m)，设置分钟
		 * $.DATEPICKER_SECOND(s)，设置秒
		 *
		 * 只传一个参数则根据类型来判断，Number类型设置日期毫秒数，Date类型为设置日期对象
		 */
		set: function (field, value) {
			var $this = $(this);
			var baseId = $this.attr("id");
			var dateObj = $this.data("WidgetData")["value"];
			var settings = $this.data("WidgetData")["settings"];
			var lang = $this.data("WidgetData")["lang"];
			var privateFunction = $this.data("WidgetData")["privateFunction"];
			var oldYear = dateObj.getFullYear();
			var oldMonth = dateObj.getMonth();
			var oldDate = dateObj.getDate();

			switch (arguments.length) {
				case 1:
					if (typeof field == "number") {
						dateObj.setTime(field);
					} else if (field instanceof Date) {
						dateObj.setTime(field.getTime());
					}
					break;
				case 2:
					if (typeof value == "number") {
						switch (field) {
							case $.DATEPICKER_FULL_YEAR:
								dateObj.setFullYear(value);
								break;
							case $.DATEPICKER_MONTH:
								dateObj.setMonth(value);
								break;
							case $.DATEPICKER_DATE:
								dateObj.setDate(value);
								break;
							case $.DATEPICKER_HOUR:
								dateObj.setHours(value);
								break;
							case $.DATEPICKER_MINUTE:
								dateObj.setMinutes(value);
								break;
							case $.DATEPICKER_SECOND:
								dateObj.setSeconds(value);
								break;
							default:
								break;
						}
					}
					break;
			}

			if (privateFunction.isDisableDate(dateObj.getFullYear(), dateObj.getMonth(), dateObj.getDate())) {
				dateObj.setFullYear(oldYear);
				dateObj.setMonth(oldMonth);
				dateObj.setDate(oldDate);
				return;
			}

			// 内嵌式或内部调用时才更新面板
			if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER || $this.data("innerCall")) {
				$this.data("WidgetData")["tempTime"].setTime(dateObj.getTime());
				privateFunction.updateControl.call($this);
			}

			// 若为弹出时更新文本框的值
			if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP || settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) {
				$this.val($this.wbDatePicker("getWithFormat"));
			}
		},
		/**
		 * 根据格式字符串设置日期字段值
		 *
		 * 支持的格式化字符：
		 *
		 * $.DATEPICKER_FULL_YEAR(yyyy)，表示年份
		 * $.DATEPICKER_MONTH(M)，表示月份
		 * $.DATEPICKER_SHORT_MONTH(MM)，表示月份
		 * $.DATEPICKER_MEDIUM_MONTH(MMM)，表示月份
		 * $.DATEPICKER_FULL_MONTH(MMMM)，表示月份
		 * $.DATEPICKER_DATE(d)，表示日期
		 * $.DATEPICKER_FULL_DATE(dd)，表示日期
		 * $.DATEPICKER_HOUR(H)，表示小时
		 * $.DATEPICKER_FULL_HOUR(HH)，表示小时
		 * $.DATEPICKER_MINUTE(m)，表示分钟
		 * $.DATEPICKER_FULL_MINUTE(mm)，表示分钟
		 * $.DATEPICKER_SECOND(s)，表示秒
		 * $.DATEPICKER_FULL_SECOND(ss)，表示秒
		 *
		 * 只传一个参数则按照settings.dateFormat来解析
		 *
		 * 注：解析时，解析特定字段，先找长的，且只解析第一个，如字段不完整，用当前日期时间填充，所以尽量填写完整，字段
		 * 间有分隔符，且不要有重复字段，解析的字段只包括“年/月/日/时（24小时制）/分/秒”
		 */
		setWithFormat: function (format, value) {
			var $this = $(this);
			var baseId = $this.attr("id");
			var dateObj = $this.data("WidgetData")["value"];
			var settings = $this.data("WidgetData")["settings"];
			var lang = $this.data("WidgetData")["lang"];
			var privateFunction = $this.data("WidgetData")["privateFunction"];
			var oldYear = dateObj.getFullYear();
			var oldMonth = dateObj.getMonth();
			var oldDate = dateObj.getDate();

			var dateStr = value;
			if (!dateStr) {
				dateStr = format;
				format = settings.dateFormat;
			}

			var fieldList = [];
			var tempDate = new Date(dateObj.getTime());

			$.each({
				year: [ $.DATEPICKER_FULL_YEAR ],
				month: [ $.DATEPICKER_FULL_MONTH, $.DATEPICKER_MEDIUM_MONTH, $.DATEPICKER_SHORT_MONTH, $.DATEPICKER_MONTH ],
				date: [ $.DATEPICKER_FULL_DATE, $.DATEPICKER_DATE ],
				hour: [ $.DATEPICKER_FULL_HOUR, $.DATEPICKER_HOUR ],
				minute: [ $.DATEPICKER_FULL_MINUTE, $.DATEPICKER_MINUTE ],
				second: [ $.DATEPICKER_FULL_SECOND, $.DATEPICKER_SECOND ]
			}, function(key, value) {
				$.each(value, function(index, element) {
					var matcher = format.match(new RegExp("([^{]|^)(" + element + ")([^}]|$)"));
					if (matcher) {
						fieldList.push({ field: matcher[2], index: matcher.index });
						return false;
					}
				})
			});

			fieldList.sort(function(element1, element2) {
				return element1.index > element2.index ? 1 : -1;
			});

			// 去除特殊符号对正则的干扰
			format = escapeRegStr(format);

			// 设置捕获组
			$.each(fieldList, function(index, element) {
				format = format.replace(element.field, "(.+)");
			});

			// 去除无需解析项的干扰
			$.each([
				{ toReplace: $.DATEPICKER_TIME, deny: "" },
				{ toReplace: $.DATEPICKER_FULL_YEAR, deny: "y" },
				{ toReplace: $.DATEPICKER_YEAR, deny: "y" },
				{ toReplace: $.DATEPICKER_FULL_DATE, deny: $.DATEPICKER_DATE },
				{ toReplace: $.DATEPICKER_DATE, deny: $.DATEPICKER_DATE },
				{ toReplace: $.DATEPICKER_FULL_HOUR, deny: $.DATEPICKER_HOUR },
				{ toReplace: $.DATEPICKER_HOUR, deny: $.DATEPICKER_HOUR },
				{ toReplace: $.DATEPICKER_FULL_MINUTE, deny: $.DATEPICKER_MINUTE },
				{ toReplace: $.DATEPICKER_MINUTE, deny: $.DATEPICKER_MINUTE },
				{ toReplace: $.DATEPICKER_FULL_SECOND, deny: $.DATEPICKER_SECOND },
				{ toReplace: $.DATEPICKER_SECOND, deny: $.DATEPICKER_SECOND },
				{ toReplace: $.DATEPICKER_FULL_MONTH, deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_MEDIUM_MONTH, deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_SHORT_MONTH, deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_MONTH, deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_FULL_DAY, deny: $.DATEPICKER_DAY },
				{ toReplace: $.DATEPICKER_MEDIUM_DAY, deny: $.DATEPICKER_DAY },
				{ toReplace: $.DATEPICKER_DAY, deny: $.DATEPICKER_DAY }
			], function(index, element) {
				format = format.replace(new RegExp("([^{" + element.deny + "]|^)"
					+ element.toReplace + "([^}" + element.deny + "]|$)", "g"), "$1.+$2");
			});

			format = format.replace(new RegExp("\\\\{|\\\\}", "g"), "");

			var setFunctionMapping = {};
			setFunctionMapping[$.DATEPICKER_FULL_YEAR] = "setFullYear";
			setFunctionMapping[$.DATEPICKER_FULL_MONTH] = "setMonth";
			setFunctionMapping[$.DATEPICKER_MEDIUM_MONTH] = "setMonth";
			setFunctionMapping[$.DATEPICKER_SHORT_MONTH] = "setMonth";
			setFunctionMapping[$.DATEPICKER_MONTH] = "setMonth";
			setFunctionMapping[$.DATEPICKER_FULL_DATE] = "setDate";
			setFunctionMapping[$.DATEPICKER_DATE] = "setDate";
			setFunctionMapping[$.DATEPICKER_FULL_HOUR] = "setHours";
			setFunctionMapping[$.DATEPICKER_HOUR] = "setHours";
			setFunctionMapping[$.DATEPICKER_FULL_MINUTE] = "setMinutes";
			setFunctionMapping[$.DATEPICKER_MINUTE] = "setMinutes";
			setFunctionMapping[$.DATEPICKER_FULL_SECOND] = "setSeconds";
			setFunctionMapping[$.DATEPICKER_SECOND] = "setSeconds";

			var dateParts = dateStr.match(new RegExp("^" + format + "$"));

			if (dateParts) {
				var isMatch = true;

				$.each(fieldList, function(index, element) {
					var fieldValue = dateParts[index + 1];
					if (element.field == $.DATEPICKER_YEAR) {
						var year = dateObj.getFullYear();
						fieldValue = year - year % 100 + fieldValue * 1;
					}
					if (element.field == $.DATEPICKER_MONTH
						|| element.field == $.DATEPICKER_SHORT_MONTH) {
						fieldValue--;
					}
					if (element.field == $.DATEPICKER_FULL_MONTH
						|| element.field == $.DATEPICKER_MEDIUM_MONTH) {
						var baseKey = element.field == $.DATEPICKER_FULL_MONTH ? "fullMonth" : "shortMonth";
						for (var i = 0; i < 12; i++) {
							if (lang[settings.lang][baseKey + i] == fieldValue) {
								fieldValue = i;
								break;
							}
						}
					}
					if (isNaN(fieldValue)) {
						isMatch = false;
						return false;
					}
					tempDate[setFunctionMapping[element.field]](fieldValue * 1);
				});
				if (isMatch) {
					dateObj.setTime(tempDate.getTime());
				}
			}

			// 转义正则表达式特殊字符
			function escapeRegStr(regStr) {
				$.each([ "\\\\", "\\.", "\\*", "\\$", "\\^", "\\{", "\\}", "\\[", "\\]", "\\(", "\\)"
					, "\\|", "\\+", "\\?" ], function(index, element) {
					regStr = regStr.replace(new RegExp(element, "g"), element);
				});
				return regStr;
			}

			if (privateFunction.isDisableDate(dateObj.getFullYear(), dateObj.getMonth(), dateObj.getDate())) {
				dateObj.setFullYear(oldYear);
				dateObj.setMonth(oldMonth);
				dateObj.setDate(oldDate);
				return;
			}

			// 内嵌式或内部调用时才更新面板
			if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_INNER || $this.data("innerCall")) {
				$this.data("WidgetData")["tempTime"].setTime(dateObj.getTime());
				privateFunction.updateControl.call($this);
			}

			// 若为弹出时更新文本框的值
			if (settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP || settings.displayMode == $.DATEPICKER_DISPLAY_MODE_POPUP_SHARE) {
				$this.val($this.wbDatePicker("getWithFormat"));
			}
		},
		/**
		 * 获取日期字段值
		 *
		 * field有效值：（假定日期时间为：2014-02-07 09:03:06）
		 *
		 * $.DATEPICKER_FULL_YEAR(yyyy)，结果：2014（Number）
		 * $.DATEPICKER_MONTH(M)，结果：1（Number）
		 * $.DATEPICKER_DATE(d)，结果：7（Number）
		 * $.DATEPICKER_HOUR(H)，结果：9（Number）
		 * $.DATEPICKER_MINUTE(m)，结果：3（Number）
		 * $.DATEPICKER_SECOND(s)，结果：6（Number）
		 * $.DATEPICKER_DAY(D)，结果：5（Number）
		 * $.DATEPICKER_TIME(T)，结果：1391734986000（Number）
		 * $.DATEPICKER_DATE_OBJ(dateObj)，结果：日期对象（Date）
		 */
		get: function (field) {
			var value = $(this).data("WidgetData")["value"];
			var lang = $(this).data("WidgetData")["lang"];
			var settings = $(this).data("WidgetData")["settings"];
			switch (field) {
				case $.DATEPICKER_FULL_YEAR:
					return value.getFullYear();
				case $.DATEPICKER_MONTH:
					return value.getMonth();
				case $.DATEPICKER_DATE:
					return value.getDate();
				case $.DATEPICKER_HOUR:
					return value.getHours();
					return (hour < 10 ? "0" : "") + hour;
				case $.DATEPICKER_MINUTE:
					return value.getMinutes();
				case $.DATEPICKER_SECOND:
					return value.getSeconds();
				case $.DATEPICKER_DAY:
					return value.getDay();
				case $.DATEPICKER_TIME:
					return value.getTime();
				case $.DATEPICKER_DATE_OBJ:
					return new Date(value.getTime());
			}
		},
		/**
		 * 根据格式字符串获取日期字段值
		 *
		 * 支持的格式化字符：（假定日期时间为：2014-02-07 09:03:06）
		 *
		 * $.DATEPICKER_YEAR(yy)，结果：14（String）
		 * $.DATEPICKER_FULL_YEAR(yyyy)，结果：2014（String）
		 * $.DATEPICKER_MONTH(M)，结果：2（String）
		 * $.DATEPICKER_SHORT_MONTH(MM)，结果：02（String）
		 * $.DATEPICKER_MEDIUM_MONTH(MMM)，结果：二、Feb、……（String）
		 * $.DATEPICKER_FULL_MONTH(MMMM)，结果：二月、February、……（String）
		 * $.DATEPICKER_DATE(d)，结果：7（String）
		 * $.DATEPICKER_FULL_DATE(dd)，结果：07（String）
		 * $.DATEPICKER_HOUR(H)，结果：9（String）
		 * $.DATEPICKER_FULL_HOUR(HH)，结果：09（String）
		 * $.DATEPICKER_HOUR_T(h)，结果：9（String）
		 * $.DATEPICKER_FULL_HOUR_T(hh)，结果：09（String）
		 * $.DATEPICKER_MINUTE(m)，结果：3（String）
		 * $.DATEPICKER_FULL_MINUTE(mm)，结果：03（String）
		 * $.DATEPICKER_SECOND(s)，结果：6（String）
		 * $.DATEPICKER_FULL_SECOND(ss)，结果：06（String）
		 * $.DATEPICKER_DAY(D)，结果：5（String）
		 * $.DATEPICKER_MEDIUM_DAY(DDD)，结果：五、Fri、……（String）
		 * $.DATEPICKER_FULL_DAY(DDDD)，结果：星期五、Friday、……（String）
		 * $.DATEPICKER_AM_PM(tt)，结果：上午、AM、……（String）
		 *
		 * 按字符串格式化，如："yy年M月d日 H时m分s秒 DDDD"，结果："14年2月7日 9时3分6秒 星期五"
		 * 省略参数则按settings.dateFormat格式化，默认："yyyy-MM-dd HH:mm:ss"，结果："2014-02-07 09:03:06"
		 *
		 * 注：转义特殊字符用大括号括起来，如"yyyy {yyyy}"格式化后为"2014 yyyy"，转义大括号使用"{{}"和"{}}"
		 */
		getWithFormat: function (format) {
			var value = $(this).data("WidgetData")["value"];
			var lang = $(this).data("WidgetData")["lang"];
			var settings = $(this).data("WidgetData")["settings"];
			var formatResult = settings.dateFormat;
			if (typeof format == "string") {
				formatResult = format;
			}
			var dayOfWeek = value.getDay();
			var second = value.getSeconds();
			var minute = value.getMinutes();
			var hour = value.getHours();
			var hour12 = hour;
			if (hour12 >= 12) {
				hour12 -= 12;
			}
			if (hour12 == 0) {
				hour12 = 12;
			}
			var date = value.getDate();
			var month = value.getMonth();
			var year = value.getFullYear();
			var es = settings.escapeSuffix;

			// 需要国际化的字段放在最后替换，因为可能出现与占位符一样的字母
			$.each([
				{ toReplace: $.DATEPICKER_TIME, replacement: value.getTime(), deny: "" },
				{ toReplace: $.DATEPICKER_FULL_YEAR, replacement: year, deny: "y" },
				{ toReplace: $.DATEPICKER_YEAR, replacement: String(year).substr(String(year).length - 2), deny: "y" },
				{ toReplace: $.DATEPICKER_FULL_DATE, replacement: (date < 10 ? "0" : "") + date, deny: $.DATEPICKER_DATE },
				{ toReplace: $.DATEPICKER_DATE, replacement: date, deny: $.DATEPICKER_DATE },
				{ toReplace: $.DATEPICKER_FULL_HOUR, replacement: (hour < 10 ? "0" : "") + hour, deny: $.DATEPICKER_HOUR },
				{ toReplace: $.DATEPICKER_HOUR, replacement: hour, deny: $.DATEPICKER_HOUR },
				{ toReplace: $.DATEPICKER_FULL_HOUR_T, replacement: (hour12 < 10 ? "0" : "") + hour12, deny: $.DATEPICKER_FULL_HOUR_T },
				{ toReplace: $.DATEPICKER_HOUR_T, replacement: hour12, deny: $.DATEPICKER_HOUR_T },
				{ toReplace: $.DATEPICKER_FULL_MINUTE, replacement: (minute < 10 ? "0" : "") + minute, deny: $.DATEPICKER_MINUTE },
				{ toReplace: $.DATEPICKER_MINUTE, replacement: minute, deny: $.DATEPICKER_MINUTE },
				{ toReplace: $.DATEPICKER_FULL_SECOND, replacement: (second < 10 ? "0" : "") + second, deny: $.DATEPICKER_SECOND },
				{ toReplace: $.DATEPICKER_SECOND, replacement: second, deny: $.DATEPICKER_SECOND },
				{ toReplace: $.DATEPICKER_AM_PM, replacement: lang[settings.lang][value.getHours() < 12 ? "am" : "pm"].replace(new RegExp($.DATEPICKER_AM_PM + "|" + $.DATEPICKER_DAY + "|" + $.DATEPICKER_MONTH, "g"), "{$&}"), deny: $.DATEPICKER_AM_PM },
				{ toReplace: $.DATEPICKER_FULL_MONTH, replacement: lang[settings.lang]["fullMonth" + month].replace(new RegExp($.DATEPICKER_DAY + "|" + $.DATEPICKER_MONTH, "g"), "{$&}"), deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_MEDIUM_MONTH, replacement: lang[settings.lang]["shortMonth" + month].replace(new RegExp($.DATEPICKER_DAY + "|" + $.DATEPICKER_MONTH, "g"), "{$&}"), deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_SHORT_MONTH, replacement: (month + 1 < 10 ? "0" : "") + (month + 1), deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_MONTH, replacement: month + 1, deny: $.DATEPICKER_MONTH },
				{ toReplace: $.DATEPICKER_FULL_DAY, replacement: lang[settings.lang]["fullDay" + dayOfWeek], deny: $.DATEPICKER_DAY },
				{ toReplace: $.DATEPICKER_MEDIUM_DAY, replacement: lang[settings.lang]["shortDay" + dayOfWeek], deny: $.DATEPICKER_DAY },
				{ toReplace: $.DATEPICKER_DAY, replacement: dayOfWeek, deny: $.DATEPICKER_DAY }
			], function(index, element) {
				formatResult = formatResult.replace(new RegExp("([^{" + element.deny + "]|^)"
					+ element.toReplace + "([^}" + element.deny + "]|$)", "g"), "$1" + element.replacement + "$2");
			});
			return formatResult.replace(new RegExp("{([^{}]+)}", "g"), "$1")
				.replace(new RegExp("([^{]|^){}([^}]|$)", "g"), "$1$2").replace("{{}", "{").replace("{}}", "}");
		}
	};

	$.fn.wbDatePicker = function (method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.wbDatePicker");
		}
	};
})(jQuery);