﻿/**
* 表格
*/

(function ($) {
    var methods = {
        /**
        * 初始化
        * @param options 初始化参数
        * @returns
        */
        init: function (options) {
            // 设定属性--覆盖默认属性
            var settings = $.extend(true, {
                // 数据类参数
                data: {
                    colNames: [], // 列显示名
                    colModel: [], // 列模型
                    rowList: [], // 数据（界面的数据映射，与界面保持同步）
                    sortorder: "asc", // 排序方式
                    sortname: "", // 排序列名（colModel中的name）
                    treeGrid: false, //是否为树表，bool，当为树表时，数据中需要包含几个特定的列
                    treeExpandColClick: false, //boolean 如果为true，则点击树展开列对应单元格的任何位置都会展开或收起
                    treeExpandColumn: "", //树展开列列名
                    treeReader: {
                        level_field: "level", //number 节点的层级，根节点的值最低（可以不为0），子节点依层级递增
                        parent_id_field: "parent", //父节点，根节点的值为null
                        leaf_field: "isLeaf", //boolean 是否叶子节点
                        expanded_field: "expanded", //boolean 初始化时是否展开该元素
                        expanded_field_colspan: "colspan"//展开列横跨列数
                    }
                },
                // 显示控制类参数
                view: {
                    width: 0, //表格外框的宽度（可见列宽之和大于该值，则出现横向滚动条），如何不设定，则以块级元素的方式展示
                    height: 0, //表格外框的宽度（可见列宽之和大于该值，则出现横向滚动条），如何不设定，则以块级元素的方式展
                    altRow: true, // 是否启用行交替色
                    selectRow: true, //是否支持选择行
                    multiselect: false, // 是否支持多选
                    treemultiselect: false, // 是否树支持多选（treeGrid为true时有效）
                    rownumbers: false, // 是否显示行号
                    headHidden: false, // 是否隐藏表头
                    colAutoInitWidth: true, //是否按网页初始宽度设置初始列宽
                    lang: 'en-us', //语言
                    colselListAjustParent: null//表头选择弹窗按钮偏移校准父容器，默认为表格
                },
                // 事件
                callback: {
                    //					beforeClickCell : null,
                    //					beforeDblClickCell : null,
                    //					beforeClickRow : null,
                    //					beforeDblClickRow : null,
                    onClickCell: null, // 单击单元格事件
                    onDblClickCell: null, // 双击单元格事件
                    onClickRow: null, // 单击行事件
                    onDblClickRow: null, // 双击行事件
                    onSortCol: null, // 排序事件
                    cellValueChanged: null, //单元格值改变事件
                    onSelectAll: null, //全选事件
                    onBeforeTreeExpand: null, //树节点展开前事件
                    onBeforeTreeCollapse: null //树节点收起前事件
                },
                gridContext: document
            }, options || {});

            var langPacket = {
                "zh-cn": {
                    "index": "序号"
                },
                "zh-tw":{
                    "index": "序號"
                },
                "en-us": {
                    "index": "Index"
                }
            };

            return this.each(function () {
                var $this = $(this);
                var pp = {
                    keyNames: [],
                    expandable: false, //是否是可展开视图
                    expandableColIndex: 0, //展开列行索引
                    inputLenFix: -10, //单元格内输入控件长度修复值
                    colStyleMap: [], //列行内样式缓存
                    colIndexStart: 1, //列起始索引
                    colModelMap: null, //列模型的列名映射
                    TreeWrapWidthBase: 36//树展开按钮包裹对象宽度基数
                };
                //私有变量
                var rowList = settings.data.rowList, colStyleMap = pp.colStyleMap,
				colNames = settings.data.colNames, colModel = settings.data.colModel,
				keyNames = pp.keyNames,
				inputLenFix = pp.inputLenFix;
                $this.data('WidgetData', {
                    target: $this,
                    settings: settings,
                    dataSource: $.extend(true, [], settings.data.rowList), //原始数据					
                    privateProperty: pp//私有属性
                });

                if (settings.view.lang && settings.view.lang in langPacket) {

                }
                else {
                    settings.view.lang = 'en-us';
                }

                var langObj = langPacket[settings.view.lang];

                //				var relativeScreenWidth = 1280;//相对屏幕分辨率宽度，colAutoInitWidth为true时，将按实际分辨率与该值的比例进行设置初始列宽
                //				var screenWidth = window.screen.width;
                //				var initColWidthScale = 1;//计算列宽比例
                //				if(relativeScreenWidth<screenWidth){
                //					initColWidthScale = screenWidth/relativeScreenWidth;
                //				}

                //是否按网页初始宽度设置初始列宽
                if (settings.view.colAutoInitWidth) {
                    var colTotleWidth = 0;
                    for (var i = 0; i < colModel.length; i++) {
                        colTotleWidth += colModel[i].hidden ? 0 : colModel[i].width;
                    }
                    if (settings.view.rownumbers)
                        colTotleWidth += 60;
                    if (settings.view.selectRow && settings.view.multiselect)
                        colTotleWidth += 30;

                    colTotleWidth += 30; //滚动条预留位置

                    var realTbWidth = settings.view.width ? settings.view.width : $this.width();

                    if (realTbWidth > colTotleWidth) {
                        var initColWidthScale = realTbWidth / colTotleWidth;
                        for (var i = 0; i < colModel.length; i++) {
                            colModel[i].width = colModel[i].width * initColWidthScale;
                        }
                    }
                }


                if (colNames.length == 0)
                    throw "no column";

                if (colNames.length != colModel.length)
                    throw "length of colNames and colModel are not equal";

                var colWidthSum = 0; //可见列宽之和				
                //初始化表头
                var initHtml = "<div class=\"grid-hdiv\"" + (settings.view.headHidden ? " style=\"display:none;\"" : "") + ">" +
            	"<table class=\"grid-htable\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\" >" +
            	"<thead>";
                var isHidden, align, fistRow = "";
                initHtml += "<th class=\"grid-status-col\"></th>";
                fistRow += "<td class=\"grid-status-col\"></td>";
                if (settings.view.rownumbers) {
                    initHtml += "<th class='grid-rownumber-col'><div class=\"grid-colname\">" + langObj["index"] + "</div></th>";
                    fistRow += "<td class='grid-rownumber-col'></td>";
                    pp.colIndexStart++;
                }
                if (settings.view.selectRow && settings.view.multiselect) {
                    initHtml += "<th class='grid-muti-col'><input type='checkbox' class='grid-check-all' /></th>";
                    fistRow += "<td class='grid-muti-col'></td>";
                    pp.colIndexStart++;
                }
                for (var i = 0; i < colNames.length; i++) {
                    if (!pp.expandable && colModel[i].formatter == "expandable") {
                        pp.expandable = true;
                        pp.expandableColIndex = i;
                    }
                    if (colModel[i].key) {
                        keyNames.push(colModel[i].name);
                    }
                    isHidden = colModel[i].hidden ? "display:none;" : "";
                    align = colModel[i].align ? "text-align:" + colModel[i].align + ";" : "";
                    colStyleMap.push(isHidden + align);
                    initHtml += "<th style=\"width:" + colModel[i].width + "px;" + isHidden + "\">" +
					"<span class=\"grid-resize" + ((colModel[i].resizable === false) ? " disabled" : "") + "\"></span>";
                    if (colModel[i].colselSwitch) {
                        initHtml += "<div class=\"grid-col-sel\"></div>";
                    }
                    initHtml += "<div class=\"grid-colname" + (colModel[i].sortable ? " grid-sort" : "") + "\">" + colNames[i] + "</div></th>";
                    fistRow += "<td style=\"width:" + colModel[i].width + "px;" + isHidden + "\"></td>";
                    if (!isHidden) {
                        colWidthSum += colModel[i].width;
                    }
                }
                initHtml += "<th style=\"width:20px;border:0px\">" + //填充滚动条宽度
				"</thead>" +
            	"</table>" +
            	"</div>";

                //初始化表体
                initHtml += "<div class=\"grid-bdiv\">" +
            	"<table class=\"grid-btable\" cellpadding=\"0\" cellspacing=\"0\" border=\"0\" >" +
            	"<thead><tr class=\"grid-firstrow\">" + fistRow + "</tr></thead>" +
            	"<tbody>" + buildBodyHtml(this) + "</tbody>" +
            	"</table></div>";
                $this.addClass('grid-container').html(initHtml);
                if (settings.view.width) {
                    $(".grid-hdiv,.grid-bdiv", $this).width(settings.view.width);
                    $this.width(settings.view.width);
                }
                if (settings.view.height)
                    $(".grid-bdiv", $this).height(settings.view.height);
                $(".grid-htable,.grid-btable", $this).width(colWidthSum);
                var $resizeMark = $("<div class='grid-resize-mark'></div>");
                $this.append($resizeMark);

                $(".grid-bdiv", $this).scroll(function () {
                    $(".grid-hdiv", $this)[0].scrollLeft = $(".grid-bdiv", $this)[0].scrollLeft;
                    $(".grid-col-sel-list", $this).hide();
                });

                bindBodyEvent($this);
                if ($(".grid-status-col.loading,.grid-status-col.success,.grid-status-col.error", $this).length > 0) {
                    $(".grid-status-col", $this).show();
                }
                else {
                    $(".grid-status-col", $this).hide();
                }

                //绑定表头选择按钮事件
                $(".grid-col-sel", $this).click(function (e) {
                    var $self = $(this);
                    $(".grid-col-sel-list", $this).hide();
                    var $th = $self.closest("th");
                    if (e.stopPropagation)
                        e.stopPropagation();
                    if ($th.length == 0)
                        return;
                    var colIndex = $th[0].cellIndex;
                    var colMdIndex = colIndex - pp.colIndexStart;
                    if (!colModel[colMdIndex].colselParam)
                        return;

                    var colselList = $("[colIndex='" + colIndex + "'].grid-col-sel-list", $this);
                    var w = colModel[colMdIndex].colselParam['width'],
            		h = colModel[colMdIndex].colselParam['height'];
                    if (colselList.length == 0) {
                        var listHtml = "<div colIndex=\"" + colIndex + "\" class=\"grid-col-sel-list\" style=\""
                		+ (w ? "width:" + w + "px;" : "")
                		+ (h ? "height:" + h + "px;" : "") + "\">";
                        if (colModel[colMdIndex].colselParam['mode'] === 'default') {
                            $.each(colModel[colMdIndex].colselParam['param'], function (i, elem) {
                                listHtml += "<div class=\"grid-col-sel-list-item\" value=\"" + elem.value + "\">" + elem.text + "</div>";
                            });
                        }
                        listHtml += "</div>";
                        colselList = $(listHtml);
                        colselList.appendTo($this);
                        if (colModel[colMdIndex].colselParam['mode'] === 'default') {
                            //列元素点击事件
                            $(">div", colselList).click(function (e) {
                                //                    			if(colModel[colMdIndex].formatter == "select"){
                                //                    				$(".grid-btable tr td:nth-child("+(colIndex+1)+")>select",$this).val($(this).attr("value"));
                                //                    				$(".grid-btable tr td:nth-child("+(colIndex+1)+")>span.cell_span",$this).hide().html($(this).text()).show();
                                //                    				//先隐藏后显示，为了解决IE下改变span内容后不偶尔会不对齐的bug
                                //                    			}
                                if (colModel[colMdIndex].colselParam['clickItem']) {
                                    colModel[colMdIndex].colselParam['clickItem'](e, $this.attr("id"), $(this).attr("value"));
                                }
                                hideList();
                            });
                        }
                        else {
                            colModel[colMdIndex].colselParam['init']($this.attr("id"), colselList);
                        }
                    }
                    else {
                        //自定义模式，且初始化模式为everyTime时重新初始化
                        if (colModel[colMdIndex].colselParam['mode'] === 'custom' && colModel[colMdIndex].colselParam['initMode'] === 'everyTime') {
                            colselList.empty();
                            colModel[colMdIndex].colselParam['init']($this.attr("id"), colselList);
                        }
                    }
                    //表头选择弹窗偏移位置计算：默认显示按钮右侧，如果宽度多于按钮到父容器右侧的宽度，显示在按钮左侧
                    var pw = $this.width(); //校准父容器宽度
                    if (settings.view.colselListAjustParent) {
                        pw = settings.view.colselListAjustParent.width();
                    }
                    var colselListleft, fix = w ? w : 100;
                    if (pw - $self.position().left < fix) {
                        colselListleft = $self.position().left - fix + $self.width() + 3;
                    }
                    else {
                        colselListleft = $self.position().left;
                    }
                    colselList.css({ "top": ($th.height() + 2) + 'px', "left": colselListleft, "max-height": $(".grid-bdiv", $this).height() - 3 });

                    colselList.show();

                    function hideList() {
                        colselList.hide();
                        $(settings.gridContext).unbind('click', 'hideList');
                    }

                    $(colselList).click(function (e) {
                        if (e.stopPropagation)
                            e.stopPropagation();
                    });
                    $(settings.gridContext).click(hideList);
                });

                var resizing = false;
                //调整列宽拖拽mousedown
                $(".grid-htable th", $this).mousedown(function (e) {
                    var $resizeSpan = $(e.target).closest("th>span.grid-resize");
                    if ($resizeSpan.length != 1 || $resizeSpan.hasClass("disabled")) { return; }
                    if (e.stopPropagation)
                        e.stopPropagation();
                    //当前调整列宽的表头
                    var $resizeTh = $(this);
                    widget.dragSetCapture(this);
                    resizing = true;
                    settings.gridContext.onselectstart = function () { return false; };

                    //最小左偏移
                    var minLeft = 0, resizeCellIndex = this.cellIndex;
                    var resizeColModelIndex = resizeCellIndex - pp.colIndexStart;
                    $(".grid-htable th", $this).each(function (i, elem) {
                        if (i < resizeCellIndex) {
                            if (i < pp.colIndexStart) {
                                minLeft += $(elem).is(":hidden") ? 0 : $(elem).width();
                            }
                            else {
                                minLeft += (!colModel[i - pp.colIndexStart].hidden ? $(elem).width() : 0);
                            }
                        }
                        else
                            return false;
                    });
                    //                    if(settings.view.selectRow && settings.view.multiselect)
                    //                    	minLeft+=$(".grid-htable .grid-muti-col",$this).width();
                    //                    if(settings.view.rownumbers)
                    //                    	minLeft+=$(".grid-htable .grid-rownumber-col",$this).width();
                    minLeft -= $(".grid-bdiv", $this).scrollLeft(); //减去相对滚动条左侧的偏移
                    minLeft += 30; //列宽最小不能超过20px;
                    var maxLeft = $this.width();

                    $resizeMark.css("left", getMouseLeft(e) + "px").show();
                    $("body").css('cursor', 'col-resize');

                    //调整列宽拖拽mouseover
                    function gird_resize_mouseover(e) {
                        if (!resizing) return;
                        var curLeft = getMouseLeft(e);
                        $resizeMark.css("left", curLeft < minLeft ? minLeft : (curLeft > maxLeft ? curLeft : curLeft) + "px");
                    }
                    $(settings.gridContext).mousemove(gird_resize_mouseover);
                    //调整列宽拖拽mouseup
                    function gird_resize_mouseup(e) {
                        resizing = false;
                        var newWidth = $resizeMark.offset().left - $resizeTh.offset().left;
                        $resizeTh.width(newWidth);
                        colModel[resizeColModelIndex].width = newWidth;
                        $(".grid-btable tr.grid-firstrow>td:eq(" + (resizeColModelIndex + pp.colIndexStart) + ")", $this).width(newWidth);
                        $(".grid-btable tr td:nth-child(" + (resizeColModelIndex + pp.colIndexStart + 1) + ")>.cell_input", $this).width(newWidth + inputLenFix);
                        tbw = getTableWidth();
                        $(".grid-htable,.grid-btable", $this).width(tbw);
                        $("body").css('cursor', 'default');
                        widget.dragReleaseCapture(this);
                        $resizeMark.hide();
                        settings.gridContext.onselectstart = function () { return true; };
                        $(settings.gridContext).unbind("mousemove", gird_resize_mouseover);
                        $(settings.gridContext).unbind("mouseup", gird_resize_mouseup);
                    }
                    $(settings.gridContext).mouseup(gird_resize_mouseup);
                });

                //排序
                $(".grid-htable div.grid-sort", $this).click(function (e) {
                    var $sort = $(this);
                    var colIndex = $(e.target).closest("th")[0].cellIndex;
                    var colMdIndex = colIndex - pp.colIndexStart;
                    if (!colModel[colMdIndex].sortable)
                        return;
                    if (!settings.callback.onSortCol)
                        throw 'no onSortCol';
                    if ($sort.hasClass('grid-sort-asc') || $sort.hasClass('grid-sort-desc')) {

                    }
                    else {
                        $(".grid-htable th", $this).removeClass('grid-sort-asc').removeClass('grid-sort-desc');
                    }
                    if (!$sort.hasClass('grid-sort-asc')) {
                        $sort.removeClass('grid-sort-desc').addClass('grid-sort-asc');
                        settings.callback.onSortCol($this.attr('id'), colModel[colMdIndex].name, 'asc');
                    }
                    else {
                        $sort.removeClass('grid-sort-asc').addClass('grid-sort-desc');
                        settings.callback.onSortCol($this.attr('id'), colModel[colMdIndex].name, 'desc');
                    }
                });

                //全选
                var $checkAll = $("input.grid-check-all", $this);
                $checkAll.click(function (e) {
                    if (e.stopPropagation)
                        e.stopPropagation();
                    checkAllEvent(e, $(this).is(":checked"));
                });

                function checkAllEvent(e, isChecked) {
                    var $check = $("input.grid-check", $this);
                    if (isChecked) {
                        $check.attr("checked", "checked");
                        $(".grid-btable tr.grid-row", $this).addClass("selected");
                    }
                    else {
                        $check.removeAttr("checked");
                        $(".grid-btable tr.grid-row", $this).removeClass("selected");
                    }
                    if (settings.callback.onSelectAll) {
                        settings.callback.onSelectAll(e, $this.attr("id"), isChecked);
                    }
                }

                var $checkAllParent = $checkAll.parent();
                $checkAllParent.click(function (e) {
                    if (e.stopPropagation)
                        e.stopPropagation();
                    var isChecked = !$checkAll.is(":checked");
                    $checkAll.attr("checked", isChecked);
                    checkAllEvent(e, isChecked);
                });


                //获取鼠标相对于表格外框的左偏移
                function getMouseLeft(e) {
                    //鼠标相对于文档的坐标
                    var pos = widget.getMousePos(e), aj = -1; //校准偏移;
                    return pos.x - $this.offset().left + aj;
                }

                //计算table宽度--每次调整列宽后
                //计算table实际宽度（列宽之和），并设置grid-htable，grid-btable的宽度
                function getTableWidth() {
                    var w = 0, $elem;
                    $(".grid-htable th", $this).each(function (i, elem) {
                        w += colModel[i] ? (colModel[i].hidden ? 0 : colModel[i].width) : 0;
                    });
                    return w;
                }
            });
        },
        /**
        * 加载数据
        * @param newRows 行数组
        */
        load: function (newRows, startRowIndex) {
            startRowIndex = startRowIndex ? startRowIndex : 0;
            var $this = $(this);
            $this.data('WidgetData').settings.data.rowList = newRows;
            $this.data('WidgetData').dataSource = $.extend(true, [], newRows);
            $(".grid-btable tbody", $this).empty();
            $(".grid-btable tbody", $this).html(buildBodyHtml(this, startRowIndex));
            bindBodyEvent(this);
            if ($(".grid-status-col.loading,.grid-status-col.success,.grid-status-col.error", $this).length > 0) {
                $(".grid-status-col", $this).show();
            }
            else {
                $(".grid-status-col", $this).hide();
            }

            //解决IE9下不自动出现滚动条的BUG
            $this.hide();
            $this.show();
        },
        /**
        * 重置（用原始数据重新加载）
        */
        reset: function () {
            var $this = $(this);
            $this.data('WidgetData').settings.data.rowList = $.extend(true, [], $this.data('WidgetData').dataSource);
            $(".grid-btable tbody", $this).empty();
            $(".grid-btable tbody", $this).html(buildBodyHtml(this));
            bindBodyEvent(this);
        },
        /**
        * 获取数据(view数据映射)
        * @returns
        */
        getRowDatas: function () {
            return $(this).data('WidgetData').settings.data.rowList;
        },
        /**
        * 获取所有行数据
        * @returns
        */
        getAllRows: function () {
            var rowDatas = $(this).data('WidgetData').settings.data.rowList, rows = [];
            for (var i = 0; i < rowDatas.length; i++) {
                rows.push({ index: i, data: rowDatas[i] });
            }
            return rows;
        },
        /**
        * 根据序号查找单行数据
        * @param index
        * @returns 单行数据
        */
        getRowByIndex: function (index) {
            var rowDatas = $(this).wbGrid("getRowDatas");
            var r = null;
            if (index >= 0 && index < rowDatas.length)
                r = { index: index, data: rowDatas[index] };
            return r;
        },
        /**
        * 根据序号查找多行数据
        * @param indexes
        * @returns 多行数据
        */
        getRowsByIndexes: function (indexes) {
            var rowDatas = $(this).wbGrid("getRowDatas");
            var r = [];
            for (var i in indexes) {
                if (indexes[i] >= 0 && indexes[i] < rowDatas.length)
                    r.push({ index: indexes[i] * 1, data: rowDatas[indexes[i]] });
            }
            return r;
        },
        /**
        * 根据ID查找单行数据
        * @param id
        * @returns
        */
        getRowById: function (id) {
            var $this = $(this),
			keyNames = $this.data('WidgetData').privateProperty.keyNames;
            if (keyNames.length == 0)
                throw "no keys";
            if (keyNames.length == 1) {
                var rowDatas = $this.wbGrid("getRowDatas");
                for (var i in rowDatas) {
                    if (rowDatas[i][keyNames[0]] === id)
                        return { index: i, data: rowDatas[i] };
                }
            }
            else {
                for (var i in keyNames) {
                    if (!(keyNames[i] in id))
                        throw "id format error";
                }
                var rowDatas = $this.wbGrid("getRowDatas");
                for (var i in rowDatas) {
                    var isEqual = true;
                    for (var k in keyNames) {
                        if (id[keyNames[k]] !== rowDatas[i][keyNames[k]]) {
                            isEqual = false;
                            break;
                        }
                    }
                    if (isEqual)
                        return { index: i, data: rowDatas[i] };
                }
            }
        },
        /**
        * 根据ID数组查找多行数据
        * @param ids
        * @returns {Array}
        */
        getRowsByIds: function (rowIds) {
            var $this = $(this), result = [], ids = rowIds.slice(0);
            keyNames = $this.data('WidgetData').privateProperty.keyNames;
            if (keyNames.length == 0)
                throw "no keys";
            if (keyNames.length == 1) {
                var rowDatas = $this.wbGrid("getRowDatas");
                for (var i in rowDatas) {
                    if (ids.length == 0) break;
                    for (var j in ids) {
                        if (rowDatas[i][keyNames[0]] === ids[j]) {
                            result.push({ index: i * 1, data: rowDatas[i] });
                            ids.splice(j, 1);
                            break;
                        }
                    }
                }
            }
            else {
                var rowDatas = $(this).wbGrid("getRowDatas"), isEqual;
                for (var i in rowDatas) {
                    if (ids.length == 0) break;
                    for (var j in ids) {
                        isEqual = true;
                        for (var k in keyNames) {
                            if (ids[j][keyNames[k]] !== rowDatas[i][keyNames[k]]) {
                                isEqual = false;
                                break;
                            }
                        }
                        if (isEqual) {
                            result.push({ index: i * 1, data: rowDatas[i] });
                            ids.splice(j, 1);
                            break;
                        }
                    }
                }
            }
            return result;
        },
        /**
        * 根据行属性键值查找单行数据
        * @param key
        * @param value
        * @returns
        */
        getRowByColName: function (key, value) {
            var rowDatas = $(this).wbGrid("getRowDatas");
            for (var i in rowDatas) {
                if (rowDatas[i][key] === value)
                    return { index: i, data: rowDatas[i] };
            }
        },
        /**
        * 根据行属性键值查找多行数据
        * @param key
        * @param value
        * @returns
        */
        getRowsByColName: function (key, value) {
            var rowDatas = $(this).wbGrid("getRowDatas"), result = [];
            for (var i in rowDatas) {
                if (rowDatas[i][key] === value)
                    result.push({ index: i * 1, data: rowDatas[i] });
            }
            return result;
        },
        /**
        * 根据行属性键值列表查找单行数据
        * @param keys
        * @param values
        * @returns
        */
        getRowByColNames: function (keys, values) {
            var rowDatas = $(this).wbGrid("getRowDatas"), isMatch;
            for (var i in rowDatas) {
                isMatch = true;
                for (var j in keys) {
                    if (rowDatas[i][keys[j]] === values[j]) {
                    }
                    else {
                        isMatch = false;
                        break;
                    }
                }
                if (isMatch)
                    return { index: i, data: rowDatas[i] };
            }
        },
        /**
        * 根据行属性键值列表查找多行数据
        * @param key
        * @param value
        * @returns
        */
        getRowsByColNames: function (keys, values) {
            var rowDatas = $(this).wbGrid("getRowDatas"), result = [], isMatch;
            for (var i in rowDatas) {
                isMatch = true;
                for (var j in keys) {
                    if (rowDatas[i][keys[j]] === values[j]) {
                    }
                    else {
                        isMatch = false;
                        break;
                    }
                }
                if (isMatch)
                    result.push({ index: i * 1, data: rowDatas[i] });
            }
            return result;
        },
        /**
        * 通过过滤方法，过滤数据
        * bool:filter:function(elem) //elem为数组元素
        * 结果为filter函数返回true的第一个元素
        * 注意：性能较低，主要用于需要通过复杂的条件过滤数据时，
        * 如仅通过单个或多个列名对应值相等来筛选数据，建议使用getRowByColName，getRowsByColName，getRowByColNames，getRowsByColNames
        */
        getRowByFilter: function (filter) {
            var rowDatas = $(this).wbGrid("getRowDatas");
            for (var i in rowDatas) {
                if (filter(rowDatas[i]))
                    return { index: i, data: rowDatas[i] };
            }
        },
        /**
        * 通过过滤方法，过滤数据
        * bool:filter:function(elem) //elem为数组元素
        * 结果为filter函数返回true的元素集
        */
        getRowsByFilter: function (filter) {
            var rowDatas = $(this).wbGrid("getRowDatas"), result = [];
            for (var i in rowDatas) {
                if (filter(rowDatas[i]))
                    result.push({ index: i * 1, data: rowDatas[i] });
            }
            return result;
        },
        /**
        * 设置单元格的值
        * @param ids 修改的行id数组
        * @param colNames 修改的列名数组
        * @param colValues 修改的值数组
        * @param ignoreEditTigger 忽略修改行为，包括记录已修改行，和触发行修改事件
        */
        setCellsValueByIds: function (ids, colNames, colValues, ignoreEditTigger) {
            var $this = $(this), rows = $this.wbGrid("getRowsByIds", ids);
            setRowsCellsValue($this, rows, colNames, colValues, ignoreEditTigger);
        },
        /**
        * 设置单元格的值
        * @param indexes 修改的行索引数组
        * @param colNames 修改的列名数组
        * @param colValues 修改的值数组
        * @param ignoreEditTigger 忽略修改行为，包括记录已修改行，和触发行修改事件
        */
        setCellsValueByIndexes: function (indexes, colNames, colValues, ignoreEditTigger) {
            var $this = $(this), rows = $this.wbGrid("getRowsByIndexes", indexes);
            setRowsCellsValue($this, rows, colNames, colValues, ignoreEditTigger);
        },
        /**
        * 设置所有行的单元格值
        * @param colNames 修改的列名数组
        * @param colValues 修改的值数组
        * @param ignoreEditTigger 忽略修改行为，包括记录已修改行，和触发行修改事件
        */
        setCellsValueOfAllRows: function (colNames, colValues, ignoreEditTigger) {
            var $this = $(this), rows = $this.wbGrid("getAllRows");
            setRowsCellsValue($this, rows, colNames, colValues, ignoreEditTigger);
        },
        /**
        * 同步数据（仅更新内部数据，不会对界面产生影响）
        * 一般用在自定义控件改变值后调用该方法更新数据映射
        * @param id
        * @param colName
        * @param colValue
        * @param ignoreEditTigger 忽略修改行为，包括记录已修改行，和触发行修改事件
        */
        syncCellDataById: function (id, colName, colValue, ignoreEditTigger) {
            var row = $(this).wbGrid("getRowById", id);
            if (row) {
                syncCellValue(this, row, colName, colValue, ignoreEditTigger);
            }
        },
        /** 
        * 同步数据（仅更新内部数据，不会对界面产生影响）
        * 一般用在自定义控件改变值后调用该方法更新数据映射
        * @param target
        * @param colValue
        * @param ignoreEditTigger 忽略修改行为，包括记录已修改行，和触发行修改事件
        */
        syncCellDataByTarget: function (target, colValue, ignoreEditTigger) {
            var $cell = $(target).data("grid_cell"), $wd = $(this).data('WidgetData'),
			colIndexStart = $wd.privateProperty.colIndexStart, expandable = $wd.privateProperty.expandable,
			colModel = $wd.settings.data.colModel,
            ci = $cell[0].cellIndex - colIndexStart,
            $tr = $cell.closest("tr"),
			ri = getRowIndex(expandable, $tr[0]);
            var row = $(this).wbGrid("getRowByIndex", ri);
            if (row) {
                syncCellValue(this, row, colModel[ci]["name"], colValue, ignoreEditTigger);
            }
        },
        /**
        * 获得选中行id
        * @returns {Array}
        */
        getSelectIds: function () {
            var ids = [], $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			keyNames = $this.data('WidgetData').privateProperty.keyNames,
			trs = $(".grid-btable tr.grid-row.selected", this);
            if (multiselect) {
                trs.each(function (i, elem) {
                    ids.push(serailId2Id(keyNames, $(elem).attr("serial")));
                });
                return ids;
            }
            else {
                if (trs.length == 0)
                    return null;
                else
                    return serailId2Id(keyNames, trs.attr("serial"));
            }
        },
        /**
        * 获得选中行数据
        * @returns {Array}
        */
        getSelectRowDatas: function () {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			rowList = $this.wbGrid("getRowDatas"),
			rowDatas = [], expandable = $this.data('WidgetData').privateProperty.expandable,
			trs = $(".grid-btable tr.grid-row.selected", this);
            if (multiselect) {
                trs.each(function (i, elem) {
                    rowDatas.push(rowList[getRowIndex(expandable, elem)]);
                });
                return rowDatas;
            }
            else {
                if (trs.length == 0)
                    return null;
                else
                    return rowList[getRowIndex(expandable, trs[0])];
            }
        },
        /**
        * 获得选中行数据(带Index)
        * @returns {Array}
        */
        getSelectRows: function () {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			rowList = $this.wbGrid("getRowDatas"),
			rowDatas = [], expandable = $this.data('WidgetData').privateProperty.expandable,
			trs = $(".grid-btable tr.grid-row.selected", this);
            if (multiselect) {
                trs.each(function (i, elem) {
                    var index = getRowIndex(expandable, elem);
                    rowDatas.push({ index: index, data: rowList[index] });
                });
                return rowDatas;
            }
            else {
                if (trs.length == 0)
                    return null;
                else {
                    var index = getRowIndex(expandable, trs[0]);
                    return { index: index, data: rowList[index] };
                }
            }
        },
        /**
        * 获得修改过的行
        * @returns {Array}
        */
        getEditedRows: function () {
            var $this = $(this),
			rowList = $this.wbGrid("getRowDatas"),
			rowDatas = [], expandable = $this.data('WidgetData').privateProperty.expandable,
			trs = $(">.grid-bdiv>.grid-btable tr.grid-row.grid-row-edited", this);
            trs.each(function (i, elem) {
                rowDatas.push(rowList[getRowIndex(expandable, elem)]);
            });
            return rowDatas;
        },
        /**
        * 通过id选中单行(如果想取消选中行，可以传id=null)
        * @param id
        */
        selectRowById: function (id) {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			keyNames = $this.data('WidgetData').privateProperty.keyNames;
            if (multiselect) {
                $("input.grid-check-all", this).removeAttr("checked");
                $("input.grid-check", this).removeAttr("checked");
            }
            $(".grid-btable tr.grid-row", this).removeClass("selected");
            if (id == null) {

            }
            else {
                var serialId = (keyNames.length == 1 ? id : getRowSerialId(keyNames, id)),
				$tr = $(".grid-btable tr.grid-row[serial='" + serialId + "']", this);
                if (multiselect) {
                    $("input.grid-check", $tr).attr("checked", "checked");
                }
                $tr.addClass("selected");
            }

        },
        /**
        * 通过id选中多行（如果想取消所有选中行，可以传ids=[]）
        * @param ids
        */
        selectRowsByIds: function (ids) {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			keyNames = $this.data('WidgetData').privateProperty.keyNames, serialId, $tr;
            if (!multiselect) {
                throw "not multiselect";
            }
            $("input.grid-check-all", this).removeAttr("checked");
            $("input.grid-check", this).removeAttr("checked");
            $(".grid-btable tr.grid-row", this).removeClass("selected");
            for (var i in ids) {
                serialId = (keyNames.length == 1 ? ids[i] : getRowSerialId(keyNames, ids[i])),
				$tr = $(".grid-btable tr.grid-row[serial='" + serialId + "']", this);
                $("input.grid-check", $tr).attr("checked", "checked");
                $tr.addClass("selected");
            }
            if ($(".grid-bdiv input.grid-check:not(:checked)", this).length == 0)
                $("input.grid-check-all", this).attr("checked", "checked");
        },
        /**
        * 通过index选中单行(如果想取消选中行，可以传index=null)
        * @param id
        */
        selectRowByIndex: function (index) {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect
            rowDatas = $this.wbGrid("getRowDatas"),
			expandable = $this.data('WidgetData').privateProperty.expandable;
            if (multiselect) {
                $("input.grid-check-all", this).removeAttr("checked");
                $("input.grid-check", this).removeAttr("checked");
            }
            $(".grid-btable tr.grid-row", this).removeClass("selected");
            if (index == null) {

            }
            else {
                if (index >= 0 && index < rowDatas.length) {
                    var $trs = getTrByIndex($this, expandable, index), $elem;
                    $.each($trs, function (i, elem) {
                        $elem = $(elem);
                        if ($elem.hasClass("grid-row")) {
                            if (multiselect) {
                                $("input.grid-check", $elem).attr("checked", "checked");
                            }
                            $elem.addClass("selected");
                        }
                    });
                }
            }
        },
        /**
        * 通过id追加选中行
        * @param ids
        */
        appendSelectedRowsByIds: function (ids) {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			keyNames = $this.data('WidgetData').privateProperty.keyNames, serialId, $tr;
            if (!multiselect) {
                throw "not multiselect";
            }
            for (var i in ids) {
                serialId = (keyNames.length == 1 ? ids[i] : getRowSerialId(keyNames, ids[i])),
				$tr = $(".grid-btable tr.grid-row[serial='" + serialId + "']", this);
                $("input.grid-check", $tr).attr("checked", "checked");
                $tr.addClass("selected");
            }
            if ($(".grid-bdiv input.grid-check:not(:checked)", this).length == 0)
                $("input.grid-check-all", this).attr("checked", "checked");
        },
        /**
        * 通过id取消选中行
        * @param ids
        */
        cancelSelectedRowsByIds: function (ids) {
            var $this = $(this), multiselect = $this.data('WidgetData').settings.view.multiselect,
			keyNames = $this.data('WidgetData').privateProperty.keyNames, serialId, $tr;
            for (var i in ids) {
                serialId = (keyNames.length == 1 ? ids[i] : getRowSerialId(keyNames, ids[i])),
				$tr = $(".grid-btable tr.grid-row[serial='" + serialId + "']", this);
                $("input.grid-check", $tr).removeAttr("checked");
                $tr.removeClass("selected");
            }
            if ($(".grid-bdiv input.grid-check:not(:checked)", this).length == 0)
                $("input.grid-check-all", this).attr("checked", "checked");
        },
        /**
        * 获取单元格中的标准视图控件引用
        * @param id
        * @param colName 列名
        * @returns 控件对象
        */
        getStandardCellCtrl: function (id, colName) {
            var $this = $(this),
			pp = $this.data('WidgetData').privateProperty,
			keyNames = pp.keyNames, colIndexStart = pp.colIndexStart,
			colModelMap = getColModelMap($this),
			colModelItem = colModelMap[colName],
			serialId = (keyNames.length == 1 ? id : getRowSerialId(keyNames, id));
            var $td = $(".grid-row[serial='" + serialId + "']>td:eq(" + (colModelItem.colIndex + colIndexStart) + ")", $this);
            switch (colModelItem.formatter) {
                case "select":
                case "textbox":
                case "password":
                case "checkbox":
                    return $(".cell_input", $td);
                    break;
                case "iconBtn":
                    return $(".grid-iocn-btn", $td);
                    break;
                case "button":
                    return $(".grid-btn", $td);
                    break;
            }
        },
        /**
        * 获取单元格中的标准视图控件引用
        * @param index
        * @param colName 列名
        * @returns 控件对象
        */
        getStandardCellCtrlByIndex: function (index, colName) {
            var $this = $(this),
			pp = $this.data('WidgetData').privateProperty,
			keyNames = pp.keyNames, colIndexStart = pp.colIndexStart,
			colModelMap = getColModelMap($this),
			colModelItem = colModelMap[colName];
            var $td = $(">.grid-bdiv>.grid-btable .grid-row:eq(" + index + ")>td:eq(" + (colModelItem.colIndex + colIndexStart) + ")", $this);
            switch (colModelItem.formatter) {
                case "select":
                case "textbox":
                case "password":
                case "checkbox":
                    return $(".cell_input", $td);
                    break;
                case "iconBtn":
                    return $(".grid-iocn-btn", $td);
                    break;
                case "button":
                    return $(".grid-btn", $td);
                    break;
            }
        },
        /**
        * 获取单元格中的自定义控件引用
        * @param id
        * @param colName 列名
        * @returns 自定义控件对象
        */
        getWidget: function (id, colName) {
            var $this = $(this),
			pp = $this.data('WidgetData').privateProperty,
			keyNames = pp.keyNames, colIndexStart = pp.colIndexStart,
			colModelMap = getColModelMap($this),
			serialId = (keyNames.length == 1 ? id : getRowSerialId(keyNames, id));
            var $td = $(".grid-row[serial='" + serialId + "']>td:eq(" + (colModelMap[colName].colIndex + colIndexStart) + ")", $this);
            return $td.data("widget");
        },
        /**
        * 根据id获取展开容器
        * @param id
        * @returns 展开容器
        */
        getExpandPL: function (id) {
            var $this = $(this);
            //禁用时，收起子区域
            var expandable = $this.data('WidgetData').privateProperty.expandable;
            if (expandable) {
                var rowData = $this.wbGrid("getRowById", id),
				keyNames = $this.data('WidgetData').privateProperty.keyNames,
				$tr = $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData.data) + "\"]", $this);
                var subRow = $tr.next("tr.grid-expand-row");
                var pl = $("div.grid-expand-pl", subRow);
                return pl;
            }
            return null;
        },
        /**
        * 通过单元格内的控件获取所属行数据
        * @param cellInput 单元格内的控件（包括自定义控件）
        * @returns
        */
        getRowDataByCellInput: function (cellInput) {
            var $this = $(this), expandable = $this.data('WidgetData').privateProperty.expandable;
            var $tr = $(cellInput).closest("tr.grid-row");
            var rowIndex = getRowIndex(expandable, $tr[0]);
            var row = $this.wbGrid("getRowByIndex", rowIndex);
            return row;
        },
        /**
        * 插入行
        * @param rowData 行数据
        */
        insertRow: function (rowData) {
            var $this = $(this),
			$wd = $this.data('WidgetData'),
			keyNames = $wd.privateProperty.keyNames;
            if (keyNames.length > 0) {
                if ($(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData) + "\"]", $this).length > 0) {
                    throw 'id already exist';
                    return;
                }
            }
            var rowDatas = $this.wbGrid("getRowDatas");
            rowDatas.push(rowData);
            var $tr = $(buildRowHtml(rowData, rowDatas.length - 1, $wd));
            $tr.appendTo($(".grid-btable>tbody", $this));
            bindBodyEvent(this, $tr, true);
        },
        /**
        * 在指定位置插入行(如果是树表时，一次加入的数据应该是同一个节点的子节点)
        * @param index 插入位置索引
        * @param addRowDatas 行数据
        */
        insertRowInPosition: function (index, addRowDatas) {
            var $this = $(this),
			$wd = $this.data('WidgetData'),
			keyNames = $wd.privateProperty.keyNames;
            if (keyNames.length > 0) {
                for (var i in addRowDatas) {
                    if ($(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, addRowDatas[i]) + "\"]", $this).length > 0) {
                        throw 'id already exist';
                        return;
                    }
                }
            }
            var rowDatas = $this.wbGrid("getRowDatas");
            var rowDataLength = rowDatas.length;
            var trsHtml = "";
            for (var i = 0, i_count = addRowDatas.length; i < i_count; i++) {
                rowDatas.splice(index + i, 0, addRowDatas[i]);
                trsHtml += buildRowHtml(addRowDatas[i], 1, $wd);
            }
            var $trs = $(trsHtml);
            if (index < rowDataLength) {
                var $tagetTr = getTrByIndex($this, $wd.privateProperty.expandable, index);
                $trs.insertBefore($($tagetTr[0]));
            }
            else {
                $trs.appendTo($(".grid-btable>tbody", $this));
            }
            $trs.each(function (i, elem) {
                bindBodyEvent($this, $(elem), true);
            });
            resetAlterRow($this);
            //如果是树表，要根据父节点来设置新加入行的选中状态
            if ($wd.settings.data.treeGrid) {
                var parentNodeId = addRowDatas[0][$wd.settings.data.treeReader.parent_id_field];
                var parentNode = $this.wbGrid("getRowById", parentNodeId);
                var $tr = getTrByIndex($this, $wd.privateProperty.expandable, parentNode["index"]);
                if ($(".grid-tree-check", $tr[0]).is(":checked")) {
                    $trs.each(function (i, elem) {
                        $("input.grid-tree-check", elem).attr("checked", "checked");
                        $(elem).addClass("selected");
                    });
                }
            }
        },
        /**
        * 通过id删除单行
        * @param id
        * @returns
        */
        deleteRowById: function (id) {
            var $this = $(this),
			rowData = $this.wbGrid("getRowById", id);
            if (!rowData)
                return;
            var rowDatas = $this.wbGrid("getRowDatas"),
			keyNames = $this.data('WidgetData').privateProperty.keyNames;
            $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData.data) + "\"]", $this).remove();
            rowDatas.splice(rowData.index, 1);
            resetAlterRow($this);
            resetRowNumber($this);
        },
        /**
        * 通过id删除多行
        * @param ids
        */
        deleteRowsByIds: function (ids) {
            var $this = $(this), rowDatas = $this.wbGrid("getRowDatas"),
			keyNames = $this.data('WidgetData').privateProperty.keyNames;
            for (var i = 0; i < ids.length; i++) {
                rowData = $this.wbGrid("getRowById", ids[i]);
                if (!rowData)
                    continue;
                $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData.data) + "\"]", $this).remove();
                rowDatas.splice(rowData.index, 1);
            }
            resetAlterRow($this);
            resetRowNumber($this);
        },
        /**
        * 通过index删除单行
        * @param index
        * @returns
        */
        deleteRowByIndex: function (index) {
            var $this = $(this),
			rowDatas = $this.wbGrid("getRowDatas"),
			expandable = $this.data('WidgetData').privateProperty.expandable;
            if (index >= 0 && index < rowDatas.length) {
                rowDatas.splice(index, 1);
                var $trs = getTrByIndex($this, expandable, index);
                for (var j in $trs)
                    $trs[j].remove();
                resetAlterRow($this);
                resetRowNumber($this);
            }
        },
        /**
        * 通过indexes删除多行
        * @param indexes
        */
        deleteRowsByIndexes: function (indexes) {
            var $this = $(this), rowDatas = $this.wbGrid("getRowDatas"),
			expandable = $this.data('WidgetData').privateProperty.expandable;
            indexes.sort(function (a, b) {
                return b - a;
            });
            for (var i = 0; i < indexes.length; i++) {
                if (indexes[i] >= 0 && indexes[i] < rowDatas.length) {
                    rowDatas.splice(indexes[i], 1);
                    var $trs = getTrByIndex($this, expandable, indexes[i]);
                    for (var j in $trs)
                        $trs[j].remove();
                }
            }
            resetAlterRow($this);
            resetRowNumber($this);
        },
        /**
        * 设置宽度
        * @param w
        */
        setWidth: function (w) {
            var $this = $(this);
            $this.data('WidgetData').settings.view.width = w;
            $(">.grid-hdiv,>.grid-bdiv", this).width(w);
            $this.width(w);
            return $this;
        },
        /**
        * 设置高度
        * @param h
        */
        setHeight: function (h) {
            var $this = $(this);
            $this.data('WidgetData').settings.view.height = h;
            $(">.grid-bdiv", $this).height(h);
            return $this;
        },
        /**
        * 刷新展开行的子数据
        * @param rowIndex
        */
        refreshExpandData: function (rowIndex) {
            var $this = $(this),
			trs = getTrByIndex($this, true, rowIndex),
			colModel = $this.data('WidgetData').settings.data.colModel,
    		expandableColIndex = $this.data('WidgetData').privateProperty.expandableColIndex,
            fp = colModel[expandableColIndex].formatterParam;
            var pl = $("div.grid-expand-pl", trs[1]);
            pl.empty();
            var subData = $this.wbGrid("getRowByIndex", rowIndex)["data"][colModel[expandableColIndex]["name"]]["data"];
            var subHtml = getExpandSubPlHtml(subData, fp);
            pl.append(subHtml);
            bindExpandSubPlEvent(this, pl);
        },
        /**
        * 通过id启用和禁用行
        * @param id
        * @param disabled
        */
        setRowDisableStatusById: function (id, disabled) {
            var $this = $(this), rowData = $this.wbGrid("getRowById", id),
			keyNames = $this.data('WidgetData').privateProperty.keyNames,
			$tr = $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData.data) + "\"]", $this);
            setRowDisableStatus(rowData, $tr, this, disabled);
        },
        /**
        * 通过序号启用和禁用行
        * @param index
        * @param disabled
        */
        setRowDisableStatusByIndex: function (index, disabled) {
            var $this = $(this), rowData = $this.wbGrid("getRowByIndex", index),
			expandable = $this.data('WidgetData').privateProperty.expandable,
			$trs = getTrByIndex($this, expandable, index);
            setRowDisableStatus(rowData, $($trs[0]), this, disabled);
        },
        /**
        * 通过id设置行状态
        * @param id
        * @param disabled
        */
        setRowLogicStatusById: function (id, status, toolTip) {
            var $this = $(this), rowData = $this.wbGrid("getRowById", id),
			keyNames = $this.data('WidgetData').privateProperty.keyNames,
			$tr = $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData.data) + "\"]", $this);
            setRowLogicStatus(rowData, $tr, this, status, toolTip);
        },
        /**
        * 通过序号设置行状态
        * @param index
        * @param disabled
        */
        setRowLogicStatusByIndex: function (index, status, toolTip) {
            var $this = $(this), rowData = $this.wbGrid("getRowByIndex", index),
			expandable = $this.data('WidgetData').privateProperty.expandable,
			$trs = getTrByIndex($this, expandable, index);
            setRowLogicStatus(rowData, $($trs[0]), this, status, toolTip);
        },
        /**
        * 清空行状态
        */
        clearRowLogicStatus: function () {
            var $this = $(this), rowData = $this.wbGrid("getRowDatas");
            for (var i in rowData)
                rowData["grid-row-status"] = "";
            $(".grid-status-col", $this).removeClass("loading success error");
            $(".grid-status-col", $this).hide();
        },
        /**
        * 清空已修改行的标识
        */
        clearRowEditedStatus: function () {
            $("tr.grid-row", this).removeClass("grid-row-edited");
        },
        /**
        * 通过id清除行的已修改标识
        */
        removeRowEditedStatusById: function (id) {
            var $this = $(this), rowData = $this.wbGrid("getRowById", id);
            if (rowData) {
                keyNames = $this.data('WidgetData').privateProperty.keyNames,
				$tr = $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData["data"]) + "\"]", $this);
                $tr.removeClass("grid-row-edited");
            }
        },
        /**
        * 通过id设置行的已修改标识
        */
        setRowEditedStatusById: function (id) {
            var $this = $(this), rowData = $this.wbGrid("getRowById", id);
            if (rowData) {
                keyNames = $this.data('WidgetData').privateProperty.keyNames,
				$tr = $(".grid-btable tr[serial=\"" + getRowSerialId(keyNames, rowData["data"]) + "\"]", $this);
                $tr.addClass("grid-row-edited");
            }
        },

        /***************************************************
        *	树表方法
        ***************************************************/

        /**
        * 获取父节点
        */
        getNodeParent: function (rowData) {
            var $this = $(this);
            var parent_id = $this.data('WidgetData').settings.data.treeReader.parent_id_field;
            if (rowData[parent_id] == "null")
                return null;
            var rowData = $this.wbGrid("getRowById", rowData[parent_id]);
            return rowData;
        },
        /**
        * 获取子节点
        */
        getNodeChildren: function (rowData) {
            var result = [], $this = $(this);
            var parent_id = $this.data('WidgetData').settings.data.treeReader.parent_id_field;
            var allRowDatas = $this.data('WidgetData').settings.data.rowList;
            var keyNames = $this.data('WidgetData').privateProperty.keyNames;
            if (keyNames.length == 0)
                throw "no id";
            else if (keyNames.length == 1) {
                $.each(allRowDatas, function (i, elem) {
                    if (elem[parent_id] == rowData[keyNames[0]]) {
                        result.push({ index: i, data: elem });
                    }
                });
            }
            else {
                $.each(allRowDatas, function (i, elem) {
                    var add = true, parentId = elem[parent_id];
                    for (var p in keyNames) {
                        if (parentId[keyNames[p]] != rowData[keyNames[p]]) {
                            add = false;
                            break;
                        }
                    }
                    if (add) {
                        result.push({ index: i, data: elem });
                    }
                });
            }
            return result;
        },
        /**
        * 通过Index获取子节点
        */
        getNodeChildrenByIndex: function (index) {
            var $this = $(this);
            var rowData = $this.wbGrid("getRowByIndex", index);
            return $this.wbGrid("getNodeChildren", rowData["data"]);
        },
        /**
        * 展开行
        */
        expandRow: function (index) {
            var $this = $(this);
            var $wd = $this.data('WidgetData');
            var rowData = $this.wbGrid("getRowByIndex", index);
            if ($wd.settings.callback.onBeforeTreeExpand) {
                if (!$wd.settings.callback.onBeforeTreeExpand($this.attr("id"), index, rowData["data"]))
                    return;
            }
            var expandable = $wd.privateProperty.expandable;
            showTreeChildren(index, $this, expandable);
            var $tr = getTrByIndex($this, expandable, index);
            $(".tree-click", $tr[0]).removeClass("tree-plus").addClass("tree-minus");
            rowData["data"][$wd.settings.data.treeReader.expanded_field] = true;
        },
        /**
        * 收起行
        */
        collapseRow: function (index) {
            var $this = $(this);
            var $wd = $this.data('WidgetData');
            var rowData = $this.wbGrid("getRowByIndex", index);
            if ($wd.settings.callback.onBeforeTreeCollapse) {
                if (!$wd.settings.callback.onBeforeTreeCollapse($this.attr("id"), index, rowData["data"]))
                    return;
            }
            var expandable = $wd.privateProperty.expandable;
            hideTreeChildren(index, $this, expandable);
            var $tr = getTrByIndex($this, expandable, index);
            $(".tree-click", $tr[0]).removeClass("tree-minus").addClass("tree-plus");
            rowData["data"][$wd.settings.data.treeReader.expanded_field] = false;
        }
    };

    function showTreeChildren(index, $tree, expandable) {
        var rowData = $tree.wbGrid("getRowByIndex", index);
        var children = $tree.wbGrid("getNodeChildren", rowData["data"]);
        $.each(children, function (i, elem) {
            var trs = getTrByIndex($tree, expandable, elem["index"]);
            $.each(trs, function (j, tr) {
                $(tr).show();
            });
            if ($(".tree-click", trs[0]).hasClass("tree-minus")) {
                showTreeChildren(elem["index"], $tree, expandable);
            }
        });
    }

    function hideTreeChildren(index, $tree, expandable) {
        var rowData = $tree.wbGrid("getRowByIndex", index);
        var children = $tree.wbGrid("getNodeChildren", rowData["data"]);
        $.each(children, function (i, elem) {
            var trs = getTrByIndex($tree, expandable, elem["index"]);
            $.each(trs, function (j, tr) {
                $(tr).hide();
            });
            hideTreeChildren(elem["index"], $tree, expandable);
        });
    }

    //获取输入控件的值
    function getInputValue($input) {
        if ($input.is(":checkbox")) {
            return $input.is(":checked");
        }
        else {
            return $input.val();
        }
    }

    //构建行HTML
    function buildRowHtml(rowData, index, $wd) {
        var colModel = $wd.settings.data.colModel,
		colStyleMap = $wd.privateProperty.colStyleMap,
		colIndexStart = $wd.privateProperty.colIndexStart,
		keyNames = $wd.privateProperty.keyNames,
		altRow = $wd.settings.view.altRow,
		expandable = $wd.privateProperty.expandable,
		rownumbers = $wd.settings.view.rownumbers,
		selectRow = $wd.settings.view.selectRow,
		multiselect = $wd.settings.view.multiselect,
		inputLenFix = $wd.privateProperty.inputLenFix,
        expandSubPlHtml = "", expand = false, serialId = keyNames.length > 0 ? getRowSerialId(keyNames, rowData) : null;
        var treeGrid = $wd.settings.data.treeGrid;
        var treeExpandColumn = $wd.settings.data.treeExpandColumn;
        var treeReader = $wd.settings.data.treeReader;

        var isRowHidden = false;
        if (treeGrid) {
            var parentNode = $wd.target.wbGrid("getNodeParent", rowData);
            if (parentNode && !parentNode["data"][treeReader.expanded_field]) {
                isRowHidden = true;
            }
        }
        var initHtml = "<tr class=\"grid-row" + (altRow && index % 2 == 1 ? " grid-altrow" : "") + (rowData["grid-row-disabled"] ? " grid-row-disabled" : "") + "\" " + (serialId ? "serial=\"" + serialId + "\"" : "") + (isRowHidden ? " style=\"display:none;\"" : "") + ">";
        initHtml += "<td class=\"grid-status-col " + (rowData["grid-row-status"] ? rowData["grid-row-status"] : "") + "\" title=\"" + (rowData["grid-row-status-init-toolTip"] ? rowData["grid-row-status-init-toolTip"] : "") + "\"  ></td>";
        if (rownumbers) {
            initHtml += "<td class='grid-rownumber-col'>" + (index + 1) + "</td>";
        }
        if (selectRow && multiselect) {
            initHtml += "<td class='grid-muti-col'><input type='checkbox' class='grid-check' /></td>";
        }
        //展开列偏移
        var wrapfix = 0;
        //列合并
        var colSpan = 0;
        for (var j = 0; j < colModel.length; j++) {
            if (colSpan > 1) {
                colSpan--;
                continue;
            }
            //格式化显示
            if (colModel[j].formatter) {
                initHtml += "<td style=\"" + colStyleMap[j] + "\">";
                switch (colModel[j].formatter) {
                    //下拉框显示                                                                                                                     
                    case "select":
                        {
                            var param = colModel[j].formatterParam;
                            if (!param) {
                                throw colModel[j].name + ' need formatterParam';
                            }
                            try {
                                var selItems;
                                if (param.getItems) {
                                    selItems = param.getItems(rowData);
                                }
                                else {
                                    selItems = param.items;
                                }
                                var optionHtml = "", displayText = "", isSel = "";
                                $.each(selItems, function (k, elem) {
                                    if (elem.value === rowData[colModel[j].name]) {
                                        displayText = elem.text;
                                        isSel = " selected='selected'";
                                    }
                                    else {
                                        isSel = "";
                                    }
                                    optionHtml += "<option value=\"" + elem.value + "\" " + isSel + " title=\"" + elem.text + "\">" + elem.text + "</option>";
                                });
                                initHtml += (param.mode != "always" ? "<span class='cell_span'>" + displayText + "</span>" : "") +
								"<select" + (rowData["grid-row-disabled"] ? " disabled=\"disabled\"" : "") + " class='cell_input" + (param.mode == "always" ? " always" : "") + "' style='width:" + (colModel[j].width + inputLenFix) + "px;" + (param.mode != "always" ? "display:none;" : "") + "'>" +
								optionHtml +
								"</select></td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //文本框显示,密码框                                                                                                                     
                    case "textbox":
                    case "password":
                        {
                            var param = colModel[j].formatterParam;
                            if (!param) {
                                throw colModel[j].name + ' need formatterParam';
                            }
                            try {
                                var type = (colModel[j].formatter == "password" ? "password" : "text");
                                var val = rowData[colModel[j].name];
                                initHtml += (param.mode != "always" ? "<span class='cell_span'>" + (type == "password" ? "******" : val) + "</span>" : "") +
								"<input" + (rowData["grid-row-disabled"] ? " disabled=\"disabled\"" : "") + " type='" + type + "' class='cell_input" + (param.mode == "always" ? " always" : "") + "' style='width:" + (colModel[j].width + inputLenFix) + "px;" + (param.mode != "always" ? "display:none;" : "") + "' value='" + val + "'" + (param.maxlength ? "maxlength='" + param.maxlength + "'" : "") + " /></td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //图标按钮显示                                                                                                                     
                    case "iconBtn":
                        {
                            var param = colModel[j].formatterParam;
                            if (!param) {
                                throw colModel[j].name + ' need formatterParam';
                            }
                            try {
                                var val = rowData[colModel[j].name];
                                initHtml += "<div class=\"grid-iocn-btn " + param.iconClass + (rowData["grid-row-disabled"] ? " disabled" : "") + "\"" + (val && val.hidden ? " style=\"display:none;\"" : "") + "></div></td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //按钮显示                                                                                                                     
                    case "button":
                        {
                            var param = colModel[j].formatterParam;
                            if (!param) {
                                throw colModel[j].name + ' need formatterParam';
                            }
                            try {
                                var val = rowData[colModel[j].name];
                                initHtml += "<input" + (rowData["grid-row-disabled"] ? " disabled=\"disabled\"" : "") + " class=\"grid-btn" + (param.btnClass ? " " + param.btnClass : "") + "\" type=\"button\" value=\"" + param.text + "\"" + (val && val.hidden ? " style=\"display:none;\"" : "") + " /></td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //可展开列显示                                                                                                                     
                    case "expandable":
                        {
                            var param = colModel[j].formatterParam;
                            if (!param) {
                                throw colModel[j].name + ' need formatterParam';
                            }
                            try {
                                if (!param.mode) param.mode = "standard";
                                if (!param.maxTextDispolayLen) param.maxTextDispolayLen = 100;
                                var subData = (rowData[colModel[j].name] ? rowData[colModel[j].name] : rowData[colModel[j].name] = { expand: false, data: [] }),
								expand = subData.expand;
                                initHtml += "<div class=\"grid-expandable" + (subData.expand ? " open" : "") + "\"></div></td>";
                                if (param.mode == "standard") {
                                    expandSubPlHtml = getExpandSubPlHtml(subData.data, param);
                                }
                                else {
                                    expandSubPlHtml = "";
                                }
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //CheckBox显示                                                                                                                     
                    case "checkbox":
                        {
                            try {
                                initHtml += "<input" + (rowData["grid-row-disabled"] ? " disabled=\"disabled\"" : "") + " type='checkbox' class='cell_input always'" + (rowData[colModel[j].name] ? " checked='checked'" : "") + " /></td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                    //自定义控件显示                                                                                                                     
                    case "custom":
                        {
                            try {
                                initHtml += "</td>";
                            }
                            catch (ex) {
                                throw ex;
                            }
                        }
                        break;
                }
            }
            //普通显示
            else {
                var strContent = colModel[j]["formatDisplay"] ? colModel[j]["formatDisplay"](rowData[colModel[j].name], rowData) : rowData[colModel[j].name];
                if (treeGrid && colModel[j].name == treeExpandColumn) {
                    if (rowData[treeReader.expanded_field_colspan] && rowData[treeReader.expanded_field_colspan] > 1) {
                        colSpan = rowData[treeReader.expanded_field_colspan];
                        initHtml += "<td class=\"grid-general-text grid-tree-expand-col\" style=\"" + colStyleMap[j] + "\" title=\"" + strContent + "\"  colspan=\"" + colSpan + "\">";
                    }
                    else {
                        initHtml += "<td class=\"grid-general-text grid-tree-expand-col\" style=\"" + colStyleMap[j] + "\" title=\"" + strContent + "\">";
                    }
                    wrapfix = $wd.privateProperty.TreeWrapWidthBase * rowData[treeReader.level_field] * 1;
                    initHtml += "<div class=\"grid-tree-wrap\" style=\"width:" + (18 + wrapfix) + "px;\">";
                    if (rowData[treeReader.leaf_field]) {
                        initHtml += "<div class=\"tree-click tree-leaf\" style=\"left:" + wrapfix + "px;\"></div>";
                    }
                    else {
                        if (rowData[treeReader.expanded_field]) {
                            initHtml += "<div class=\"tree-click tree-minus\" style=\"left:" + wrapfix + "px;\"></div>";
                        }
                        else {
                            initHtml += "<div class=\"tree-click tree-plus\" style=\"left:" + wrapfix + "px;\"></div>";
                        }
                    }
                    initHtml += "</div>";
                    if ($wd.settings.view.treemultiselect) {
                        initHtml += "<input type=\"checkbox\" class=\"grid-tree-check\" />";
                    }
                }
                else {
                    initHtml += "<td class=\"grid-general-text\" style=\"" + colStyleMap[j] + "\" title=\"" + strContent + "\">";
                }
                initHtml += "<span>" + strContent + "</span></td>";
            }
        }
        initHtml += "</tr>";
        if (expandable) {
            initHtml += "<tr class=\"grid-expand-row\" " + (expand ? "" : "style='display:none;'") + " serial=\"" + serialId + "\"><td colspan='" + (colModel.length + colIndexStart) + "'><div class=\"grid-expand-pl\">" + expandSubPlHtml + "</div></td></tr>";
        }
        return initHtml;
    }

    //构建表体HTML
    function buildBodyHtml(grid, startRowIndex) {
        startRowIndex = startRowIndex ? startRowIndex : 0;
        var $wd = $(grid).data('WidgetData');
        var initHtml = "", rowList = $wd.settings.data.rowList;
        for (var i = 0; i < rowList.length; i++) {
            initHtml += buildRowHtml(rowList[i], startRowIndex + i, $wd);
        }
        return initHtml;
    }
    //绑定表体事件
    function bindBodyEvent(grid, context, isInsert) {
        var $this = $(grid), settings = $this.data('WidgetData').settings,
		pp = $this.data('WidgetData').privateProperty,
		colModel = settings.data.colModel,
		rowList = settings.data.rowList,
		context = context ? context : $this,
		expandable = pp.expandable,
		colIndexStart = pp.colIndexStart,
		keyNames = pp.keyNames;

        //单元格输入框blur事件--mousedown模式
        function cell_input_eblur(e) {
            var $input = $(e.target),
			$td = $input.closest("td"), $tr = $input.closest("tr"),
			$span = $("span.cell_span", $td),
        	m = colModel[$td[0].cellIndex - colIndexStart];
            $input.hide();
            $span.html(getDisplayText($input)).show();
            //更新对应行数据
            var rowIndex = getRowIndex(expandable, $tr[0]), newVal = getInputValue($input);
            if (rowList[rowIndex][m.name] == newVal || newVal === null || newVal === undefined) {

            }
            else {
                rowList[rowIndex][m.name] = newVal;
                //触发cellValueChanged
                $tr.addClass("grid-row-edited");
                if (settings.callback.cellValueChanged) {
                    settings.callback.cellValueChanged($this.attr("id"), rowIndex, rowList[rowIndex], m.name);
                }
            }
        }
        //单元格输入框blur事件--always模式
        function cell_input_eblur1(e) {
            var $input = $(e.target),
			$td = $input.closest("td"), $tr = $input.closest("tr"),
        	m = colModel[$td[0].cellIndex - colIndexStart];
            //更新对应行数据
            var rowIndex = getRowIndex(expandable, $tr[0]), newVal = getInputValue($input);
            if (rowList[rowIndex][m.name] == newVal || newVal === null || newVal === undefined) {

            }
            else {
                rowList[rowIndex][m.name] = newVal;
                //触发cellValueChanged
                $tr.addClass("grid-row-edited");
                if (settings.callback.cellValueChanged) {
                    settings.callback.cellValueChanged($this.attr("id"), rowIndex, rowList[rowIndex], m.name);
                }
            }
        }

        //单元格基础事件
        var $cells, $cell_inputs;
        if (isInsert) {
            $cells = $("td", context.filter(".grid-row"));
            $cell_inputs = $(".cell_input.always", context.filter(".grid-row"));
        }
        else {
            $cells = $(".grid-row td", $this);
            $cell_inputs = $(".cell_input.always", $this);
        }

        //所有显示的（always）模式的输入框，绑定blur事件
        $cell_inputs.blur(cell_input_eblur1);

        $cells.mousedown(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            $(".grid-col-sel-list", $this).hide();
            var $td = $(this);
            var m = colModel[this.cellIndex - colIndexStart];
            if (!m) return;
            var $tr = $td.closest("tr");
            //所在行为选中状态
            $(".grid-row", $this).removeClass("selected");
            $tr.addClass("selected");
            if (m.formatter == "select" || m.formatter == "textbox" || m.formatter == "password" || m.formatter == "checkbox") {
                var $input = $(".cell_input", this);
                if (m.formatterParam && m.formatterParam.mode != "always") {
                    var $span = $("span.cell_span", this);
                    $span.hide();
                    $input.width(m.width - 10).show();
                    $input.unbind('blur', cell_input_eblur);
                    $input.blur(cell_input_eblur);
                    window.setTimeout(function () {
                        $input.focus();
                    }, 0);
                }
                else {
                }
            }
            else if (m.formatter == "custom") {

            }
        });

        $cells.click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            var $td = $(this);

            if ($td.hasClass("grid-muti-col") || (settings.data.treeExpandColClick && $td.hasClass("grid-tree-expand-col")))
                return;

            var $tr = $td.closest("tr");
            if (settings.view.selectRow) {
                //所在行为选中状态
                $(".grid-row", $this).removeClass("selected");
                $tr.addClass("selected");
                //处理多选
                if (settings.view.multiselect) {
                    $("input.grid-check-all", $this).removeAttr("checked");
                    $("input.grid-check", $this).removeAttr("checked");
                    $("input.grid-check", $tr).attr("checked", "checked");
                }
                if (settings.view.treemultiselect) {
                    $("input.grid-tree-check", $this).removeAttr("checked");
                    $("input.grid-tree-check", $tr).attr("checked", "checked");
                    var rowIndex = getRowIndex(expandable, $tr[0]);
                    var rowData = $this.wbGrid("getRowByIndex", rowIndex);
                    //处理后代节点
                    selectDescendant(rowData, true);
                    //处理祖先节点
                    selectAncestor(rowData, true);
                }
            }

            //单元格单击事件
            if (settings.callback.onClickCell) {
                var rowIndex = getRowIndex(expandable, $td.closest("tr")[0]);
                settings.callback.onClickCell(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            }

            //行单击事件
            if (settings.callback.onClickRow) {
                var rowIndex = getRowIndex(expandable, $td.closest("tr")[0]);
                settings.callback.onClickRow(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            }
        });

        var treeReader = settings.data.treeReader;
        //递归选择后代
        function selectDescendant(rowData, isChecked) {
            var children = $this.wbGrid("getNodeChildren", rowData["data"]);
            $.each(children, function (i, elem) {
                var trs = getTrByIndex($this, expandable, elem["index"]);
                $.each(trs, function (j, tr) {
                    $("input.grid-tree-check", tr).attr("checked", isChecked);
                    if (isChecked) {
                        $(tr).addClass("selected");
                    }
                    else {
                        $(tr).removeClass("selected");
                    }
                });
                if (!elem["data"][treeReader.leaf_field]) {
                    selectDescendant(elem, isChecked);
                }
            });
        }
        //递归处理祖先的选中状态
        function selectAncestor(rowData, isChecked) {
            if (rowData["data"][treeReader.parent_id_field] == "null")
                return;

            var parentRowData = $this.wbGrid("getRowById", rowData["data"][treeReader.parent_id_field]);
            var trs = getTrByIndex($this, expandable, parentRowData["index"]);
            if (isChecked) {
                var children = $this.wbGrid("getNodeChildren", parentRowData["data"]);
                var parentChecked = true;
                $.each(children, function (i, elem) {
                    var childrenTrs = getTrByIndex($this, expandable, elem["index"]);
                    if ($(childrenTrs[0]).hasClass("selected")) {
                        return true;
                    }
                    else {
                        parentChecked = false;
                        return false;
                    }
                });

                if (parentChecked) {
                    $.each(trs, function (j, tr) {
                        $("input.grid-tree-check", tr).attr("checked", isChecked);
                        $(tr).addClass("selected");
                    });
                    selectAncestor(parentRowData, isChecked);
                }

            }
            else {
                $.each(trs, function (j, tr) {
                    $("input.grid-tree-check", tr).attr("checked", isChecked);
                    $(tr).removeClass("selected");
                });
                selectAncestor(parentRowData, isChecked);
            }
        }

        //树表事件
        if (settings.data.treeGrid) {
            //树展开事件
            var $expandTarget;
            if (settings.data.treeExpandColClick) {
                if (isInsert) {
                    $expandTarget = $("td.grid-tree-expand-col", context.filter(".grid-row"));
                }
                else {
                    $expandTarget = $("td.grid-tree-expand-col", $this);
                }
            }
            else {
                if (isInsert) {
                    $expandTarget = $("div.tree-click", context.filter(".grid-row"));
                }
                else {
                    $expandTarget = $("div.tree-click", $this);
                }
            }
            $expandTarget.click(function (e) {
                if (e.stopPropagation)
                    e.stopPropagation();
                var $wrap;
                if (settings.data.treeExpandColClick) {
                    $wrap = $(".tree-click", this);
                }
                else {
                    $wrap = $(this);
                }
                var rowIndex = getRowIndex(expandable, $(this).closest("tr")[0]);
                if ($wrap.hasClass("tree-minus")) {
                    $this.wbGrid("collapseRow", rowIndex);
                }
                else {
                    $this.wbGrid("expandRow", rowIndex);
                }
            });

            if (settings.view.treemultiselect) {
                //树多选事件
                var $treeCheck;
                if (isInsert) {
                    $treeCheck = $("input.grid-tree-check", context.filter(".grid-row"));
                }
                else {
                    $treeCheck = $("input.grid-tree-check", $this);
                }

                $treeCheck.click(function (e) {
                    if (e.stopPropagation)
                        e.stopPropagation();
                    var $tr = $(this).closest("tr");
                    var rowIndex = getRowIndex(expandable, $tr[0]);
                    var isChecked = false;
                    if ($(this).is(":checked")) {
                        $tr.addClass("selected");
                        isChecked = true;
                    }
                    else {
                        $tr.removeClass("selected");
                    }
                    var rowData = $this.wbGrid("getRowByIndex", rowIndex);
                    //处理后代节点
                    selectDescendant(rowData, isChecked);
                    //处理祖先节点
                    selectAncestor(rowData, isChecked);
                });
            }
        }

        //单元格双击事件
        if (settings.callback.onDblClickCell) {
            $cells.dblclick(function (e) {
                if (e.stopPropagation)
                    e.stopPropagation();
                var rowIndex = getRowIndex(expandable, $(this).closest("tr")[0]);
                settings.callback.onDblClickCell(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            });
        }

        //行双击事件
        if (settings.callback.onDblClickRow) {
            $cells.dblclick(function (e) {
                if (e.stopPropagation)
                    e.stopPropagation();
                var rowIndex = getRowIndex(expandable, $(this).closest("tr")[0]);
                settings.callback.onDblClickRow(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            });
        }

        //图标按钮点击事件,按钮点击事件
        $("div.grid-iocn-btn,input.grid-btn", context).click(function (e) {
            //            if (e.stopPropagation)
            //                e.stopPropagation();
            var $target = $(this);
            if ($target.hasClass("disabled"))
                return false;
            var ci = $target.closest("td")[0].cellIndex - colIndexStart;
            var ri = getRowIndex(expandable, $target.closest("tr")[0]);
            colModel[ci].formatterParam.click(e, $this.attr("id"), ri, rowList[ri]);
        });

        //下拉框值改变事件--特殊处理
        $("select.cell_input", context).change(function (e) {
            var $input = $(e.target),
			$td = $input.closest("td"), $tr = $input.closest("tr"),
        	m = colModel[$td[0].cellIndex - colIndexStart];
            //更新对应行数据
            var rowIndex = getRowIndex(expandable, $tr[0]), newVal = getInputValue($input);
            rowList[rowIndex][m.name] = newVal;
            //触发cellValueChanged
            $tr.addClass("grid-row-edited");
            if (settings.callback.cellValueChanged) {
                settings.callback.cellValueChanged($this.attr("id"), rowIndex, rowList[rowIndex], m.name);
            }
        });

        for (var i = 0; i < colModel.length; i++) {
            //基础控件自定义事件
            if ((colModel[i].formatter == "textbox" || colModel[i].formatter == "password"
        		|| colModel[i].formatter == "checkbox" || colModel[i].formatter == "select")
        		&& colModel[i].formatterParam && colModel[i].formatterParam.events
        		&& colModel[i].formatterParam.events.length > 0) {
                var cellInputs;
                if (isInsert) {
                    cellInputs = $("td:nth-child(" + (i + colIndexStart + 1) + ") .cell_input", context.filter('.grid-row'));
                }
                else {
                    cellInputs = $(".grid-btable tr td:nth-child(" + (i + colIndexStart + 1) + ") .cell_input", $this);
                }
                var events = colModel[i].formatterParam.events;
                for (var j in events) {
                    cellInputs.bind(events[j]["name"], events[j]["handler"]);
                }
            }
            //自定义控件初始化
            else if (colModel[i].formatter == "custom") {
                var rows, rowData, rowIndex, id, wdgt;
                expandable = $this.data('WidgetData').privateProperty.expandable;
                if (isInsert) {
                    rows = context.filter('.grid-row');
                }
                else {
                    rows = $(".grid-btable tr.grid-row", $this);
                }

                rows.each(function (index, elem) {
                    rowIndex = getRowIndex(expandable, elem);
                    rowData = rowList[rowIndex];
                    //        			id = getRowId(keyNames,rowData);
                    $td = $(">td:eq(" + (i + colIndexStart) + ")", elem);
                    wdgt = colModel[i].formatterParam.init($td, rowData);
                    if (colModel[i].formatterParam.setWidgetDisableStatus) {
                        if (rowData["grid-row-disabled"]) {
                            colModel[i].formatterParam.setWidgetDisableStatus(wdgt, true, rowData);
                        }
                        else {
                            colModel[i].formatterParam.setWidgetDisableStatus(wdgt, false, rowData);
                        }
                    }
                    $td.data("widget", wdgt);
                    wdgt.data("grid_cell", $td);
                });
            }
        }

        //展开收起事件
        $("div.grid-expandable", context).click(function (e) {
            //            if (e.stopPropagation)
            //                e.stopPropagation();
            var $target = $(this);
            var ci = $target.closest("td")[0].cellIndex - colIndexStart;
            var $tr = $target.closest("tr");
            var ri = getRowIndex(expandable, $tr[0]);
            var subRow = $tr.next("tr.grid-expand-row");
            var pl = $("div.grid-expand-pl", subRow);
            var fp = colModel[ci].formatterParam;
            var rowData = $this.wbGrid("getRowByIndex", ri)["data"];
            if (rowData["grid-row-disabled"]) {
                return;
            }
            if ($target.hasClass("open")) {
                subRow.hide();
                $target.removeClass("open");
                rowData[colModel[ci].name]["expand"] = false;
                if (fp.oncollapse) {
                    fp.oncollapse(e, ri, colModel[ci].name, rowList[ri], pl);
                }
            }
            else {
                if (fp.mode == "standard" && fp.expandReload) {
                    pl.empty();
                    rowData[colModel[ci].name]["expand"] = true;

                    if (fp.async) {
                        //如果是异步方式加载数据，返回数据后，更新rowData["data"]，然后调用refreshExpandData
                        fp.getSubData(ri, colModel[ci].name, rowList[ri]);
                    }
                    else {
                        rowData[colModel[ci].name]["data"] = fp.getSubData(ri, colModel[ci].name, rowList[ri]);
                        var subHtml = getExpandSubPlHtml(rowData[colModel[ci].name]["data"], fp);
                        pl.append(subHtml);
                        //绑定展开行中的按钮事件
                        bindExpandSubPlEvent(grid, pl);
                    }
                }
                subRow.show();
                $target.addClass("open");
                if (fp.onexpand) {
                    fp.onexpand(e, ri, colModel[ci].name, rowList[ri], pl);
                }
            }
        });

        //绑定展开行中的按钮事件
        bindExpandSubPlEvent(grid, context);

        //多选控制
        var $checkAll = $("input.grid-check-all", $this);
        var $check_context = $("input.grid-check", context);
        $check_context.click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            check_context(e, $(this).is(":checked"), $(this));
        });

        function check_context(e, isChecked, $chkbox) {
            var $check = $("input.grid-check", $this);
            if (isChecked) {
                if ($(".grid-bdiv input.grid-check:not(:checked)", $this).length == 0)
                    $checkAll.attr("checked", "checked");
                else
                    $checkAll.removeAttr("checked");
                $chkbox.closest("tr.grid-row").addClass("selected");
            }
            else {
                $checkAll.removeAttr("checked");
                $chkbox.closest("tr.grid-row").removeClass("selected");
            }

            //单击多选checkbox时，不能直接冒泡到行单击事件，因为行单击也会处理选中状态，导致选中处理执行两次，所有这里单独执行一次行/单元格点击处理函数
            //单元格单击事件
            if (settings.callback.onClickCell) {
                var rowIndex = getRowIndex(expandable, $chkbox.closest("tr")[0]);
                settings.callback.onClickCell(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            }

            //行单击事件
            if (settings.callback.onClickRow) {
                var rowIndex = getRowIndex(expandable, $chkbox.closest("tr")[0]);
                settings.callback.onClickRow(e, $this.attr("id"), rowIndex, rowList[rowIndex]);
            }
        }

        //点击多选单元格，相当于点击多选按钮
        $(".grid-muti-col", context).click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            var $chkbox = $("input[type='checkbox']", $(this));
            if ($chkbox.hasClass("grid-check-all"))
                return;
            var isChecked = !$chkbox.is(":checked");
            $chkbox.attr("checked", isChecked);
            check_context(e, isChecked, $chkbox);
        });
    }

    //获取展开行的子内容区域HTML
    function getExpandSubPlHtml(subData, fp) {
        var subHtml = "";
        if (subData && subData.length > 0) {
            for (var i in subData) {
                subHtml += "<div class=\"grid-expand-subItem\" value=\"" + subData[i].value + "\">" +
				(fp.showIcon ? "<div class=\"grid-expand-subItem-icon " + (fp.iconClass ? fp.iconClass : "") + "\"></div>" : "") +
				"<div class=\"grid-expand-subItem-text\" title=\"" + subData[i].text + "\">" + (subData[i].text.length > fp.maxTextDispolayLen ? (subData[i].text.substr(0, fp.maxTextDispolayLen) + "...") : subData[i].text) + "</div>" +
				(fp.showEdit ? "<div class=\"grid-expand-editItem\"></div>" : "") +
				(fp.showDel ? "<div class=\"grid-expand-delItem\"></div>" : "") +
				"</div>";
            }
        }
        if (fp.showAdd) {
            subHtml += "<div class=\"grid-expand-addItem\"></div>";
        }
        return subHtml;
    }

    function bindExpandSubPlEvent(grid, context) {
        var $this = $(grid),
		pp = $this.data('WidgetData').privateProperty,
		expandable = pp.expandable,
		expandableColIndex = pp.expandableColIndex,
		colModel = $this.data('WidgetData').settings.data.colModel;
        //新增
        $(".grid-expand-addItem", context).click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            var $target = $(this);
            var $tr = $target.closest("tr");
            var mainRow = $tr.prev("tr.grid-row");
            var ri = getRowIndex(expandable, mainRow[0]);
            var pl = $("div.grid-expand-pl", $tr);
            var fp = colModel[expandableColIndex].formatterParam;
            var subData = $this.wbGrid("getRowByIndex", ri)["data"];
            if (fp.onAdd) {
                fp.onAdd(ri, colModel[expandableColIndex].name, subData, pl);
            }
        });

        //删除
        $(".grid-expand-delItem", context).click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            var $target = $(this);
            var $tr = $target.closest("tr");
            var mainRow = $tr.prev("tr.grid-row");
            var ri = getRowIndex(expandable, mainRow[0]);
            var pl = $("div.grid-expand-pl", $tr);
            var fp = colModel[expandableColIndex].formatterParam;
            var subData = $this.wbGrid("getRowByIndex", ri)["data"];
            var subItem = $target.closest("div.grid-expand-subItem");
            if (fp.onDel) {
                fp.onDel(ri, colModel[expandableColIndex].name, subData, subItem.attr("value"), $(".grid-expand-subItem-text", subItem).text(), subItem, pl);
            }
        });

        //修改
        $(".grid-expand-editItem", context).click(function (e) {
            if (e.stopPropagation)
                e.stopPropagation();
            var $target = $(this);
            var $tr = $target.closest("tr");
            var mainRow = $tr.prev("tr.grid-row");
            var ri = getRowIndex(expandable, mainRow[0]);
            var pl = $("div.grid-expand-pl", $tr);
            var fp = colModel[expandableColIndex].formatterParam;
            var subData = $this.wbGrid("getRowByIndex", ri)["data"];
            var subItem = $target.closest("div.grid-expand-subItem");
            if (fp.onDel) {
                fp.onDel(ri, colModel[expandableColIndex].name, subData, subItem.attr("value"), $(".grid-expand-subItem-text", subItem).text(), pl);
            }
        });
    }

    //获取行号
    function getRowIndex(expandable, tr) {
        var rowIndex = tr.rowIndex - 1;
        if (expandable)
            rowIndex = rowIndex / 2;
        return rowIndex;
    }

    //根据index获取行
    function getTrByIndex($grid, expandable, index) {
        var trs = [];
        trs.push($("tr.grid-row:eq(" + index + ")", $grid));
        if (expandable) {
            trs.push($("tr.grid-expand-row:eq(" + index + ")", $grid));
        }
        return trs;
    }

    //获取行id
    function getRowId(keyNames, rowData) {
        if (keyNames.length == 0)
            throw "no id";
        else if (keyNames.length == 1)
            return rowData[keyNames[0]];
        else {
            var id = {};
            for (var i in keyNames) {
                id[keyNames[i]] = rowData[keyNames[i]];
            }
            return id;
        }
    }

    //获取行id序列
    function getRowSerialId(keyNames, rowData) {
        if (keyNames.length == 0)
            throw "no id";
        else if (keyNames.length == 1)
            return rowData[keyNames[0]];
        else {
            var id = "";
            for (var i in keyNames) {
                id += "," + rowData[keyNames[i]];
            }
            return id.substr(1);
        }
    }

    //序列id转id
    function serailId2Id(keyNames, serailId) {
        if (keyNames.length == 1)
            return serailId;
        else {
            var id = {}, arr = serailId.split(',');
            for (var i in keyNames) {
                id[i] = arr[i];
            }
            return id;
        }
    }

    //获取列模型列名映射
    function getColModelMap($grid) {
        var colModelMap = $grid.data('WidgetData').privateProperty.colModelMap,
		colModel = $grid.data('WidgetData').settings.data.colModel;
        if (!colModelMap) {
            colModelMap = {};
            for (var i in colModel) {
                colModel[i]['colIndex'] = i * 1;
                colModelMap[colModel[i].name] = colModel[i];
            }
        }
        return colModelMap;
    }

    //获得控件的显示值
    function getDisplayText($input) {
        switch ($input[0].tagName.toLowerCase()) {
            case "select":
                return $("option:selected", $input).text();
            case "input":
                if ($input.attr("type") == "password")
                    return "******";
                else
                    return $input.val();
            default:
                return $input.val();
        }
    }

    //重置交替行样式
    function resetAlterRow($grid) {
        var altRow = $grid.data('WidgetData').settings.view.altRow;
        if (!altRow)
            return;
        var $gridRows = $(".grid-btable tr.grid-row", $grid);
        $gridRows.removeClass("grid-altrow");
        $gridRows.each(function (i, elem) {
            if (i % 2 == 1)
                $(elem).addClass("grid-altrow");
        });
    }

    //重置行号行样式
    function resetRowNumber($grid) {
        var rownumbers = $grid.data('WidgetData').settings.view.rownumbers;
        if (!rownumbers)
            return;
        var $gridRows = $(".grid-btable tr.grid-row", $grid);
        $gridRows.each(function (i, elem) {
            $("td.grid-rownumber-col", elem).html(i + 1);
        });
    }

    function setRowsCellsValue($gird, rows, colNames, colValues, ignoreEditTigger) {
        var $this = $gird, $tr, $td,
		pp = $this.data('WidgetData').privateProperty,
		settings = $this.data('WidgetData').settings,
		keyNames = pp.keyNames, colIndexStart = pp.colIndexStart,
		colModelMap = getColModelMap($this), colModelItem, callBackTriggered;
        for (var i in rows) {
            callBackTriggered = false;
            $tr = $(".grid-row:eq(" + rows[i].index + ")", $this);
            for (var j in colNames) {
                if (rows[i]['data'][colNames[j]] == colValues[j]) {
                    continue;
                }
                rows[i]['data'][colNames[j]] = colValues[j];
                colModelItem = colModelMap[colNames[j]];
                $td = $(">td:eq(" + (colModelItem.colIndex + colIndexStart) + ")", $tr);
                //设置单元格的显示值
                if (colModelItem.formatter) {
                    switch (colModelItem.formatter) {
                        //下拉框显示                                                                                                                     
                        case "select":
                            //文本框显示
                        case "textbox":
                            //密码框显示
                        case "password":
                            {
                                var param = colModelItem.formatterParam;
                                $(".cell_input", $td).val(colValues[j]);
                                if (param.mode != "always") {
                                    $(".cell_span", $td).html(getDisplayText($(".cell_input", $td)));
                                }
                            }
                            break;
                        //CheckBox显示                                                                                                                     
                        case "checkbox":
                            {
                                if (colValues[j])
                                    $(".cell_span", $td).attr("checked", "checked");
                                else
                                    $(".cell_span", $td).removeAttr("checked");
                            }
                            break;
                        //自定义控件显示                                                                                                                     
                        case "custom":
                            {
                                colModelItem.formatterParam.setWidgetValue($td.data("widget"), colValues[j]);
                            }
                            break;
                    }
                }
                //普通显示
                else {
                    $td.html(colModelItem["formatDisplay"] ? colModelItem["formatDisplay"](colValues[j], rows[i]['data']) : colValues[j]);
                }

                if (!ignoreEditTigger) {
                    $tr.addClass("grid-row-edited");
                    if (settings.callback.cellValueChanged && !callBackTriggered) {
                        settings.callback.cellValueChanged($gird.attr("id"), rows[i].index, rows[i]['data'], colNames[j]);
                        callBackTriggered = true;
                    }
                }
            }
        }
    }

    //同步单元格数据，并触发cellValueChanged事件
    function syncCellValue(grid, row, colName, colValue, ignoreEditTigger) {
        if (row['data'][colName] == colValue)
            return;
        row['data'][colName] = colValue;
        var $this = $(grid),
		settings = $this.data('WidgetData').settings,
		$tr = $(".grid-row:eq(" + row.index + ")", $this);
        if (!ignoreEditTigger) {
            $tr.addClass("grid-row-edited");
            if (settings.callback.cellValueChanged) {
                settings.callback.cellValueChanged($this.attr("id"), row.index, row['data'], colName);
            }
        }
    }

    //设置行禁用状态
    function setRowDisableStatus(rowData, $tr, grid, disabled) {
        var $this = $(grid);
        if (!rowData)
            return;

        var $wd = $this.data('WidgetData'),
		colModel = $wd.settings.data.colModel,
		colIndexStart = $wd.privateProperty.colIndexStart;

        if (disabled) {
            rowData["data"]["grid-row-disabled"] = true;
            $(".cell_input,.grid-btn", $tr).attr("disabled", "disabled");
            $(".grid-iocn-btn", $tr).addClass("disabled");
            for (var i in colModel) {
                if (colModel[i].formatter == "custom") {
                    var $td = $(">td:eq(" + (i * 1 + colIndexStart) + ")", $tr[0]);
                    colModel[i].formatterParam.setWidgetDisableStatus($td.data("widget"), true, rowData["data"]);
                }
            }
            $tr.addClass("grid-row-disabled");

            //禁用时，收起子区域
            var expandable = $this.data('WidgetData').privateProperty.expandable;
            if (expandable) {
                var subRow = $tr.next("tr.grid-expand-row");
                var expandableColIndex = $this.data('WidgetData').privateProperty.expandableColIndex;
                var $target = $("div.grid-expandable", $tr[0]);
                if ($target.hasClass("open")) {
                    subRow.hide();
                    $target.removeClass("open");
                    rowData["data"][colModel[expandableColIndex].name]["expand"] = false;
                }
            }

        }
        else {
            rowData["data"]["grid-row-disabled"] = false;
            $(".cell_input,.grid-btn", $tr).removeAttr("disabled");
            $(".grid-iocn-btn", $tr).removeClass("disabled");
            for (var i in colModel) {
                if (colModel[i].formatter == "custom") {
                    var $td = $(">td:eq(" + (i * 1 + colIndexStart) + ")", $tr[0]);
                    colModel[i].formatterParam.setWidgetDisableStatus($td.data("widget"), false, rowData["data"]);
                }
            }
            $tr.removeClass("grid-row-disabled");
        }
    }

    //设置行逻辑状态
    function setRowLogicStatus(rowData, $tr, grid, status, toolTip) {
        var $this = $(grid);
        if (!rowData)
            return;
        rowData["data"]["grid-row-status"] = status;
        switch (status) {
            case "loading":
            case "success":
            case "error":
                $(".grid-status-col", $tr).addClass(status).attr("title", toolTip ? toolTip : "");
                break;
            default:
                $(".grid-status-col", $tr).removeClass("loading success error").removeAttr("title");
        }

        if ($(".grid-status-col.loading,.grid-status-col.success,.grid-status-col.error", $this).length > 0) {
            $(".grid-status-col", $this).show();
        }
        else {
            $(".grid-status-col", $this).hide();
        }
    }

    $.fn.wbGrid = function (method) {
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(
					arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist on jQuery.wbGrid');
        }
    };
})(jQuery);