﻿/*
IpInput控件
*/

(function($) {
	// 定义常量
	$.extend({
		// IpInput错误处理模式
		IPINPUT_INVALIDATE_MODE_REPLACE: "replace",
		IPINPUT_INVALIDATE_MODE_PREVENT: "prevent"
	});

	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				containerCss: "ip_input_container", // 容器默认样式
				inputCss: "ip_input", // 文本框默认样式
				splitCss: "ip_input_split", // 分隔符默认样式
				count: 4, // 输入框个数
				split: ".",// 分隔符
				max: 255,// 最大值
				min: 0,// 最小值
				defaultValue: "0.0.0.0", // 默认值
				autoWidth: true,// 是否自动列宽
				autoRecoverEmpty: true,// 是否自动修复空值（getValue时）
				invalidateMode: $.IPINPUT_INVALIDATE_MODE_REPLACE,// 错误处理模式
				context: document// 上下文环境
			}, options);

			return this.each(function() {
				var $this = $(this);
				$this.data("IpInput", {
					target: $this,
					settings: settings
				});

				var maxLength = (settings.max + "").length;
				var containerId = $this.attr("id");
				var ipInputContainer = $("<div id=\"" + containerId + "_ip_input_container\" class=\"" + settings.containerCss + "\"></div>");
				for (var i = 0; i < settings.count; i++) {
					ipInputContainer.append($("<input id=\"" + containerId + "_ip_input" + (i + 1) + "\" type=\"text\" style=\"width: " + maxLength * 8 + "px;\" maxlength=\"" + maxLength + "\" class=\"" + settings.inputCss + "\" />"));
					if (i != settings.count - 1) {
						ipInputContainer.append($("<span id=\"" + containerId + "_ip_input_split\" class=\"" + settings.splitCss + "\">" + settings.split + "</span>"));
					}
				}
				ipInputContainer.attr("disabled", !settings.enable)[settings.enable ? "removeClass" : "addClass"]("disabled")
					.click(function(event) {
						if (settings.enable && $(event.target).is($(this))) {
							$this.find(":text:first").focus();
						}
					});
				$this.append(ipInputContainer);

				var ipInput = $this.find(":text");
				ipInput.attr("disabled", !settings.enable);
				ipInput.focus(function(event) {
					$(this).select();
				}).blur(function(event) {
					if ($this.ipInput("getValue") != $this.data("oldValue")) {
						$this.data("changed", true);
					}
					if ($this.find(":focus").size() == 0) {
						$this.blur();
					}
				});
				ipInput.bind("paste", function(event) { return false; });
				ipInput.keydown(function(event) {
					var keyCode = event.which;

					if (keyCode == 9// Tab
						|| keyCode == 16// Shift
						|| keyCode == 17// Ctrl
						|| keyCode == 18// Alt
						|| keyCode == 35// End
						|| keyCode == 36// Home
						|| keyCode == 13// 回车
					) {
						return true;
					}

					var oldText = $(this).val();
					var range = getSelectRange(this, settings.context);
					var startIndex = range.start;
					var endIndex = range.end;
					var letter = String.fromCharCode(keyCode);
					var posIndex = startIndex + letter.length;

					if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
						letter = String.fromCharCode(keyCode - 48);
					}
					if (keyCode == 110 || keyCode == 190) {// 句点
						if (startIndex > 0) {
							$(this).nextAll(":text:first").focus();
						}
					}
					if (keyCode == 8) {// 退格
						if (startIndex == 0 && endIndex == 0) {
							$(this).prevAll(":text:first").focus();
							return false;
						}
						posIndex = startIndex - (startIndex == endIndex ? 1 : 0);
						$(this).val(oldText.substring(0, posIndex) + oldText.substring(endIndex));
						setSelectRange(this, posIndex, posIndex, settings.context);
						if (settings.autoWidth) {
							autoWidth();
						}
						return false;
					}
					if (keyCode == 46) {// 删除
						$(this).val(oldText.substring(0, startIndex) + oldText.substring(endIndex + (startIndex == endIndex ? 1 : 0)));
						setSelectRange(this, startIndex, startIndex, settings.context);
						if (settings.autoWidth) {
							autoWidth();
						}
						return false;
					}
					if (keyCode == 37) {// 向左
						if (startIndex == 0) {
							$(this).prevAll(":text:first").focus();
						} else {
							return true;
						}
					}
					if (keyCode == 39) {// 向右
						if (endIndex == oldText.length) {
							$(this).nextAll(":text:first").focus();
						} else {
							return true;
						}
					}
					if (keyCode == 32) {// 空格
						return false;
					}
					if (keyCode == 38 || keyCode == 40) {// 向上或向下
						letter = oldText * 1 + (keyCode == 38 ? 1 : -1) + "";
						startIndex = 0;
						endIndex = oldText.length > letter.length ? oldText.length : letter.length;
						posIndex = endIndex;
					}

					var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
					if (!isNaN(newText) && !/^0\d+$/.test(newText)) {
						if (startIndex == maxLength && endIndex == maxLength) {
							$(this).nextAll(":text:first").focus();
							return false;
						}
						if (newText * 1 < settings.min) {
							if (settings.invalidateMode == $.IPINPUT_INVALIDATE_MODE_REPLACE) {
								newText = settings.min;
							} else if (settings.invalidateMode == $.IPINPUT_INVALIDATE_MODE_PREVENT) {
								return false;
							}
						}
						if (newText * 1 > settings.max) {
							if (settings.invalidateMode == $.IPINPUT_INVALIDATE_MODE_REPLACE) {
								newText = settings.max;
							} else if (settings.invalidateMode == $.IPINPUT_INVALIDATE_MODE_PREVENT) {
								return false;
							}
						}
						$(this).val(newText);
						setSelectRange(this, posIndex, posIndex, settings.context);
						// 向上或向下时输入满三位不跳到下一格
						if (posIndex == maxLength && keyCode != 38 && keyCode != 40) {
							$(this).nextAll(":text:first").focus();
						}
						if (settings.autoWidth) {
							autoWidth();
						}
					}
					return false;
				});

				$this.ipInput("setValue", settings.defaultValue);

				// 获取选中区间
				function getSelectRange(textControl, context) {
					var range = {};
					if (!context) {
						context = document;
					}
					if (context.selection) {
						if (textControl.tagName != undefined && textControl.tagName == "INPUT") {
							var selectionRange = context.selection.createRange();
							var textControlRange = textControl.createTextRange();
							range.text = selectionRange.text;
							var oldLength = range.text.length;
							selectionRange.setEndPoint("EndToStart", textControlRange);
							var newLength = selectionRange.text.length;
							range.start = newLength;
							range.end = newLength + oldLength;
						}
					} else {
						range.start = textControl.selectionStart;
						range.end = textControl.selectionEnd;
						range.text = textControl.value.substring(range.start, range.end);
					}
					return range;
				}
	
				// 设置选中区间
				function setSelectRange(textControl, startIndex, endIndex, context) {
					if (!context) {
						context = document;
					}
					if (context.selection) {
						var range = textControl.createTextRange();
						range.collapse(true);
						range.moveStart("character", startIndex);
						range.moveEnd("character", endIndex - startIndex);
						range.select();
					} else {
						textControl.setSelectionRange(startIndex, endIndex);
						textControl.focus();
					}
				}
	
				// 自动调整列宽
				function autoWidth() {
					$this.find(":text").each(function(index, element) {
						var value = $(this).val();
						$(this).css("width", (value.length == 0 ? 1 : value.length) * 8);
					});
				}
			});
		},
		setValue: function(value) {
			if (typeof value == "string") {
				var $this = $(this);
				var settings = $this.data("IpInput")["settings"];
				var valueParts = value.split(settings.split);
				if (valueParts.length == settings.count) {
					$this.data("oldValue", value);
					$this.data("changed", false);
					$this.find(":text").each(function(index, element) {
						$(this).val(valueParts[index]);
						if (settings.autoWidth) {
							$(this).css("width", (valueParts[index].length == 0 ? 1 : valueParts[index].length) * 8);
						}
					});
				}
			}
		},
		getValue: function(value) {
			var $this = $(this);
			var settings = $this.data("IpInput")["settings"];
			var defaultValueParts = settings.defaultValue.split(settings.split);
			var valuePart = [];
			$this.find(":text").each(function(index, element) {
				var value = $(this).val();
				if (value == "") {
					value = defaultValueParts[index];
					if (settings.autoRecoverEmpty) {
						$(this).val(value);
					}
				}
				valuePart.push(value);
			});
			return valuePart.join(settings.split);
		},
		/**
		* 设置是否可用
		*/
		setEnable: function(enabled) {
			var $this = $(this);
			var settings = $this.data("IpInput")["settings"];
			settings.enable = enabled;
			$this.attr("disabled", !enabled).children()[enabled ? "removeClass" : "addClass"]("disabled")
				.find(":text").attr("disabled", !enabled);
		}
	};

	$.fn.ipInput = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.ipInput");
		}
	};
})(jQuery);