﻿/*
MacInput控件
*/

(function($) {
	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				css: "mac_input",// 控件的默认样式
				reg: /^([A-F\d]{2}:){5}[A-F\d]{2}$/, // 校验正则表达式
				defaultValue: "00:00:00:00:00:00" // 默认值
			}, options);

			return this.each(function() {
				this.style.imeMode = "disabled";
				$(this).addClass(settings.css);
				$(this).attr("readonly", true);// 为了解决IE10和IE11采用IE8引擎执行时-ms-clear样式失效问题
				$(this).attr("disabled", !settings.enable);
				$(this).val(settings.defaultValue);
				$(this).bind("paste", function(event) { return false; });
				$(this).keydown(function(event) {
					var keyCode = event.which;

					if (keyCode == 9// Tab
						|| keyCode == 16// Shift
						|| keyCode == 17// Ctrl
						|| keyCode == 18// Alt
						|| keyCode == 35// End
						|| keyCode == 36// Home
						|| keyCode == 37// 向左
						|| keyCode == 39// 向右
						|| keyCode == 13// 回车
					) {
						return true;
					}

					var splitPoints = [
						{ index: 2, length: 1 },
						{ index: 5, length: 1 },
						{ index: 8, length: 1 },
						{ index: 11, length: 1 },
						{ index: 14, length: 1 }
					];

					var range = getSelectRange(this);
					var isSplitIndex = false;
					var isAfterSplitIndex = false;
					var index = 0;
					for (; index < splitPoints.length; index++) {
						if (splitPoints[index]["index"] == range.start) {
							isSplitIndex = true;
							break;
						}
						if (splitPoints[index]["index"] == range.start - splitPoints[index]["length"]) {
							isAfterSplitIndex = true;
							break;
						}
					}
					var $this = $(this);
					var oldText = $this.val();
					var startIndex = range.start + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var endIndex = range.start + 1 + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var letter = String.fromCharCode(keyCode);
					var posIndex = endIndex;

					if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
						letter = String.fromCharCode(keyCode - 48);
					}
					if (event.shiftKey && keyCode == 186) {// 冒号
						letter = oldText.charAt(startIndex);
						var splitIndex = oldText.indexOf(":", isSplitIndex ? startIndex - 1 : startIndex);
						if (splitIndex != -1) {
							posIndex = splitIndex + 1;
						} else {
							posIndex = oldText.length;
						}
					}
					if (keyCode == 38) {// 向上
						var charCode = oldText.charCodeAt(startIndex);
						letter = charCode == 57 ? "A" : String.fromCharCode(charCode + 1);
						posIndex = startIndex;
					}
					if (keyCode == 40) {// 向下
						var charCode = oldText.charCodeAt(startIndex);
						letter = charCode == 65 ? "9" : String.fromCharCode(charCode - 1);
						posIndex = startIndex;
					}
					if (keyCode == 8) {// 退格
						startIndex = range.start - 1 - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						endIndex = range.start - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 46) {// 删除
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 32) {// 空格
						letter = "0";
					}
					var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
					if (settings.reg.test(newText)) {
						$this.val(newText);
						setSelectRange(this, posIndex, posIndex);
					}
					return false;
				});
			});

			// 获取选中区间
			function getSelectRange(textControl) {
				var range = {};
				if (document.selection) {
					if (textControl.tagName != undefined && textControl.tagName == "INPUT") {
                        var selectionRange;
                        if(document.selection.createRange().text == ""){
                            selectionRange = window.top.document.selection.createRange();
                        }else{
                            selectionRange = document.selection.createRange();
                        }
//						var selectionRange = document.selection.createRange();
						var textControlRange = textControl.createTextRange();
						range.text = selectionRange.text;
						var oldLength = range.text.length;
						selectionRange.setEndPoint("EndToStart", textControlRange);
						var newLength = selectionRange.text.length;
						range.start = newLength;
						range.end = newLength + oldLength;
					}
				} else {
					range.start = textControl.selectionStart;
					range.end = textControl.selectionEnd;
					range.text = textControl.value.substring(range.start, range.end);
				}
				return range;
			}

			// 设置选中区间
			function setSelectRange(textControl, startIndex, endIndex) {
				if (document.selection) {
					var range = textControl.createTextRange();
					range.collapse(true);
					range.moveStart("character", startIndex);
					range.moveEnd("character", endIndex - startIndex);
					range.select();
				} else {
					textControl.setSelectionRange(startIndex, endIndex);
					textControl.focus();
				}
			}
		}
	};

	$.fn.macInput = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.macInput");
		}
	};
})(jQuery);