/**
 * 分页控件
 */

(function($) {
	var methods = {
		init : function(options) {
			// 设定属性--覆盖默认属性
			var settings = $.extend(true, {
				pageIndex : 1,// 当前页号
				pageDataCount : 20,// 每页条数
				dataCount : 0,// 总条数
				lang:'en-us',//语言
				pageDataCountItems:[20,30],//每页条数选择范围
				onPage:null,//翻页事件
				_preIndex : 0//上一次的页号
			}, options || {});
			
			return this.each(function() {
				var $this = $(this);
				
				var langPacket = {
					"zh-cn":{
						"pg_txt1":"",
						"pg_txt2":"共{0}页",
						"pg_info":"{0}&nbsp;&nbsp;共{1}条"
					},
                    "zh-tw":{
                        "pg_txt1":"",
                        "pg_txt2":"共{0}頁",
                        "pg_info":"{0}&nbsp;&nbsp;共{1}條"
                    },
					"en-us":{
						"pg_txt1":"Page ",
						"pg_txt2":" of {0}",
						"pg_info":"View {0} of {1}"
					}
				};
				
				$(this).data('WidgetData', {
					target : $this,
					settings : settings,
					langPacket:langPacket
				});
				
				if(settings.lang && settings.lang in langPacket){
					
				}
				else{
					settings.lang = 'en-us';
				}
								
				var langObj = langPacket[settings.lang];
				var initHtml = "<span class=\"pager-info\">"+widget.stringFormat(langObj["pg_info"],"0 - 0",0)+"</span>"+
					"<div class=\"pager-ctrl\">"+
					"<span class=\"pager-icon pager-first disabled\"></span>"+
					"<span class=\"pager-icon pager-prev disabled\"></span>"+
					"<span class=\"pager-spliter\"></span>"+
					"<span class=\"pager-pg-txt-1\">"+langObj["pg_txt1"]+"</span>"+
					"<input class=\"pager-pg-input\" type=\"text\" disabled=\"disabled\" value=\"0\" />"+
					"<span class=\"pager-pg-txt-2\">"+widget.stringFormat(langObj["pg_txt2"],"0")+"</span>"+
					"<span class=\"pager-spliter\"></span>"+
					"<span class=\"pager-icon pager-next disabled\"></span>"+
					"<span class=\"pager-icon pager-last disabled\"></span>"+
					"<select class=\"pager-rownum-select\">";

				for(var i in settings.pageDataCountItems){
					initHtml += "<option value=\""+settings.pageDataCountItems[i]+"\" "+(settings.pageDataCount==settings.pageDataCountItems[i]?" selected=\"selected\"":"")+">"+settings.pageDataCountItems[i]+"</option>";
				}
				initHtml += "</select>"+
				"</div>";
				$this.addClass('pager-container').html(initHtml);
				
				//上一次的页号
				settings._preIndex = settings.pageIndex;
				
				//首页
				$(".pager-first",$this).click(function(){
					if(settings.pageIndex <=1)
						return;
					setCtrlStatus($this,1,true);
				});
				
				//上一页
				$(".pager-prev",$this).click(function(){
					if(settings.pageIndex <=1)
						return;
					setCtrlStatus($this,settings.pageIndex-1,true);
				});
				
				//下一页
				$(".pager-next",$this).click(function(){
					if(settings.pageIndex >=getPageCount(settings.dataCount,settings.pageDataCount))
						return;
					setCtrlStatus($this,settings.pageIndex+1,true);
				});
				
				//尾页
				$(".pager-last",$this).click(function(){
					if(settings.pageIndex >=getPageCount(settings.dataCount,settings.pageDataCount))
						return;
					setCtrlStatus($this,getPageCount(settings.dataCount,settings.pageDataCount),true);
				});
				
				//手动输入页号
				$(".pager-pg-input",$this).keydown(function(ev) {
			        if(widget.getKeyCode(ev) == 13){
			        	setCtrlStatus($this, (/^\d+$/.test($(this).val())?$(this).val()*1:1),true);
			        }
			    });				
				//改变每页笔数
				$(".pager-rownum-select",$this).change(function(){
					settings.pageDataCount = $(this).val()*1;
					setCtrlStatus($this, settings.pageIndex, true);
				});
				setCtrlStatus($this, settings.pageIndex, false);
			});
		},
		/**
		 * 获得当前页号
		 */
		getPageIndex:function(){
			var settings = $(this).data('WidgetData').settings;
			return settings.pageIndex;
		},
		/**
		 * 设置当前页号
		 * @param pageIndex
		 * @param callbackFlag
		 */
		setPageIndex:function(pageIndex, callbackFlag){
			var settings = $(this).data('WidgetData').settings;
			settings.pageIndex = pageIndex;
			setCtrlStatus($(this), pageIndex, callbackFlag);
		},
		/**
		 * 获得当前每页行数
		 */
		getPageDataCount:function(){
			var settings = $(this).data('WidgetData').settings;
			return settings.pageDataCount;
		},
		/**
		 * 设置总条数
		 * @param dataCount
		 * @param callbackFlag
		 */
		setDataCount:function(dataCount, callbackFlag){
			var settings = $(this).data('WidgetData').settings;
			settings.dataCount = dataCount;
			setCtrlStatus($(this), settings.pageIndex, callbackFlag);
		},
		/**
		 * 设置语言
		 * @param lang
		 */
		setLang:function(lang){
			var $this=$(this),settings=$this.data('WidgetData').settings;
			settings.lang = lang;
			$(".pager-pg-txt-1",$this).html($this.data('WidgetData').langPacket[lang]["pg_txt1"]);
			setCtrlStatus($(this), settings.pageIndex);
		}
	};
	//设置状态
	function setCtrlStatus($pager, pageIndex, callbackFlag){
		var settings = $pager.data('WidgetData').settings,
		langObj=$pager.data('WidgetData').langPacket[settings.lang],
		isEnable=[true,true,true,true];
		var pageCount = getPageCount(settings.dataCount,settings.pageDataCount);
		$(".pager-pg-txt-2",$pager).html(widget.stringFormat(langObj["pg_txt2"],pageCount));
		if(pageCount<=1){
			isEnable[0] = false;
			isEnable[1] = false;
			isEnable[2] = false;
			isEnable[3] = false;
			$(".pager-pg-input",$pager).attr("disabled","disabled");
		}
		else{
			$(".pager-pg-input",$pager).removeAttr("disabled");
		}
		if(settings.dataCount>0){
			if(pageIndex>=1 && pageIndex<=pageCount){
				if(pageIndex==1){
					isEnable[0] = false;
					isEnable[1] = false;
				}
				if(pageIndex==pageCount){
					isEnable[2] = false;
					isEnable[3] = false;
				}
				settings.pageIndex=settings._preIndex=pageIndex;
				$(".pager-pg-input",$pager).val(pageIndex);
				var f = (pageIndex-1)*settings.pageDataCount+1;
				var e = f+settings.pageDataCount-1;
				e = e>settings.dataCount?settings.dataCount:e;
				$(".pager-info",$pager).html(widget.stringFormat(langObj["pg_info"],f+" - "+e,settings.dataCount));
				if(callbackFlag && settings.onPage){
					settings.onPage($pager.attr("id"), settings.pageIndex , settings.pageDataCount);
				}
			}
			else{
				setCtrlStatus($pager, settings._preIndex>pageCount?pageCount:settings._preIndex, callbackFlag);
				return;
			}
		}
		else{
			$(".pager-pg-input",$pager).val(0);
			$(".pager-info",$pager).html(widget.stringFormat(langObj["pg_info"],"0 - 0",0));
			if(callbackFlag && settings.onPage){
				settings.onPage($pager.attr("id"), settings.pageIndex , settings.pageDataCount);
			}			
		}
		isEnable[0]?$(".pager-first",$pager).removeClass("disabled"):$(".pager-first",$pager).addClass("disabled");
		isEnable[1]?$(".pager-prev",$pager).removeClass("disabled"):$(".pager-prev",$pager).addClass("disabled");
		isEnable[2]?$(".pager-next",$pager).removeClass("disabled"):$(".pager-next",$pager).addClass("disabled");
		isEnable[3]?$(".pager-last",$pager).removeClass("disabled"):$(".pager-last",$pager).addClass("disabled");
	}
	//获得页数
	function getPageCount(dataCount,pageDataCount){
		return Math.ceil(dataCount/pageDataCount);
	}

	$.fn.wbPager = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(
					arguments, 1));
		} else if (typeof method === 'object' || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error('Method ' + method + ' does not exist on jQuery.wbPager');
		}
	};
})(jQuery);