﻿/*
SelectInput控件
*/
var objList = [];
var tempObj;

(function($) {
	// 定义常量
	$.extend({
		// SelectInput纠错模式
		SELECTINPUT_INVALID_MODE_FIRSTITEM: "firstItem",
		SELECTINPUT_INVALID_MODE_CLEAR: "clear",
		SELECTINPUT_INVALID_MODE_CUSTOM: "custom"
	});

	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				editable: true, // 是否可编辑
				containerCss: "select_input_container", // 容器默认样式
				inputCss: "select_input", // 文本框默认样式
				focusCss: "select_input_focus", // 文本框获得焦点默认样式（不可编辑时有效）
				buttonCss: "select_input_button", // 选择按钮默认样式
				listCss: "select_input_list", // 列表默认样式
				listMaskCss: "select_input_list_mask", // 列表遮罩层默认样式
				itemCss: "select_input_item", // 列表项默认样式
				itemSelectedCss: "select_input_item_selected", // 列表项悬停默认样式
				context: document, // 指定的DOM文档对象
				defaultData: { text: "", value: "" }, // 默认值
				invalidMode: $.SELECTINPUT_INVALID_MODE_FIRSTITEM, // 纠错模式，可选项：SELECTINPUT_INVALID_MODE_FIRSTITEM（首项），SELECTINPUT_INVALID_MODE_CLEAR（清除），SELECTINPUT_INVALID_MODE_CUSTOM（自定义）
				viewCount: 10, // 可视区域项数
				selectCircle: false, // 选项移动是否循环
				autoMatchListValue: true, // 是否自动匹配列表项
				autoConvertText: true, // 是否自动将文本转换为匹配文本
				autoAddItem: false, // 是否自动将新输入的合法值加入列表
				allowEmpty: true, // 是否允许为空
				list: [], // 列表项，格式{ text: "", value: "" }
				mapping: function(text) {
					return { text: text, value: text };
				}, // 输入文本映射为文本/值对象的函数，控件使用者主要需要重写的模板函数，返回false表示输入文本不合法
				onchange: null,// 值改变事件
				oninvalidate: null // 输入值不合法
			}, options);

			return this.each(function() {
				var scrollWidth = 17;
				var $this = $(this);
                $this.empty();
                objList.push($this);

				$this.data("SelectInput", {
					target: $this,
					settings: settings
				});

				var containerId = $this.attr("id");
				var selectInputContainer = $("<div id=\"" + containerId + "_select_input_container\" style=\"width: 100%;\" class=\"" + settings.containerCss + "\"></div>", settings.context);
				$this.append(selectInputContainer);

				var selectInput = $("<input id=\"" + containerId + "_select_input\" type=\"text\" style=\"width: " + (selectInputContainer.parent().width() - scrollWidth - 6) + "px;\" class=\"" + settings.inputCss + "\" />", settings.context);
				selectInput.attr("disabled", !settings.enable);
				if (!settings.editable) {
					selectInput.attr("readonly", true);
					selectInput.css("cursor", "default");
					setUnselectable(selectInput);
					selectInput.focus(function(event) {
						if ($("#" + containerId + "_list", settings.context).is(":visible")) {
							return;
						}
						selectInput.removeClass(settings.inputCss);
						selectInput.addClass(settings.focusCss);
					}).blur(function(event) {
						selectInput.addClass(settings.inputCss);
						selectInput.removeClass(settings.focusCss);
					});
				}
				selectInputContainer.append(selectInput);
				selectInput.keydown(function(event) {
					var isShowingList = $("#" + containerId + "_list", settings.context).is(":visible");
					var originalValue = $this.data("data").value;
					switch (event.which) {
						case 27:// Esc
							$(this).blur();
						case 9:// Tab
							hideList();
							// 先让按钮获取焦点再触发Tab键，让焦点跳到下一个控件，即按钮不能通过Tab键获得焦点
							$("#" + containerId + "_select_input_button").focus();
							break;
						case 13:// 回车
							if (isShowingList) {
								var data = settings.list[getSelectedIndex()];
								$this.data("data", data);
								selectInput.val(data.text);
								hideList();
								if (settings.onchange && originalValue != data.value) {
									settings.onchange($this, data.value);
								}
							} else {
								selectInput.blur();
							}
							break;
						case 38:// 向上
							if (isShowingList) {
								var selectedIndex = getSelectedIndex();
								if (selectedIndex > 0) {
									selectedIndex--;
								} else if (selectedIndex < 0) {
									selectedIndex = 0;
								} else if (settings.selectCircle) {
									selectedIndex = settings.list.length - 1;
								}
								setSelectedIndex(selectedIndex);
							} else {
								var selectedIndex = $this.selectInput("getSelectedIndex");
								if (selectedIndex < 0) {
									selectedIndex = 0;
								}
								if (selectedIndex == 0 && settings.selectCircle) {
									selectedIndex = settings.list.length - 1;
								} else if (selectedIndex > 0) {
									selectedIndex--;
								}
								$this.selectInput("setSelectedIndex", selectedIndex);
								var currentValue = $this.data("data").value;
								if (settings.onchange && originalValue != currentValue) {
									settings.onchange($this, currentValue);
								}
							}
							// 阻止滚动条向上滚动默认行为
							return false;
						case 40:// 向下
							if (isShowingList) {
								var selectedIndex = getSelectedIndex();
								if (selectedIndex < settings.list.length - 1) {
									selectedIndex++;
								} else if (settings.selectCircle) {
									selectedIndex = 0;
								}
								setSelectedIndex(selectedIndex);
							} else {
								var selectedIndex = $this.selectInput("getSelectedIndex");
								selectedIndex++;
								if (selectedIndex > settings.list.length - 1 && settings.selectCircle) {
									selectedIndex = 0;
								}
								$this.selectInput("setSelectedIndex", selectedIndex);
								var currentValue = $this.data("data").value;
								if (settings.onchange && originalValue != currentValue) {
									settings.onchange($this, currentValue);
								}
							}
							// 阻止滚动条向下滚动默认行为
							return false;
						default:
							if (!settings.editable) {
								return false;
							}
							break;
					}
				}).bind("propertychange input", function(event) {
					if ((event.type == "propertychange" && event.originalEvent.propertyName == "value")
						|| event.type == "input") {
						// 延迟选中是为了解决IE11文本框莫名自动消失的问题
						setTimeout(function() {
							var divList = $("#" + containerId + "_list", settings.context);
							if (divList.is(":visible")) {
								initSelected();
							}
						}, 0);
					}
				}).mousedown(function(event) {
					$this.data("selfMousedown", true);
					if (settings.editable) {
						return;
					}
					selectInputButton.focus();
					selectInputButton.mousedown();
				}).mouseup(function(event) {
					if (settings.editable) {
					$this.data("selfMousedown", false);
						return;
					}
					selectInputButton.mouseup();
				}).bind("mousewheel DOMMouseScroll", function(event) {
					var divList = $("#" + containerId + "_list", settings.context);

					if ($(settings.context.activeElement).attr("id") == containerId + "_select_input" && !divList.is(":visible")) {
						var originalValue = $this.data("data").value;
						var delta = event.originalEvent.wheelDelta || -event.originalEvent.detail;

						var selectedIndex = $this.selectInput("getSelectedIndex");
						if (delta > 0) {
							if (selectedIndex < 0) {
								selectedIndex = 0;
							}
							if (selectedIndex == 0 && settings.selectCircle) {
								selectedIndex = settings.list.length - 1;
							} else if (selectedIndex > 0) {
								selectedIndex--;
							}
						} else {
							selectedIndex++;
							if (selectedIndex > settings.list.length - 1 && settings.selectCircle) {
								selectedIndex = 0;
							}
						}
						$this.selectInput("setSelectedIndex", selectedIndex);
						var currentValue = $this.data("data").value;
						if (settings.onchange && currentValue != originalValue) {
							settings.onchange($this, currentValue);
						}

						return false;
					}
				}).dblclick(function(event) {
					if (!settings.editable) {
						return;
					}
					var divList = $("#" + containerId + "_list", settings.context);
					if (!divList.is(":visible")) {
						var offset = selectInputContainer.offset();
						var height = selectInputContainer.height();
						showList(offset.left, offset.top + height);
						initSelected();
					}
				}).change(function(event) {
					// 如果正在选择选项，则不校验值
					if ($this.data("mouseInList")) {
						return;
					}

					var text = $(this).val();
					var currentValue = $this.data("data").value;

					if (settings.allowEmpty && text == "") {
						$this.data("data", { text: "", value: "" });
						if (settings.onchange) {
							settings.onchange($this, "");
						}
						return;
					}

					var isInList = false;
					$.each(settings.list, function(index, element) {
						if (element.text == text
							|| element.value == text) {
							$this.data("data", element);
							if (settings.autoMatchListValue) {
								selectInput.val(element.text);
							}
							isInList = true;
							if (settings.onchange && currentValue != element.value) {
								settings.onchange($this, element.value);
							}
							return false;
						}
					});
					if (!isInList) {
						var data = settings.mapping(text);
						if (data === false) {
							var originalData = $this.data("data");
							if (originalData) {
								$(this).val(originalData.text).focus();
							} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_FIRSTITEM) {
								$this.selectInput("setSelectedIndex", 0);
							} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_CLEAR) {
								$this.data("data", { text: "", value: "" });
								$(this).val("").focus();
							}
							if (settings.oninvalidate) {
								settings.oninvalidate($this, text);
							}
						} else {
							$this.data("data", data);
							if (settings.autoAddItem) {
								settings.list.push(data);
							}
							if (settings.autoConvertText) {
								$(this).val(data.text);
							}
							if (settings.onchange && currentValue != data.value) {
								settings.onchange($this, data.value);
							}
						}
					}
				});

				var selectInputButton = $("<input type=\"button\" id=\"" + containerId + "_select_input_button\" style=\"width: " + scrollWidth + "px; height: " + selectInput.outerHeight() + "px;\" class=\"" + settings.buttonCss + "\"/>", settings.context);
				selectInputButton.attr("disabled", !settings.enable);
				selectInputContainer.append(selectInputButton);

				selectInputButton.mousedown(function(event) {
					$this.data("selfMousedown", true);
                    tempObj = $this;

					var divList = $("#" + containerId + "_list", settings.context);
					if (divList.is(":visible")) {
						hideList();
					} else {
						var offset = selectInputContainer.offset();
						var height = selectInputContainer.height();
						showList(offset.left, offset.top + height);
						initSelected();
					}
				}).mouseup(function(event) {
					$this.data("selfMousedown", false);
					selectInput.focus();
					$(this).blur();
				});

				$this.selectInput("setData", !settings.editable && settings.defaultData.text == "" && settings.list.length > 0 ? settings.list[0] : settings.defaultData);

				// 显示列表
				function showList(left, top) {
					var divList = $("#" + containerId + "_list", settings.context);
					var divListMask = $("#" + containerId + "_list_mask", settings.context);
					if (divList.size() == 0) {
						divList = $("<div id=\"" + containerId + "_list\" class=\"" + settings.listCss + "\"></div>", settings.context);
						divListMask = $("<iframe id=\"" + containerId + "_list_mask\" class=\"" + settings.listMaskCss + "\" scrolling=\"no\" frameborder=\"0\" src=\"about:blank\"></iframe>", settings.context);
						divList.hover(function(event) {
							$this.data("mouseInList", true);
						}, function(event) {
							$this.data("mouseInList", false);
						}).mousedown(function(event) {
							// 使点击下拉框的滚动条时文本框也能保持焦点（滚动条不支持click和mouseup），延迟获取解决焦点冲突问题
							setTimeout(function() {
								selectInput.focus();
							}, 0);
							$this.data("selfMousedown", true);
						});
						var body = $("body", settings.context);
						body.append(divList);
						body.append(divListMask);
						divList.hide();
						divListMask.hide();
					}
					divList.empty();
					$.each(settings.list, function(index, element) {
						var divItem = $("<div id=\"" + containerId + "_item" + index + "\" class=\"" + settings.itemCss + "\" style=\"-moz-user-select: none;\">" + element.text + "</div>", settings.context);
						divItem.data("itemData", element);
						divItem.bind("selectstart", function(event) {
							return false;
						}).mouseover(function(event) {
							divList.find("div." + settings.itemSelectedCss).removeClass().addClass(settings.itemCss);
							$(this).removeClass().addClass(settings.itemSelectedCss);
						}).mousedown(function(event) {
							event.stopPropagation();
						}).click(function(event) {
							var originalValue = $this.data("data").value;
							var data = $(this).data("itemData");
							$this.data("data", data);
							selectInput.focus();
							selectInput.val(data.text);
							hideList();
							if (settings.onchange && originalValue != data.value) {
								settings.onchange($this, data.value);
							}
						});
						divList.append(divItem);
					});

					// 针对IE位置偏移和宽度有修正
					divList.css({
						left: left,
						top: $.browser.msie ? top - 1 : top,
						width: selectInputContainer.outerWidth() - ($.browser.msie ? 2 : 4)
					});
					divList.show();

					// 设置视口可显示的项数
					if (settings.list.length > settings.viewCount) {
						divList.css("height", divList.find("div:eq(0)").outerHeight(true) * settings.viewCount);
					}

					divListMask.css({
						left: left,
						top: $.browser.msie ? top - 1 : top,
						width: selectInputContainer.outerWidth() - ($.browser.msie ? 2 : 4) + 2,
						height: divList.outerHeight()
					});
					divListMask.show();
					setAutoHide(divList);
				}

				// 隐藏列表
				function hideList() {
					$("#" + $this.attr("id") + "_list", settings.context).hide();
					$("#" + $this.attr("id") + "_list_mask", settings.context).hide();
				}

				// 设置项的选中状态
				function setSelectedIndex(index) {
					var divList = $("#" + $this.attr("id") + "_list", settings.context);
					divList.find("div." + settings.itemSelectedCss).removeClass().addClass(settings.itemCss);
					var item = divList.find("div." + settings.itemCss + ":eq(" + index + ")");
					item.removeClass(settings.itemCss).addClass(settings.itemSelectedCss);
					if (item.length > 0) {
						scrollIntoView(item);
					}
				}

				// 获取有选中状态的项索引
				function getSelectedIndex() {
					var divList = $("#" + $this.attr("id") + "_list", settings.context);
					return divList.find("div").index(divList.find("div." + settings.itemSelectedCss));
				}

				// 初始化选中项
				function initSelected() {
					var hasSelected = false;
					$.each(settings.list, function(index, element) {
						if (element.text == selectInput.val()
							|| element.value == selectInput.val()) {
							setSelectedIndex(index);
							hasSelected = true;
							return false;
						}
					});
					if (!hasSelected) {
						setSelectedIndex(0);
					}
				}

				// 设置点击页面其他位置自动隐藏列表
				function setAutoHide($element, w) {
                    var selectInput = $("#" + $this.attr("id"), settings.context);
                    var str = "";
                    $.each(objList, function(index, element) {
                        str += selectInput.data("selfMousedown") + ",";
                    });
					var autoHide = function(event) {
                            /**
                             * 当点击控件本身时，由它们自己处理事件
                             * 不在控件上阻止事件冒泡到document，是为了防止两个selectInput控件互相不能隐藏对方的下拉框
                             */
                            if ($this.data("selfMousedown")) {
                                // 延迟修改解决IE在焦点进入文本框时也触发window的blur事件的问题
                                setTimeout(function() {
                                    $this.data("selfMousedown", false);
                                }, 0);
                                return;
                            };
						$element.hide();
						$("#" + $element.attr("id") + "_mask", settings.context).hide();
					};
					if (!w) {
						w = window.top;
					}
					var $document = $(w.document);
                    var bindFunc = $document.data("autoHide_" + $this.attr("id"));
                    $(w).unbind("blur", bindFunc).blur(autoHide);
                    $document.unbind("mousedown", bindFunc).mousedown(autoHide);
                    $document.data("autoHide_" + $this.attr("id"), autoHide);


					// 防止多次绑定事件处理函数
					/*if (!$document.data("autoHide_" + $this.attr("id"))) {
						$(w).blur(autoHide);
						$document.mousedown(autoHide);
						$document.data("autoHide_" + $this.attr("id"), autoHide);
					}*/
					$("frame, iframe", w.document).each(function(index, element) {
						setAutoHide($element, element.contentWindow);
					});
				}

				// 元素滚动到父容器视口
				function scrollIntoView(element, alignType) {
					var topY = element.position().top + element.parent().scrollTop();
					var bottomY = topY + element.outerHeight();

					if (alignType == "top") {
						if (bottomY > element.parent().scrollTop() + element.parent().height()
							|| topY < element.parent().scrollTop()) {
							element.parent().scrollTop(topY);
						}
					} else if (alignType == "bottom") {
						if (bottomY > element.parent().scrollTop() + element.parent().height()
							|| topY < element.parent().scrollTop()) {
							element.parent().scrollTop(bottomY - element.parent().height());
						}
					} else {
						// 元素部分或全部在视口下方时
						if (bottomY > element.parent().scrollTop() + element.parent().height()) {
							element.parent().scrollTop(bottomY - element.parent().height());
						}
						// 元素部分或全部在视口上方时
						if (topY < element.parent().scrollTop()) {
							element.parent().scrollTop(topY);
						}
					}
				}

				// 设置元素不能被选中
				function setUnselectable($element) {
					$element
						.attr("unselectable", "on")
						.css({
							"-moz-user-select": "-moz-none",
							"-moz-user-select":"none",
							"-o-user-select": "none",
							"-khtml-user-select": "none",
							"-webkit-user-select":"none",
							"-ms-user-select": "none",
							"user-select": "none"
						})
						.bind("selectstart", function() {
							return false;
						});
				}
			});
		},
		/**
		* 设置数据
		*/
		setData: function(data) {
			var $this = $(this);
			var settings = $this.data("SelectInput")["settings"];
			var isInList = false;

			$.each(settings.list, function(index, element) {
				if (element.value == data.value
					|| element.text == data.text) {
					$this.data("data", element);
					$this.find(":text").val(settings.autoConvertText ? element.text : data.text);
					isInList = true;
					return false;
				}
			});

			if (!isInList) {
				var mappingData = settings.mapping(data.text);
				if (mappingData === false) {
					var originalData = $this.data("data");
					if (originalData) {
						$this.find(":text").val(originalData.text);
					} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_FIRSTITEM) {
						$this.selectInput("setSelectedIndex", 0);
					} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_CLEAR) {
						$this.data("data", { text: "", value: "" });
						$this.find(":text").val("");
					}
					if (settings.oninvalidate) {
						settings.oninvalidate($(this), text);
					}
				} else {
					$this.data("data", mappingData);
					if (settings.autoAddItem) {
						settings.list.push(mappingData);
					}
					$this.find(":text").val(settings.autoConvertText ? mappingData.text : data.text);
				}
			}
		},
		/**
		* 设置值
		*/
		setValue: function(value) {
			var $this = $(this);
			var settings = $this.data("SelectInput")["settings"];
			var isInList = false;

			$.each(settings.list, function(index, element) {
				if (element.value == value) {
					$this.data("data", element);
					$this.find(":text").val(element.text);
					isInList = true;
					return false;
				}
			});

			if (!isInList) {
				$this.data("data", { text: value, value: value });
				if (settings.autoAddItem) {
					settings.list.push({ text: value, value: value });
				}
				$this.find(":text").val(value);
			}
		},
		/**
		* 设置文本
		*/
		setText: function(text) {
			var $this = $(this);
			var settings = $this.data("SelectInput")["settings"];
			var isInList = false;

			$.each(settings.list, function(index, element) {
				if (element.text == text) {
					$this.data("data", element);
					$this.find(":text").val(text);
					isInList = true;
					return false;
				}
			});

			if (!isInList) {
				var data = settings.mapping(text);
				if (data === false) {
					var originalData = $this.data("data");
					if (originalData) {
						$this.find(":text").val(originalData.text);
					} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_FIRSTITEM) {
						$this.selectInput("setSelectedIndex", 0);
					} else if (settings.invalidMode == $.SELECTINPUT_INVALID_MODE_CLEAR) {
						$this.data("data", { text: "", value: "" });
						$this.find(":text").val("");
					}
					if (settings.oninvalidate) {
						settings.oninvalidate($(this), text);
					}
				} else {
					$this.data("data", data);
					if (settings.autoAddItem) {
						settings.list.push(data);
					}
					$this.find(":text").val(settings.autoConvertText ? data.text : text);
				}
			}
		},
		/**
		* 获取数据
		*/
		getData: function() {
			var data = $(this).data("data");
			var settings = $(this).data("SelectInput")["settings"];
			var text = $(this).find(":text").val();
			var selectedIndex = -1;

			if (data) {
				return data;
			}

			$.each(settings.list, function(index, element) {
				if (element.text == text
					|| element.value == text) {
					selectedIndex = index;
					return false;
				}
			});
			return selectedIndex == -1 ? settings.mapping(text) === false ? { text: "", value: "" } : settings.mapping(text) : settings.list[selectedIndex];
		},
		/**
		* 获取值
		*/
		getValue: function() {
			return $(this).selectInput("getData").value;
		},
		/**
		* 获取文本
		*/
		getText: function() {
			return $(this).selectInput("getData").text;
		},
		/**
		* 设置选中索引
		*/
		setSelectedIndex: function(index) {
			var settings = $(this).data("SelectInput")["settings"];
			if (index > settings.list.length - 1) {
				index = settings.list.length - 1;
			}
			if (index < 0) {
				index = 0;
			}
			$(this).data("data", settings.list[index])
			$(this).find(":text").val(settings.list[index]["text"]);
		},
		/**
		* 获取选中索引
		*/
		getSelectedIndex: function() {
			var settings = $(this).data("SelectInput")["settings"];
			var text = $(this).find(":text").val();

			var selectedIndex = -1;
			$.each(settings.list, function(index, element) {
				if (element.text == text
					|| element.value == text) {
					selectedIndex = index;
					return false;
				}
			});
			return selectedIndex;
		},
        /**
         * 设置是否可用
         */
        setEnable: function(enabled) {
            $(this).find(":input").attr("disabled", !enabled);
        },
        /**
         * 设置列表项
         */
        setList: function (list) {
            var $this = $(this);
            var oldSettings = $this.data("SelectInput")["settings"];
            $this.selectInput($.extend(oldSettings, { list: list }));
        }
	};

	$.fn.selectInput = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.selectInput");
		}
	};
})(jQuery);