﻿/*
Slider控件
*/

(function($) {
    // 定义常量
    $.extend({
        // Slider方向
        SLIDER_ORIENTATION_HORIZONTAL: "horizontal",
        SLIDER_ORIENTATION_VERTICAL: "vertical"
    });

    var methods = {
        //初始化方法
        init: function(options) {
            //设定属性--覆盖默认属性
            var settings = $.extend({
                enable: true,
                orientation: $.SLIDER_ORIENTATION_HORIZONTAL, //方向
                maxWidth: 116, //滑动宽度（水平方向有效）
                maxHeight: 116, //滑动高度（垂直方向有效）
                minValue: 1, //最小值
                maxValue: 8, //最大值
                borderCss: "slider_border",
                vBorderCss: "slider_v_border",
                lineCss: "slider_line",
                vLineCss: "slider_v_line",
                thumbCss: "slider_thumb",
                value: 1,
                buildTip: function(value) {
                    return value;
                },
                valueChangeEvent: null,
                setValueCompleteEvent: null
            }, options);

            // 解决IE8以下OCX加载后滑动条拖动不正常问题的hack
            var hack = $("#___hack");
            if (hack.size() == 0) {
                hack = $("<div id=\"___hack\"></div>");
                $(this).append(hack);
            }

            //设定属性--缓存属性到data中
            return this.each(function() {
                var $this = $(this);
                $this.data('InteData', {
                    target: $this,
                    settings: settings
                });

                var id = $this.attr("id");
                var isHorizontal = settings.orientation == $.SLIDER_ORIENTATION_HORIZONTAL;

                var line = $("<div id=\"" + id + "_line\" class=\"" + (isHorizontal ? settings.lineCss : settings.vLineCss) + "\" ></div>");
                var thumb = $("<div id=\"" + id + "_thumb\" class=\"" + settings.thumbCss + "\" ></div>");

                line.appendTo($this);
                thumb.appendTo($this);
                $this.addClass(isHorizontal ? settings.borderCss : settings.vBorderCss);

                thumb.bind("mousedown", sliderMove);

                line.bind("mousedown", function(e) {
                    if (!settings.enable)
                        return false;
                    getPos(e, thumb, Math.round((isHorizontal ? thumb.width() : thumb.height()) / 2));
                    sliderMove(e);
                });
                $this.attr("title", settings.buildTip(settings.value));

                function sliderMove(e) {
                    if (!settings.enable)
                        return false;
                    if (e.stopPropagation)
                        e.stopPropagation();
                    DragSetCapture(this);
                    var pos = mousePosition(e);
                    var innerOffset = isHorizontal ? pos.x - thumb.offset().left : thumb.offset().top + thumb.height() - pos.y;
                    $("body").css('cursor', 'pointer');
                    $(document).mousemove(function(e) {
                        // 解决IE8以下OCX加载后滑动条拖动不正常问题的hack
                        hack.html(" ");
                        getPos(e, thumb, innerOffset);
                    });

                    $(document).bind("mouseup", function(e) {
                        var setting = $this.data("InteData").settings;

                        $("body").css('cursor', 'default');
                        DragReleaseCapture(this);
                        $(document).unbind("mousemove");
                        $(document).unbind("mouseup");

                        if (setting.setValueCompleteEvent) {
                            var minLeft = line.offset().left;
                            var minBottom = line.offset().top + line.height();
                            var maxWidth = setting.maxWidth;
                            var maxHeight = setting.maxHeight;
                            var minValue = setting.minValue;
                            var maxValue = setting.maxValue;
                            var pos = mousePosition(e);
                            var now = 0;

                            if (isHorizontal) {
                                if (pos.x < minLeft + innerOffset)
                                    now = 0;
                                else
                                    now = pos.x - minLeft - innerOffset; // 目前的值
                                if (now > maxWidth) {
                                    now = maxWidth; // 大于原始值，不拖动
                                }
                            } else {
                                if (pos.y > minBottom - innerOffset)
                                    now = 0;
                                else
                                    now = minBottom - innerOffset - pos.y; // 目前的值
                                if (now > maxHeight) {
                                    now = maxHeight; // 大于原始值，不拖动
                                }
                            }
                            setting.setValueCompleteEvent(Math.round(now * (maxValue - minValue) / (isHorizontal ? maxWidth : maxHeight)) + minValue);
                        }
                    });
                }

                /*
                * 设置拖动块的位置
                */
                function getPos(e, mover, innerOffset) {
                    var setting = $this.data('InteData').settings;
                    var minLeft = line.offset().left;
                    var minBottom = line.offset().top + line.height();
                    var maxWidth = setting.maxWidth;
                    var maxHeight = setting.maxHeight;
                    var minValue = setting.minValue;
                    var maxValue = setting.maxValue;
                    var pos = mousePosition(e);
                    var now = 0;
                    if (isHorizontal) {
                        if (pos.x < minLeft + innerOffset)
                            now = 0;
                        else
                            now = pos.x - minLeft - innerOffset; // 目前的值
                        if (now > maxWidth) {
                            now = maxWidth; // 大于原始值，不拖动
                        }
                        $(mover).css({
                            left: now
                        });
                    } else {
                        if (pos.y > minBottom - innerOffset)
                            now = 0;
                        else
                            now = minBottom - innerOffset - pos.y; // 目前的值
                        if (now > maxHeight) {
                            now = maxHeight; // 大于原始值，不拖动
                        }
                        $(mover).css({
                            top: maxHeight - now
                        });
                    }
                    var value = Math.round(now * (maxValue - minValue) / (isHorizontal ? maxWidth : maxHeight)) + minValue;
                    $this.attr("title", setting.buildTip(value));
                    line.attr("title", setting.buildTip(value));
                    setting.value = value;
                    if (setting.valueChangeEvent) {
                        setting.valueChangeEvent(value);
                    }
                }

                /*
                * 获取鼠标的位置
                */
                function mousePosition(ev) {
                    if (ev.pageX || ev.pageY) {
                        return {
                            x: ev.pageX,
                            y: ev.pageY
                        };
                    }
                    return {
                        x: ev.clientX + document.body.scrollLeft - document.body.clientLeft,
                        y: ev.clientY + document.body.scrollTop - document.body.clientTop
                    };
                }
                methods.setValue.apply(this, [settings.value])
            });
        },
        /**
        * 获取值
        * @param time
        */
        getValue: function() {
            var setting = $(this).data('InteData').settings;
            return setting.value * 1;
        },

        /**
        * 设置值
        */
        setValue: function(value) {
            var setting = $(this).data('InteData').settings;
            var isHorizontal = setting.orientation == $.SLIDER_ORIENTATION_HORIZONTAL;
            if (value > setting.maxValue) {
                value = setting.maxValue;
            }
            else if (value < setting.minValue) {
                value = setting.minValue
            }
            var now = (value - setting.minValue) * (isHorizontal ? setting.maxWidth : setting.maxHeight) / (setting.maxValue - setting.minValue);
            $("#" + $(this).attr("id") + "_thumb").css(isHorizontal ? {
                left: now
            } : {
                top: setting.maxHeight - now
            });
            $(this).attr("title", setting.buildTip(value));
            $("#" + $(this).attr("id") + "_line").attr("title", setting.buildTip(value));
            setting.value = value;
            if (setting.valueChangeEvent) {
                setting.valueChangeEvent(value);
            }
            
        },

        /**
        * 设置值并触发事件
        */
        setValueWithEvent: function(value) {
            var setting = $(this).data("InteData").settings;
            methods.setValue.apply(this, [ value ]);
            if (setting.setValueCompleteEvent) {
                setting.setValueCompleteEvent(value);
            }
        },

        /**
        * 设置属性
        */
        setProperty: function(key, value) {
            var setting = $(this).data('InteData').settings;
            if (key in setting) {
                setting[key] = value
            }
        },

        /**
        * 获取属性
        */
        getProperty: function(key) {
            var setting = $(this).data('InteData').settings;
            if (key in setting) {
                return setting[key];
            }
            return null;
        }
    };

    $.fn.slider = function(method) {
        if (methods[method]) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('Method ' + method + ' does not exist on jQuery.tab');
        }
    };

})(jQuery);