﻿/*
TimeInput控件
*/

(function($) {
	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				css: "time_input", // 控件的默认样式
				format: "hh:mm:ss", // 时间模式
				defaultValue: "00:00:00", // 默认值
				lenient: true, // 是否宽松
				filterTimeSpans: [] // 需要过滤掉的时间段
			}, options);

			return this.each(function() {

                var $this = $(this);
                $this.data("TimeInput", {
                    target: $this,
                    settings: settings
                });

				this.style.imeMode = "disabled";
				$(this).addClass(settings.css);
				$(this).attr("disabled", !settings.enable);
				$(this).val(settings.defaultValue);
				$(this).bind("paste", function(event) { return false; });
				$(this).keydown(function(event) {
					var keyCode = event.which;

					if (keyCode == 9// Tab
						|| keyCode == 16// Shift
						|| keyCode == 17// Ctrl
						|| keyCode == 18// Alt
						|| keyCode == 35// End
						|| keyCode == 36// Home
						|| keyCode == 37// 向左
						|| keyCode == 39// 向右
						|| keyCode == 13// 回车
					) {
						return true;
					}

					var splitPoints = [];
					var index = -1;
					while ((index = settings.format.indexOf(":", index + 1)) != -1) {
						splitPoints.push({ index: index, length: 1 });
					}

					var format = settings.format;
					var timePartFrags = [ false, false, false ];
					var formatFrags = [ "hh", "mm", "ss" ];
					var regFrags = [ "(2[0-3]|[01]?\\d)", "([0-5]?\\d)", "([0-5]?\\d)" ];
					for (var i = 0; i < formatFrags.length; i++) {
						var formatIndex = format.indexOf(formatFrags[i]);
						if (formatIndex != -1) {
							timePartFrags[i] = true;
							format = format.replace(formatFrags[i], regFrags[i]);
						}
					}
					var reg = new RegExp("^" + format + "$");

					var range = getSelectRange(this);
					var isSplitIndex = false;
					var isAfterSplitIndex = false;
					var index = 0;
					for (; index < splitPoints.length; index++) {
						if (splitPoints[index]["index"] == range.start) {
							isSplitIndex = true;
							break;
						}
						if (splitPoints[index]["index"] == range.start - splitPoints[index]["length"]) {
							isAfterSplitIndex = true;
							break;
						}
					}
					var $this = $(this);
					var oldText = $this.val();
					var startIndex = range.start + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var endIndex = range.start + 1 + (isSplitIndex ? splitPoints[index]["length"] : 0);
					var letter = String.fromCharCode(keyCode);
					var posIndex = endIndex;

					if (keyCode >= 96 && keyCode <= 105) {// 数字小键盘
						letter = String.fromCharCode(keyCode - 48);
					}
					if (event.shiftKey && keyCode == 186) {// 冒号
						letter = oldText.charAt(startIndex);
						var splitIndex = oldText.indexOf(":", isSplitIndex ? startIndex - 1 : startIndex);
						if (splitIndex != -1) {
							posIndex = splitIndex + 1;
						} else {
							posIndex = oldText.length;
						}
					}
					if (keyCode == 38 || keyCode == 40) {// 向上或向下
						posIndex = isSplitIndex ? startIndex - 1 : startIndex;
						var splitBefore = oldText.lastIndexOf(":", isSplitIndex ? posIndex - 1 : posIndex);
						var splitAfter = oldText.indexOf(":", posIndex);
						if (splitAfter == -1) {
							splitAfter = oldText.length;
						}
						startIndex = splitBefore + 1;
						endIndex = splitAfter;
						
						var timePart = oldText.substring(startIndex, endIndex) * 1;
						timePart += keyCode == 38 ? 1 : -1;
						letter = timePart < 10 ? "0" + timePart : timePart;
					}
					if (keyCode == 8) {// 退格
						startIndex = range.start - 1 - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						endIndex = range.start - (isAfterSplitIndex ? splitPoints[index]["length"] : 0);
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 46) {// 删除
						letter = "0";
						posIndex = startIndex;
					}
					if (keyCode == 32) {// 空格
						letter = "0";
					}
					var newText = oldText.substring(0, startIndex) + letter + oldText.substring(endIndex);
					if (reg.test(newText)) {
						// 过滤掉时间段
						var len = settings.filterTimeSpans.length;
						if (len > 0) {
							var totalSeconds = 0;
							for (var i = 0; i < timePartFrags.length; i++) {
								if (timePartFrags[i]) {
									var multiple = i == 0 ? 3600 : i == 1 ? 60 : 1;
									totalSeconds += newText.substr(settings.format.indexOf(formatFrags[i]), formatFrags[i].length) * multiple;
								}
							}
							for (var i = 0; i < len; i++) {
								var startTotalSeconds = 0;
								var endTotalSeconds = 0;
								for (var j = 0; j < timePartFrags.length; j++) {
									if (timePartFrags[j]) {
										var multiple = j == 0 ? 3600 : j == 1 ? 60 : 1;
										startTotalSeconds += settings.filterTimeSpans[i][0].substr(settings.format.indexOf(formatFrags[j]), formatFrags[j].length) * multiple;
										endTotalSeconds += settings.filterTimeSpans[i][1].substr(settings.format.indexOf(formatFrags[j]), formatFrags[j].length) * multiple;
									}
								}
								if (totalSeconds >= startTotalSeconds && totalSeconds <= endTotalSeconds) {
									return false;
								}
							}
						}
						$this.val(newText);
						setSelectRange(this, posIndex, posIndex);
					} else if (settings.lenient && (keyCode == 38 || keyCode == 40)) {
						var formatParts = [];
						for (var i = 0; i < timePartFrags.length; i++) {
							if (timePartFrags[i]) {
								formatParts.push("(\\d-?\\d)");
							}
						}
						var reg = new RegExp("^" + formatParts.join(":") + "$");
						var timeParts = reg.exec(newText);
						if (timeParts) {
							var hourParts;
							var minuteParts;
							var secondParts;
							var hour;
							var minute;
							var second;
							var minute2Hour = 0;
							var second2Minute = 0;

							if (timePartFrags[0]) {
								hourParts = /^(\d)(-?\d)$/.exec(timeParts[1]);
							}
							if (timePartFrags[1]) {
								minuteParts = /^(\d)(-?\d)$/.exec(timeParts[timePartFrags[0] ? 2 : 1]);
							}
							if (timePartFrags[2]) {
								secondParts = /^(\d)(-?\d)$/.exec(timeParts[timePartFrags[0] && timePartFrags[1] ? 3 : timePartFrags[0] || timePartFrags[1] ? 2 : 1]);
							}

							if (secondParts) {
								second = secondParts[1] * 10 + secondParts[2] * 1;
								second2Minute = second < 0 ? -1 : second > 59 ? 1 : 0;
								second = (second + 60) % 60;
							}
							if (minuteParts) {
								minute = minuteParts[1] * 10 + minuteParts[2] * 1 + second2Minute;
								minute2Hour = minute < 0 ? -1 : minute > 59 ? 1 : 0;
								minute = (minute + 60) % 60;
							}
							if (hourParts) {
								hour = (hourParts[1] * 10 + hourParts[2] * 1 + minute2Hour + 24) % 24;
							}
							
							$this.val($.map([ hour, minute, second ], function(element) {
								return element < 10 ? "0" + element : element;
							}).join(":"));
							setSelectRange(this, posIndex, posIndex);
						}
					}
					return false;
				});
			});

			// 获取选中区间
			function getSelectRange(textControl) {
				var range = {};
				if (document.selection) {
					if (textControl.tagName != undefined && textControl.tagName == "INPUT") {
						var selectionRange = document.selection.createRange();
						var textControlRange = textControl.createTextRange();
						range.text = selectionRange.text;
						var oldLength = range.text.length;
						selectionRange.setEndPoint("EndToStart", textControlRange);
						var newLength = selectionRange.text.length;
						range.start = newLength;
						range.end = newLength + oldLength;
					}
				} else {
					range.start = textControl.selectionStart;
					range.end = textControl.selectionEnd;
					range.text = textControl.value.substring(range.start, range.end);
				}
				return range;
			}

			// 设置选中区间
			function setSelectRange(textControl, startIndex, endIndex) {
				if (document.selection) {
					var range = textControl.createTextRange();
					range.collapse(true);
					range.moveStart("character", startIndex);
					range.moveEnd("character", endIndex - startIndex);
					range.select();
				} else {
					textControl.setSelectionRange(startIndex, endIndex);
					textControl.focus();
				}
			}
		},
        /**
         * 设置列表项
         */
        setFormat: function (format) {
            var $this = $(this);
            var oldSettings = $this.data("TimeInput")["settings"];
            $this.timeInput($.extend(oldSettings, { format: format }));
        }
	};

	$.fn.timeInput = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.timeInput");
		}
	};
})(jQuery);