﻿/*
Timeline控件
*/

(function($) {
	// 定义常量
	$.extend({
		// Timeline修改模式
		TIMELINE_MODE_ADD: "add",
		TIMELINE_MODE_ERASE: "erase",

		// Timeline当前状态
		TIMELINE_STATE_NORMAL: "normal",
		TIMELINE_STATE_SELECTING: "selecting",

		// Timeline输入输出数据类型
		TIMELINE_DATA_MODE_DATE: "Date",
		TIMELINE_DATA_MODE_STRING: "String",
		TIMELINE_DATA_MODE_SIMPLE: "Simple",

		// Timeline正在选择时间显示位置
		TIMELINE_SELECTING_TIME_ABOVE: "above",
		TIMELINE_SELECTING_TIME_BELOW: "below",
		TIMELINE_SELECTING_TIME_FIX: "fix"
	});

	var methods = {
		// 初始化
		init: function(options) {
			var settings = $.extend({
				enable: true, // 是否启用
				readonly: false, // 是否只读
				context: document,// 上下文环境
				mode: $.TIMELINE_MODE_ADD, // 选择模式，可选项：TIMELINE_MODE_ADD（添加），TIMELINE_MODE_ERASE（擦除）
				contentContainerCss: "time_line_content_container", // 时间线内容容器默认样式
				labelContainerCss: "time_line_label_container", // 时间线标签容器默认样式
				labelCss: "time_line_label", // 时间线刻度默认样式
				paddingLeftCss: "time_line_padding_left", // 时间线左间距默认样式
				lineCss: "time_line", // 时间线默认样式
				lineUnitCss: "time_line_unit", // 时间线单元块默认样式
				paddingRightCss: "time_line_padding_right", // 时间线右间距默认样式
				barCss: "time_line_bar", // 时间线功能条默认样式
				selectingTimeCss: "time_line_selecting_time", // 时间线当前选择时间默认样式
				infoCss: "time_line_info", // 时间线信息默认样式
				functionCss: "time_line_function", // 时间线功能区默认样式
				manualCss: "time_line_manual", // 时间线手动输入按钮样式
				manualDisabledCss: "time_line_manual_disabled", // 时间线手动输入按钮禁用样式
				selectAllCss: "time_line_selectAll", // 时间线全选按钮样式
				selectAllDisabledCss: "time_line_selectAll_disabled", // 时间线全选按钮禁用样式
				inverseCss: "time_line_inverse", // 时间线反选按钮样式
				inverseDisabledCss: "time_line_inverse_disabled", // 时间线反选按钮禁用样式
				clearCss: "time_line_clear", // 时间线清空按钮样式
				clearDisabledCss: "time_line_clear_disabled", // 时间线清空按钮禁用样式
				customCss: "time_line_custom", // 时间线自定义按钮样式
				customDisabledCss: "time_line_custom_disabled", // 时间线自定义按钮禁用样式
				selectingCss: "time_line_selecting", // 时间线选择中样式
				selectedCss: "time_line_selected", // 时间线选择后样式
				erasingCss: "time_line_erasing", // 时间线擦除中样式
				manualControlCss: "time_line_manual_control", // 时间线手动输入控件样式
				manualInputCss: "time_line_manual_input", // 时间线手动输入文本框样式
				manualButtonCss: "time_line_manual_button", // 时间线手动输入确定按钮样式
				paddingLeft: 10,// 左间距宽度
				paddingRight: 10,// 右间距宽度
				startTime: null, // 开始时间
				endTime: null, // 结束时间
				manualStartTime: null, // 时间线手动输入开始时间
				manualEndTime: null, // 时间线手动输入结束时间
				manualStartFilters: [ [ "23:59", "23:59" ] ], // 时间线手动输入开始时间过滤
				manualEndFilters: [ [ "00:00", "00:00" ] ], // 时间线手动输入结束时间过滤
				labelInterval: 4, // 标签间隔
				labelFormat: "h", // 标签格式
				showMajorGraduation: true, // 是否显示主刻度
				majorGraduationInterval: 4, // 主刻度间隔
				showMinorGraduation: true, // 是否显示次刻度
				minorGraduationInterval: 1, // 次刻度间隔
				showSelectingTime: true, // 是否显示正在选择的时间
				selectingTimePos: $.TIMELINE_SELECTING_TIME_FIX, // 正在选择的时间显示位置
				showTimes: true, // 显示时间线的时间
				timeFormat: "hh:mm", // 显示时间的格式
				maxTimeShow: 0, // 显示时间的最大项数
				showManualButton: true, // 显示手动输入按钮
				manualButtonText: "手动输入", // 手动输入按钮文字
				showSelectAllButton: true, // 显示全选按钮
				selectAllButtonText: "全选", // 全选按钮文字
				showInverseButton: true, // 显示反选按钮
				inverseButtonText: "反选", // 反选按钮文字
				showClearButton: true, // 显示清除按钮
				clearButtonText: "清除", // 清除按钮文字
				okButtonText: "确定", // 确定按钮文字
				timeSeparator: "-", // 简单时间格式时的开始和结束时间分隔符
				customButtons: [], // 用户自定义按钮集合
				onselected: null, // 选择完成触发的事件
				onvaluechanged: null, // 值改变触发的事件
				oninvalidate: null // 手动输入不合法
			}, options);

			// 设定属性——缓存属性到data中
			return this.each(function() {
				var $this = $(this);

				// 初始化时间线单元块集合
				var units = new Array(96);
				for (var i = 0; i < units.length; i++) {
					units[i] = [];
				}

				if (!settings.startTime || !settings.endTime) {
					settings.startTime = new Date();
					settings.startTime.setHours(0, 0, 0, 0);
					settings.endTime = new Date(settings.startTime.getTime());
					settings.endTime.setDate(settings.endTime.getDate() + 1);
				}

				$this.data("Timeline", {
					target: $this,
					settings: settings,
					units: units,
					state: $.TIMELINE_STATE_NORMAL,
					unitTimeSpan: (settings.endTime.getTime() - settings.startTime.getTime()) / units.length,
					startIndex: -1,
					endIndex: -1
				});

				var containerId = $this.attr("id");

				var contentContainer = $("<div id=\"" + containerId + "_time_line_content_container\" style=\"-moz-user-select: none; width: 100%;\" class=\"" + settings.contentContainerCss + "\"></div>");
				var labelContainer = $("<div id=\"" + containerId + "_time_line_label\" style=\"-moz-user-select: none; width: 100%;\" class=\"" + settings.labelContainerCss + "\"></div>");
				var timeline = $("<div id=\"" + containerId + "_time_line\" style=\"cursor: " + (settings.enable ? settings.readonly ? "default" : "text" : "not-allowed") + ";\" class=\"" + settings.lineCss + "\"></div>");
				var barContainer = $("<div id=\"" + containerId + "_time_line_bar\" style=\"-moz-user-select: none; width: 100%;\" class=\"" + settings.barCss + "\"></div>");
				var paddingLeft = $("<div id=\"" + containerId + "_time_line_padding_left\" style=\"-moz-user-select: none; width: " + settings.paddingLeft + "px;\" class=\"" + settings.paddingLeftCss + "\"></div>");
				var paddingRight = $("<div id=\"" + containerId + "_time_line_padding_right\" style=\"-moz-user-select: none; width: " + settings.paddingRight + "px;\" class=\"" + settings.paddingRightCss + "\"></div>");

				contentContainer.append(labelContainer);
				contentContainer.append(paddingLeft);
				contentContainer.append(timeline);
				contentContainer.append(paddingRight);
				$this.append(contentContainer);

				if (settings.showTimes || (settings.showSelectingTime && settings.selectingTimePos == $.TIMELINE_SELECTING_TIME_FIX)
					|| settings.showManualButton || settings.showSelectAllButton || settings.showInverseButton || settings.showClearButton || settings.customButtons.length > 0) {
					$this.append(barContainer);
					if (settings.showTimes) {
						var infoContainer = $("<div id=\"" + containerId + "_time_line_info\" style=\"-moz-user-select: none;\" class=\"" + settings.infoCss + "\"></div>");
						barContainer.append(infoContainer);
					}

					var functionContainer = $("<div id=\"" + containerId + "_time_line_function\" style=\"-moz-user-select: none;\" class=\"" + settings.functionCss + "\"></div>");
					barContainer.append(functionContainer);

					if (settings.showSelectingTime && settings.selectingTimePos == $.TIMELINE_SELECTING_TIME_FIX) {
						var selectingTime = $("<div id=\"" + containerId + "_time_line_selecting_time\" style=\"-moz-user-select: none; display: inline-block; *display: inline;\" class=\"" + settings.selectingTimeCss + "\">&nbsp;</div>");
						functionContainer.append(selectingTime);
					}

					$.each(settings.customButtons, function(index, element) {
						var customButton = $("<a id=\"" + containerId + "_time_line_custom_" + element["name"] + "\" href=\"javascript:void(0)\" class=\"" + (settings.enable ? settings.customCss : settings.customDisabledCss) + "\">" + element["text"] + "</a>");
						functionContainer.append(customButton);
						customButton.attr("disabled", !settings.enable);
						customButton.click(function(event) {
							if (typeof element["click"] == "function") {
								element["click"].call(customButton, $this);
							}
						});
					});

					if (settings.showManualButton && !settings.readonly) {
						var manualButton = $("<a id=\"" + containerId + "_time_line_manual\" href=\"javascript:void(0)\" class=\"" + (settings.enable ? settings.manualCss : settings.manualDisabledCss) + "\">" + settings.manualButtonText + "</a>");
						functionContainer.append(manualButton);
						manualButton.attr("disabled", !settings.enable);
						manualButton.click(function(event) {
							$this.timeline("showManualControl");
						});
					}

					if (settings.showSelectAllButton && !settings.readonly) {
						var selectAllButton = $("<a id=\"" + containerId + "_time_line_selectAll\" href=\"javascript:void(0)\" class=\"" + (settings.enable ? settings.selectAllCss : settings.selectAllDisabledCss) + "\">" + settings.selectAllButtonText + "</a>");
						functionContainer.append(selectAllButton);
						selectAllButton.attr("disabled", !settings.enable);
						selectAllButton.click(function(event) {
							$this.timeline("selectAll");
						});
					}

					if (settings.showInverseButton && !settings.readonly) {
						var inverseButton = $("<a id=\"" + containerId + "_time_line_inverse\" href=\"javascript:void(0)\" class=\"" + (settings.enable ? settings.inverseCss : settings.inverseDisabledCss) + "\">" + settings.inverseButtonText + "</a>");
						functionContainer.append(inverseButton);
						inverseButton.attr("disabled", !settings.enable);
						inverseButton.click(function(event) {
							$this.timeline("inverse");
						});
					}

					if (settings.showClearButton && !settings.readonly) {
						var clearButton = $("<a id=\"" + containerId + "_time_line_clear\" href=\"javascript:void(0)\" class=\"" + (settings.enable ? settings.clearCss : settings.clearDisabledCss) + "\">" + settings.clearButtonText + "</a>");
						functionContainer.append(clearButton);
						clearButton.attr("disabled", !settings.enable);
						clearButton.click(function(event) {
							$this.timeline("clearTimes");
						});
					}

					// 时间线的时间自适应宽度
					if (settings.showTimes && settings.maxTimeShow <= 0) {
						$("#" + containerId + "_time_line_info", settings.context).width($this.width() - functionContainer.width() - 100);
					}
				}

				// 禁止标签文字被选中产生干扰
				labelContainer.bind("selectstart", function() { return false; });

				var id = $this.attr("id");
				var width = $this.width();

				// 总宽度自动调整，防止出现单元块宽度小数点情况
				if (width % units.length != 0) {
					width -= width % units.length;
				}
                $this.css("width", width);
                timeline.css("width", width);
                barContainer.css("width", width);

				$this.css("width", $this.width() + settings.paddingLeft + settings.paddingRight);
				$this.css("min-width", $this.width());
				labelContainer.css("left", settings.paddingLeft);
				barContainer.css("width", barContainer.width() + settings.paddingLeft + settings.paddingRight);

				var objFragment = document.createDocumentFragment();

				// 添加时间线单元
				var unitWidth = width / units.length;
				for (var i = 0; i < units.length; i++) {
					var unitDiv = $("<div index=\"" + i + "\" style=\"-moz-user-select: none; position: absolute; width: " + unitWidth + "px; height: " + timeline.css("height") + "; top: 0px; left: " + (i * unitWidth) + "px\" class=\"" + settings.lineUnitCss + "\"></div>");
					unitDiv.mousedown(onSelectStart);
					unitDiv.mouseover(onSelect);
					unitDiv.mouseover(showSelectStartLine);
					unitDiv.mouseout(hideSelectStartLine);
					unitDiv.mouseup(onSelectEnd);
					unitDiv.bind("dragstart", function(e) { return false; }); // 禁止元素产生拖拽效果
					objFragment.appendChild(unitDiv[0]);
				}

				// 为了提高性能，先将节点添加到文档碎片中，然后一次性添加到页面上，减少DOM结构改变的次数
				timeline.append($(objFragment));

				// 像素级别的校准
				unitDivFirst = timeline.find("div[index=0]");
                unitDivFirst.css("border-left", "1px solid #000000");
				unitDivFirst.css("left", "0px");

                unitDivLast = timeline.find("div[index=" + (units.length - 1) + "]");
                unitDivLast.css("border-right", "1px solid #000000");
                unitDivLast.css("width", unitWidth - 1 + "px");

				// 在页面任何位置松开都结束选择中状态
				$(settings.context).mouseup(onSelectEnd);

				if (settings.showSelectingTime && settings.selectingTimePos != $.TIMELINE_SELECTING_TIME_FIX) {
					timeline.mouseleave(function(event) {
						$("#" + id + "_time_line_selecting_time", settings.context).hide();
					});
				}

				createTimeLabel();
//				createUnitTip();

				// 生成时间标签
				function createTimeLabel() {
					labelContainer.html("");
					var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
					var labelCount = Math.ceil(units.length / settings.labelInterval);
					var labelDate = new Date(settings.startTime.getTime());
					var hasZero = false;

					var objFragment = document.createDocumentFragment();
					for (var i = 0; i < labelCount + 1; i++) {
						var left = unitWidth * settings.labelInterval * i;
						var labelText = $this.timeline("formatTime", labelDate, settings.labelFormat);
						// 再次遇到0点时，标签变为24点
						if (hasZero && labelDate.getHours() == 0) {
							var regResult = /h{1,2}/.exec(settings.labelFormat);
							if (regResult) {
								labelText = labelText.substring(0, regResult.index) + "24" + labelText.substring(regResult.index + regResult[0].length);
							}
						}
						var labelDiv = $("<div style=\"position: absolute; left: " + left + "px; top: 0px;\" class=\"" + settings.labelCss + "\">" + labelText + "</div>");
						objFragment.appendChild(labelDiv[0]);
						if (labelDate.getHours() == 0) {
							hasZero = true;
						}
						labelDate.setTime(labelDate.getTime() + unitTimeSpan * settings.labelInterval);
					}

					labelContainer.append($(objFragment));

					// 使标签相对刻度居中
					$("div." + settings.labelCss, $this).each(function(index, element) {
						var $this = $(this);
						$this.css("left", parseInt($this.css("left")) - $this.width() / 2);
					});

					if (settings.showMajorGraduation || settings.showMinorGraduation) {
						objFragment = document.createDocumentFragment();
						for (var i = 0; i < units.length + 1; i++) {
							var graduationDiv = $("<div style=\"border-left: 1px solid #000000; position: absolute; width: " + unitWidth + "px; height: " + (i % settings.majorGraduationInterval == 0 ? (settings.showMajorGraduation ? 5 : 0) : i % settings.minorGraduationInterval == 0 ? (settings.showMinorGraduation ? 2 : 0) : 0) + "px; bottom: 0px; left: " + (i * unitWidth) + "px\"></div>");
							// 像素级别的校准
							if (i == units.length) {
								graduationDiv.css("left", i * unitWidth - 1 + "px");
							}
							objFragment.appendChild(graduationDiv[0]);
						}

						labelContainer.append($(objFragment));
					}
				}

				// 生成单元块时间提示
				function createUnitTip() {
					var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
					var unitDate = new Date(settings.startTime.getTime());
					$this.find("[index]").each(function() {
						var index = parseInt($(this).attr("index"));
						unitDate.setTime(settings.startTime.getTime() + unitTimeSpan * index);
						$(this).attr("title", $this.timeline("formatTime", unitDate, settings.timeFormat));
					});
				}

				// 开始选择时间
				function onSelectStart(e) {
					// 若时间线被禁用或只读则不进行任何操作
					if (!settings["enable"] || settings["readonly"]) {
						return;
					}

					// 若不是点击的左键也不进行任何操作
					if (e.which != 1) {
						return;
					}

					var unitDiv = $(this);
					$this.data("Timeline")["endIndex"] = $this.data("Timeline")["startIndex"] = parseInt(unitDiv.attr("index"));
					$this.data("Timeline")["state"] = $.TIMELINE_STATE_SELECTING;

					if (settings.showSelectingTime && settings.selectingTimePos != $.TIMELINE_SELECTING_TIME_FIX) {
						var selectingTime = $("<div id=\"" + $this.attr("id") + "_time_line_selecting_time\" style=\"-moz-user-select: none; position: absolute;\" class=\"" + settings.selectingTimeCss + "\"></div>");
						$this.append(selectingTime);
					}
				}

				// 选择时间
				function onSelect(e) {
					// 若时间线被禁用或只读则不进行任何操作
					if (!settings["enable"] || settings["readonly"]) {
						return;
					}

					// 若为选择中状态
					if ($this.data("Timeline")["state"] == $.TIMELINE_STATE_SELECTING) {
						var unitDiv = $(this);
						var startIndex = $this.data("Timeline")["startIndex"];
						var units = $this.data("Timeline")["units"];
						var endIndex = $this.data("Timeline")["endIndex"] = parseInt(unitDiv.attr("index"));
						var className = settings[settings["mode"] == $.TIMELINE_MODE_ADD ? "selectingCss" : "erasingCss"];

						if (startIndex > endIndex) {
							var tempIndex = endIndex;
							endIndex = startIndex;
							startIndex = tempIndex;
						}

						if (settings.showSelectingTime) {
							var lineStartTime = settings["startTime"];
							var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
							var endTime = new Date(lineStartTime.getTime() + (endIndex + 1) * unitTimeSpan);
							var endTimeText = $this.timeline("formatTime", endTime, settings.timeFormat);
							// 结束时间遇到0点时，变为24点
							if (endTime.getHours() == 0 && endTime.getMinutes() == 0) {
								var regResult = /h{1,2}/.exec(settings.timeFormat);
								if (regResult) {
									endTimeText = endTimeText.substring(0, regResult.index) + "24" + endTimeText.substring(regResult.index + regResult[0].length);
								}
							}
							var timeSpan = $this.timeline("formatTime", new Date(lineStartTime.getTime() + startIndex * unitTimeSpan), settings.timeFormat)
								+ settings.timeSeparator
								+ endTimeText;
							var selectingTime = $("#" + $this.attr("id") + "_time_line_selecting_time", settings.context);
							var timeline = $("#" + $this.attr("id") + "_time_line", settings.context);
							var position = timeline.position();
							selectingTime.html(timeSpan);
							if (settings.selectingTimePos != $.TIMELINE_SELECTING_TIME_FIX) {
								selectingTime.css("left", e.pageX - selectingTime.outerWidth() / 2);
								selectingTime.css("top", settings.selectingTimePos == $.TIMELINE_SELECTING_TIME_ABOVE
									? position.top - 1 - selectingTime.outerHeight()
									: position.top + 2 + timeline.outerHeight()
								);
							}
							if (selectingTime.is(":hidden")) {
								selectingTime.show();
							}
						}

						$this.find("[index]").each(function() {
							var unit = $(this);
							var index = parseInt(unit.attr("index"));
							unit.removeClass();
							if (index >= startIndex && index <= endIndex) {
								unit.addClass(className);
							} else {// 其他单元块回复原来的样式
								var orignalClassName = settings[units[index].length == 0 ? "lineUnitCss" : "selectedCss"];
								unit.addClass(orignalClassName);
							}
						});
					}
				}

				// 结束选择时间
				function onSelectEnd(e) {
					// 若时间线被禁用或只读则不进行任何操作
					if (!settings["enable"] || settings["readonly"]) {
						return;
					}

					// 若为选择中状态
					if ($this.data("Timeline") && $this.data("Timeline")["state"] == $.TIMELINE_STATE_SELECTING) {
						var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
						var startIndex = $this.data("Timeline")["startIndex"];
						var endIndex = $this.data("Timeline")["endIndex"];
						var units = $this.data("Timeline")["units"];

						if (startIndex > endIndex) {
							var tempIndex = endIndex;
							endIndex = startIndex;
							startIndex = tempIndex;
						}

						if (settings.showSelectingTime) {
							$("#" + $this.attr("id") + "_time_line_selecting_time", settings.context)[ settings.selectingTimePos != $.TIMELINE_SELECTING_TIME_FIX ? "remove" : "hide" ]();
						}

						$this.find("[index]").each(function() {
							var index = parseInt($(this).attr("index"));
							if (index >= startIndex && index <= endIndex) {
								units[index] = [];
								if (settings["mode"] == $.TIMELINE_MODE_ADD) {
									var startDate = new Date(settings.startTime.getTime());
									startDate.setTime(settings.startTime.getTime() + unitTimeSpan * index);
									var endDate = new Date(startDate.getTime() + unitTimeSpan);
									units[index].push({ startTime: $this.timeline("formatTime", startDate, settings.timeFormat)
										, endTime: $this.timeline("formatTime", endDate, settings.timeFormat) });
								}
							}
						});

						$this.timeline("renderTimeline");

						$this.data("Timeline")["state"] = $.TIMELINE_STATE_NORMAL;
						$this.data("Timeline")["startIndex"] = -1;
						$this.data("Timeline")["endIndex"] = -1;
						
						$this.timeline("showTimes");

						// 触发选择事件
						if (settings.onselected && typeof settings.onselected == "function") {
							settings.onselected($this);
						}

						// 触发值改变事件
						if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
							settings.onvaluechanged($this);
						}
					}
				}

				// 显示选择开始线
				function showSelectStartLine(e) {
					if ($this.data("Timeline")["state"] == $.TIMELINE_STATE_NORMAL
							&& $this.data("Timeline")["settings"]["enable"]
							&& !$this.data("Timeline")["settings"]["readonly"]) {
						var unitDiv = $(this);
						var borderLeft = unitDiv.css("border-left");
						var units = $this.data("Timeline")["units"];
						unitDiv.data("border-left", borderLeft ? borderLeft : "0px");
//                        unitDiv.data("border-left", borderLeft ? borderLeft : unitDiv.attr("index") == "0" ? "1px solid #000000" : "0px");
						unitDiv.css("border-left", "1px dotted #000000");
						if (unitDiv.attr("index") * 1 == units.length - 1) {
							unitDiv.width($this.width() / units.length - 2);
						}
					}
				}

				// 隐藏选择开始线
				function hideSelectStartLine(e) {
					var unitDiv = $(this);
					var borderLeft = unitDiv.data("border-left");
					var units = $this.data("Timeline")["units"];
					unitDiv.css("border-left", borderLeft);
					if (unitDiv.attr("index") * 1 == units.length - 1) {
						unitDiv.width($this.width() / units.length - 1);
					}
				}
			});
		},
		/**
		* 修改时间线模式
		*/
		changeMode: function(mode) {
			var $this = $(this);
			$this.each(function() {
				$(this).data("Timeline")["settings"]["mode"] = mode;
			});
		},
		/**
		* 渲染时间线的样式
		*/
		renderTimeline: function() {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var units = $this.data("Timeline")["units"];
			$this.find("[index]").each(function() {
				var index = parseInt($(this).attr("index"));
				$(this).removeClass();
				$(this).addClass(settings[units[index].length == 0 ? "lineUnitCss" : "selectedCss"]);
			});
		},
		/**
		* 获取时间集合
		*/
		getTimes: function(mode) {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var units = $this.data("Timeline")["units"];
			var timeSeparator = $this.data("Timeline")["settings"]["timeSeparator"];
			var resultTimes = [];
			for (var i = 0; i < units.length; i++) {
				for (var j = 0; j < units[i].length; j++) {
					resultTimes.push([ $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat),
						$this.timeline("parseTime", units[i][j]["endTime"], settings.timeFormat) ]);
				}
			}

			// 最后检查并合并一下时间段
			var finalResultTimes = [];
			for (var i = 0; i < resultTimes.length; i++) {
				var finalResultTime = [];
				var finalResultStartTime = resultTimes[i][0];
				var finalResultEndTime = resultTimes[i][1];

				for (; resultTimes[i + 1] && resultTimes[i][1].getTime() == resultTimes[i + 1][0].getTime(); i++) {
					finalResultEndTime = resultTimes[i + 1][1];
				}

				if (mode == $.TIMELINE_DATA_MODE_DATE) {
					finalResultTime.push(finalResultStartTime);
					finalResultTime.push(finalResultEndTime);
				} else if (mode == $.TIMELINE_DATA_MODE_STRING) {
					// 结束时间00:00变为24:00
					var endTimeText = $this.timeline("formatTime", finalResultEndTime, settings.timeFormat);
                    finalResultTime.push($this.timeline("formatTime", finalResultStartTime, settings.timeFormat))
					if (endTimeText == "00:00") {
						endTimeText = "24:00";
                        finalResultTime.push("23:59");
					}else{
                        finalResultTime.push(endTimeText);
                    }

				} else {
					// 结束时间00:00变为24:00
					var endTimeText = $this.timeline("formatTime", finalResultEndTime, settings.timeFormat);
					if (endTimeText == "00:00") {
						endTimeText = "24:00";
					}
					finalResultTime = $this.timeline("formatTime", finalResultStartTime, settings.timeFormat)
						+ timeSeparator + endTimeText;
				}
				finalResultTimes.push(finalResultTime);
			}
			return finalResultTimes;
		},
		/**
		* 设置时间
		*/
		setTime: function(timeSpan, mode) {
			if (!timeSpan) {
				return;
			}

			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var timeSeparator = settings["timeSeparator"];
			var startTime;
			var endTime;

			if (mode == $.TIMELINE_DATA_MODE_DATE) {
				if ($.isArray(timeSpan) && timeSpan.length == 2) {
					startTime = timeSpan[0];
					endTime = timeSpan[1];
				}
			} else if (mode == $.TIMELINE_DATA_MODE_STRING) {
				if ($.isArray(timeSpan) && timeSpan.length == 2) {
					startTime = $this.timeline("parseTime", timeSpan[0], settings.timeFormat);
					endTime = $this.timeline("parseTime", timeSpan[1], settings.timeFormat);
				}
			} else {
				if (typeof timeSpan == "string") {
					var timeSpanParts = timeSpan.split(timeSeparator);
					if (timeSpanParts.length == 2) {
						startTime = $this.timeline("parseTime", timeSpanParts[0], settings.timeFormat);
						endTime = $this.timeline("parseTime", timeSpanParts[1], settings.timeFormat);
					}
				}
			}

			if (startTime && endTime && startTime instanceof Date && endTime instanceof Date) {
				if (startTime.getTime() == endTime.getTime()) {
					return;
				}
			
				var units = $this.data("Timeline")["units"];
				var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
				var tlStartTime = settings["startTime"];
				startTime.setFullYear(tlStartTime.getFullYear());
				startTime.setMonth(tlStartTime.getMonth());
				startTime.setDate(tlStartTime.getDate());
				endTime.setFullYear(tlStartTime.getFullYear());
				endTime.setMonth(tlStartTime.getMonth());
				endTime.setDate(tlStartTime.getDate());

				for (var i = 0; i < units.length; i++) {
					var currentTimeStamp = tlStartTime.getTime() + i * unitTimeSpan;
					var startMills = startTime.getTime();
					var endMills = endTime.getTime();
					if (startMills <= currentTimeStamp && endMills >= currentTimeStamp + unitTimeSpan) {// 包含
						units[i] = [ { startTime: $this.timeline("formatTime", new Date(currentTimeStamp), settings.timeFormat)
							, endTime: $this.timeline("formatTime", new Date(currentTimeStamp + unitTimeSpan), settings.timeFormat) } ];
					} else if (startMills >= currentTimeStamp && endMills <= currentTimeStamp + unitTimeSpan) {// 被包含
						if (units[i].length == 0) {// 没有任何时间段时直接添加
							units[i].push({ startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
								, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
						} else {
							var startIndex = 0;
							var endIndex = units[i].length - 1;

							// 锚定影响的时间段范围
							for (var j = 0; j < units[i].length; j++) {
								var spanStartMills = $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat).getTime();
								var spanEndMills = $this.timeline("parseTime", units[i][j]["endTime"], settings.timeFormat).getTime();
								if (startMills > spanEndMills) {
									startIndex = j + 1;
								}
								if (endMills < spanStartMills) {
									endIndex = j - 1;
									break;
								}
							}

							if (startIndex > units[i].length - 1) {// 没有交集，添加到最后
								units[i].push({ startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
									, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
							} else if (endIndex < 0) {// 没有交集，添加到最前
								units[i].unshift({ startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
									, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
							} else if (endIndex < startIndex) {// 没有交集，添加到中间
								units[i].splice(startIndex, 0, { startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
									, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
							} else {
								var newTimes = [];
								var newTime = {};
								for (var j = 0; j < units[i].length; j++) {
									var spanStartMills = $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat).getTime();
									var spanEndMills = $this.timeline("parseTime", units[i][j]["endTime"], settings.timeFormat).getTime();
									if (j < startIndex || j > endIndex) {// 影响范围外的时间段
										newTimes.push(units[i][j]);
									} else {
										if (j == startIndex) {// 影响范围第一个时间段
											if (startMills <= spanStartMills) {// 左包含
												newTime["startTime"] = $this.timeline("formatTime", startTime, settings.timeFormat);
											} else {// 左相交
												newTime["startTime"] = units[i][j]["startTime"];
											}
										}
										if (j == endIndex) {// 影响范围最后一个时间段
											if (endMills >= spanEndMills) {//  右包含
												newTime["endTime"] = $this.timeline("formatTime", endTime, settings.timeFormat);
											} else {// 右相交
												newTime["endTime"] = units[i][j]["endTime"];
											}
											newTimes.push(newTime);
										}
									}
								}
								units[i] = newTimes;
							}
						}
					} else if (startMills > currentTimeStamp && startMills < currentTimeStamp + unitTimeSpan && endMills > currentTimeStamp + unitTimeSpan) {// 左相交
						var realEndTime = new Date(currentTimeStamp + unitTimeSpan);

						if (units[i].length == 0) {// 没有任何时间段时直接添加
							units[i].push({ startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
								, endTime: $this.timeline("formatTime", realEndTime, settings.timeFormat) });
						} else {
							var startIndex = 0;

							// 锚定影响的时间段范围
							for (var j = 0; j < units[i].length; j++) {
								var spanEndMills = $this.timeline("parseTime", units[i][j]["endTime"], settings.timeFormat).getTime();
								if (startMills > spanEndMills) {
									startIndex = j + 1;
								}
							}

							if (startIndex > units[i].length - 1) {// 没有交集，添加到最后
								units[i].push({ startTime: $this.timeline("formatTime", startTime, settings.timeFormat)
									, endTime: $this.timeline("formatTime", realEndTime, settings.timeFormat) });
							} else {
								var newTimes = [];
								var newTime = {};
								for (var j = 0; j < units[i].length; j++) {
									var spanStartMills = $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat).getTime();
									if (j < startIndex) {// 影响范围外的时间段
										newTimes.push(units[i][j]);
									} else {
										if (startMills <= spanStartMills) {// 左包含
											newTime["startTime"] = $this.timeline("formatTime", startTime, settings.timeFormat);
											
										} else {// 左相交
											newTime["startTime"] = units[i][j]["startTime"];
										}
										newTime["endTime"] = $this.timeline("formatTime", realEndTime, settings.timeFormat);
										newTimes.push(newTime);
										break;
									}
								}
								units[i] = newTimes;
							}
						}
					} else if (startMills < currentTimeStamp && endMills > currentTimeStamp && endMills < currentTimeStamp + unitTimeSpan) {// 右相交
						var realStartTime = new Date(currentTimeStamp);

						if (units[i].length == 0) {// 没有任何时间段时直接添加
							units[i].push({ startTime: $this.timeline("formatTime", realStartTime, settings.timeFormat)
								, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
						} else {
							var endIndex = units[i].length - 1;

							// 锚定影响的时间段范围
							for (var j = 0; j < units[i].length; j++) {
								var spanStartMills = $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat).getTime();
								if (endMills < spanStartMills) {
									endIndex = j - 1;
									break;
								}
							}

							if (endIndex < 0) {// 没有交集，添加到最前
								units[i].unshift({ startTime: $this.timeline("formatTime", realStartTime, settings.timeFormat)
									, endTime: $this.timeline("formatTime", endTime, settings.timeFormat) });
							} else {
								var newTimes = [];
								var newTime = {};
								for (var j = 0; j < units[i].length; j++) {
									var spanStartMills = $this.timeline("parseTime", units[i][j]["startTime"], settings.timeFormat).getTime();
									var spanEndMills = $this.timeline("parseTime", units[i][j]["endTime"], settings.timeFormat).getTime();
									if (j > endIndex) {// 影响范围外的时间段
										newTimes.push(units[i][j]);
									} else if (j == endIndex) {// 影响范围最后一个时间段
										newTime["startTime"] = $this.timeline("formatTime", realStartTime, settings.timeFormat);
										if (endMills >= spanEndMills) {//  右包含
											newTime["endTime"] = $this.timeline("formatTime", endTime, settings.timeFormat);
										} else {// 右相交
											newTime["endTime"] = units[i][j]["endTime"];
										}
										newTimes.push(newTime);
									}
								}
								units[i] = newTimes;
							}
						}
					}
				}

				$this.timeline("renderTimeline");
				$this.timeline("showTimes");

				// 触发值改变事件
				if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
					settings.onvaluechanged($this);
				}
			}
		},
		/**
		* 移除时间
		*/
		removeTime: function(timeSpan, mode) {
			if (!timeSpan) {
				return;
			}

			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var times = $this.timeline("getTimes", $.TIMELINE_DATA_MODE_DATE);
			var newTimes = [];
			var timeSeparator = settings["timeSeparator"];
			var startTime;
			var endTime;

			if (mode == $.TIMELINE_DATA_MODE_DATE) {
				if ($.isArray(timeSpan) && timeSpan.length == 2) {
					startTime = timeSpan[0];
					endTime = timeSpan[1];
				}
			} else if (mode == $.TIMELINE_DATA_MODE_STRING) {
				if ($.isArray(timeSpan) && timeSpan.length == 2) {
					startTime = $this.timeline("parseTime", timeSpan[0], settings.timeFormat);
					endTime = $this.timeline("parseTime", timeSpan[1], settings.timeFormat);
				}
			} else {
				if (typeof timeSpan == "string") {
					var timeSpanParts = timeSpan.split(timeSeparator);
					if (timeSpanParts.length == 2) {
						startTime = $this.timeline("parseTime", timeSpanParts[0], settings.timeFormat);
						endTime = $this.timeline("parseTime", timeSpanParts[1], settings.timeFormat);
					}
				}
			}

			if (startTime && endTime && startTime instanceof Date && endTime instanceof Date) {
				var startMills = startTime.getTime();
				var endMills = endTime.getTime();
				for (var i = 0; i < times.length; i++) {
					var curStartMills = times[i][0].getTime();
					var curEndMills = times[i][1].getTime();
					if (endMills <= curStartMills || startMills >= curEndMills) { // 没有交集
						newTimes.push([ times[i][0], times[i][1] ]);
					} else if (startMills <= curStartMills && endMills >= curStartMills && endMills <= curEndMills) { // 前交集
						newTimes.push([ endTime, times[i][1] ]);
					} else if (startMills >= curStartMills && startMills <= curEndMills && endMills >= curEndMills) { // 后交集
						newTimes.push([ times[i][0], startTime ]);
					} else if (startMills <= curStartMills && endMills >= curEndMills) { // 包含
						continue;
					} else if (startMills > curStartMills && endMills < curEndMills) { // 被包含
						newTimes.push([ times[i][0], startTime ]);
						newTimes.push([ endTime, times[i][1] ]);
					}
				}
			}

			$this.timeline("clearTimes");

			for (var i = 0; i < newTimes.length; i++) {
				$this.timeline("setTime", newTimes[i], $.TIMELINE_DATA_MODE_DATE);
			}

			$this.timeline("renderTimeline");
			$this.timeline("showTimes");

			// 触发值改变事件
			if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
				settings.onvaluechanged($this);
			}
		},
		/**
		* 全选时间
		*/
		selectAll: function() {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var startMills = settings.startTime.getTime();
			var units = $this.data("Timeline")["units"];
			var unitTimeSpan = $this.data("Timeline")["unitTimeSpan"];
			for (var i = 0; i < units.length; i++) {
				var endTimeText = $this.timeline("formatTime", new Date(startMills + (i + 1) * unitTimeSpan), settings.timeFormat);
				units[i] = [ { startTime: $this.timeline("formatTime", new Date(startMills + i * unitTimeSpan), settings.timeFormat)
					, endTime: endTimeText == "00:00" ? "23:59" : endTimeText } ];
			}
			$this.timeline("renderTimeline");
			$this.timeline("showTimes");

			// 触发值改变事件
			if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
				settings.onvaluechanged($this);
			}
		},
		/**
		* 反选时间
		*/
		inverse: function() {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var times = $this.timeline("getTimes", $.TIMELINE_DATA_MODE_DATE);
			var inverseTimes = [];
			var startTime = new Date();
			var endTime = new Date();
			startTime.setHours(0, 0, 0, 0);
			endTime.setHours(23, 59, 0, 0);

			var inverseStartTime = null;
			var inverseEndTime = null;

			if (times.length == 0) {
				inverseTimes.push([ startTime, endTime ]);
			}

			if (times.length == 1) {
				if (startTime.getTime() != times[0][0].getTime()) {
					inverseTimes.push([ startTime, times[0][0] ]);
				}

				if (endTime.getTime() != times[0][1].getTime() && (times[0][1].getHours() != 0 || times[0][1].getMinutes() != 0 || times[0][1].getSeconds() != 0)) {
					inverseTimes.push([ times[0][1], endTime ]);
				}
			}

			if (times.length > 1) {
				for (var i = 0; i < times.length; i++) {
					if (i == 0) {
						if (startTime.getTime() != times[i][0].getTime()) {
							inverseTimes.push([ startTime, new Date(times[i][0].getTime()) ]);
						}

						inverseStartTime = new Date(times[i][1].getTime());
					} else if (i == times.length - 1) {
						inverseEndTime = new Date(times[i][0].getTime());
						inverseTimes.push([ inverseStartTime, inverseEndTime ]);

						if (endTime.getTime() != times[i][1].getTime() && (times[i][1].getHours() != 0 || times[i][1].getMinutes() != 0 || times[i][1].getSeconds() != 0)) {
							inverseTimes.push([ new Date(times[i][1].getTime()), endTime ]);
						}
					} else {
						inverseEndTime = new Date(times[i][0].getTime());
						inverseTimes.push([ inverseStartTime, inverseEndTime ]);
						inverseStartTime = new Date(times[i][1].getTime());
					}
				}
			}

			$this.timeline("clearTimes");

			for (var i = 0; i < inverseTimes.length; i++) {
				$this.timeline("setTime", inverseTimes[i], $.TIMELINE_DATA_MODE_DATE);
			}

			$this.timeline("renderTimeline");
			$this.timeline("showTimes");

			// 触发值改变事件
			if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
				settings.onvaluechanged($this);
			}
		},
		/**
		* 清除时间
		*/
		clearTimes: function() {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var units = $this.data("Timeline")["units"];
			for (var i = 0; i < units.length; i++) {
				units[i] = [];
			}
			$this.timeline("renderTimeline");
			$this.timeline("showTimes");

			// 触发值改变事件
			if (settings.onvaluechanged && typeof settings.onvaluechanged == "function") {
				settings.onvaluechanged($this);
			}
		},
		/**
		* 显示时间集合
		*/
		showTimes: function() {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			if (settings.showTimes) {
				var times = $this.timeline("getTimes");
				var timesToShow = [];
				for (var i = 0, len = times.length; i < len; i++) {
					if (i < settings.maxTimeShow || settings.maxTimeShow <= 0) {
						// 23:59显示为24:00
						timesToShow.push(times[i].replace("23:59", "24:00"));
					}
				}
				var showTimeControl = $("#" + $this.attr("id") + "_time_line_info", $this);
				showTimeControl.html(timesToShow.join(",&nbsp;&nbsp;&nbsp;") + (timesToShow.length < settings.maxTimeShow || settings.maxTimeShow <= 0 ? "" : "..."));
				showTimeControl.attr("title", times.join("\n"));
			}
		},
		/**
		* 显示手动输入控件
		*/
		showManualControl: function(left, top) {
			function setManuanlTime() {
				var startTime = $this.timeline("parseTime", startInput.val(), settings.timeFormat);
				var endTime = $this.timeline("parseTime", endInput.val(), settings.timeFormat);
				if (startTime >= endTime) {
					if (settings.oninvalidate) {
						settings.oninvalidate(startTime, endTime, startInput, endInput);
					}
					return;
				}
				$this.timeline(settings["mode"] == $.TIMELINE_MODE_ADD ? "setTime" : "removeTime", startInput.val() + settings.timeSeparator + endInput.val());
				$("." + settings.manualControlCss, settings.context).remove();
			}
		
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			var manualButton = $this.find("#" + $this.attr("id") + "_time_line_manual");
			var manualControl = $("<div id=\"" + $this.attr("id") + "_time_line_manual_control\" class=\"" + settings.manualControlCss + "\" style=\"position: absolute;\"></div>");
			var startInput = $("<input type=\"text\" class=\"" + settings.manualInputCss + "\"/>");
			var endInput = $("<input type=\"text\" class=\"" + settings.manualInputCss + "\"/>");
			var okButton = $("<input type=\"button\" class=\"" + settings.manualButtonCss + "\" value=\"" + settings.okButtonText + "\"/>");

			$("." + settings.manualControlCss, settings.context).remove();
			
			var inputWidth = settings.timeFormat.length * 8;
			startInput.css("width", inputWidth);
			endInput.css("width", inputWidth);
			
			startInput.keypress(function(event) {
				if (event.which == 13) {
					setManuanlTime();
				}
			});

			endInput.keypress(function(event) {
				if (event.which == 13) {
					setManuanlTime();
				}
			});
			
			if (startInput.timeInput) {
				startInput.timeInput({
					format: settings.timeFormat,
					filterTimeSpans: settings.manualStartFilters
				});
			}
			if (endInput.timeInput) {
				endInput.timeInput({
					format: settings.timeFormat,
					filterTimeSpans: settings.manualEndFilters
				});
			}

			var startTime = settings.manualStartTime;
			var endTime = settings.manualEndTime;
			if (!startTime) {
				startTime = new Date();
				startTime.setHours(startTime.getHours() - 1);
			}
			if (!endTime) {
				endTime = new Date();
			}
			startInput.val($this.timeline("formatTime", startTime, settings.timeFormat));
			endInput.val($this.timeline("formatTime", endTime, settings.timeFormat));

			manualControl.append(startInput);
			manualControl.append(settings.timeSeparator);
			manualControl.append(endInput);
			manualControl.append(okButton);
			manualButton.parent().append(manualControl);
			
			startInput.focus();

			if (isNaN(left) || isNaN(top)) {
				var position = manualButton.position();
				left = position.left - (manualControl.width() - manualButton.width()) / 2;
				top = position.top + manualButton.outerHeight();
			}
			manualControl.css("left", left);
			manualControl.css("top", top);

			okButton.click(function(event) {
				setManuanlTime();
			});

			manualControl.mousedown(function(event) {
				event.stopPropagation();
			});

			$(settings.context).mousedown(function(event) {
				$("." + settings.manualControlCss, settings.context).remove();
				$(this).unbind("mousedown");
			});
		},
		/**
		* 格式化时间显示文本
		*/
		formatTime: function(date, format) {
			if (date instanceof Date) {
				var formatResult = format;
				var hours = date.getHours();
				var minutes = date.getMinutes();
				var seconds = date.getSeconds();
				var formatFrags = [ "hh", "mm", "ss", "h", "m", "s" ];
				var timeFrags = [ hours, minutes, seconds, hours, minutes, seconds ];
				for (var i = 0; i < formatFrags.length; i++) {
					formatResult = formatResult.replace(formatFrags[i], formatFrags[i].length > 1 && timeFrags[i] < 10 ? "0" + timeFrags[i] : timeFrags[i]);
				}
			}
			return formatResult;
		},
		/**
		* 解析时间显示文本
		*/
		parseTime: function(text, format) {
			if (typeof text == "string") {
				var partFrags = [ false, false, false ];
				var parseFrags = [ "hh", "mm", "ss", "h", "m", "s" ];
				var regFrags = [ "(2[0-3]|[01]?\\d)", "([0-5]?\\d)", "([0-5]?\\d)", "(2[0-3]|[01]?\\d)", "([0-5]?\\d)", "([0-5]?\\d)" ];
				for (var i = 0; i < parseFrags.length; i++) {
					var partIndex = format.indexOf(parseFrags[i]);
					if (partIndex != -1) {
						partFrags[i % 3] = true;
						format = format.replace(parseFrags[i], regFrags[i]);
					}
				}
				var reg = new RegExp("^" + format + "$");
				var timeParts = text.match(reg);
				var parseResult = new Date();
				if (timeParts) {
					parseResult.setHours(partFrags[0] ? timeParts[1] : 0
						, partFrags[1] ? timeParts[partFrags[0] ? 2 : 1] : 0
						, partFrags[2] ? timeParts[partFrags[0] && partFrags[1] ? 3 : !partFrags[0] && !partFrags[1] ? 1 : 2] : 0, 0);
					return parseResult;
				}
			}
			return null;
		},
		/**
		* 设置是否可用
		*/
		setEnable: function(enabled) {
			var $this = $(this);
			var settings = $this.data("Timeline")["settings"];
			settings.enable = enabled;
			$this.find("#" + $this.attr("id") + "_time_line").css("cursor", enabled ? "text" : "not-allowed");
			if (settings.showManualButton) {
				var manualButton = $("#" + $this.attr("id") + "_time_line_manual", settings.context);
				manualButton.removeClass();
				manualButton.addClass(enabled ? settings.manualCss : settings.manualDisabledCss);
				manualButton.attr("disabled", !settings.enable);
			}

			if (settings.showSelectAllButton) {
				var selectAllButton = $("#" + $this.attr("id") + "_time_line_selectAll", settings.context);
				selectAllButton.removeClass();
				selectAllButton.addClass(enabled ? settings.selectAllCss : settings.selectAllDisabledCss);
				selectAllButton.attr("disabled", !settings.enable);
			}

			if (settings.showInverseButton) {
				var inverseButton = $("#" + $this.attr("id") + "_time_line_inverse", settings.context);
				inverseButton.removeClass();
				inverseButton.addClass(enabled ? settings.inverseCss : settings.inverseDisabledCss);
				inverseButton.attr("disabled", !settings.enable);
			}

			if (settings.showClearButton) {
				var clearButton = $("#" + $this.attr("id") + "_time_line_clear", settings.context);
				clearButton.removeClass();
				clearButton.addClass(enabled ? settings.clearCss : settings.clearDisabledCss);
				clearButton.attr("disabled", !settings.enable);
			}
		}
	};

	$.fn.timeline = function(method) {
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} else if (typeof method === "object" || !method) {
			return methods.init.apply(this, arguments);
		} else {
			$.error("Method \"" + method + "()\" does not exist on jQuery.timeline");
		}
	};
})(jQuery);